//
// ********************************************************************
// * License and Disclaimer                                           *
// *                                                                  *
// * The  MODAR software  is  copyright of Commissariat a l'Energie   * 
// * Atomique (CEA).						      *
// * Neither the authors of this software system, nor their employing *
// * institutes,nor the agencies providing financial support for this *
// * work  make  any representation or  warranty, express or implied, *
// * regarding  this  software system or assume any liability for its *
// * use.							      *
// * By using,  copying,  modifying or distributing the software (or  *
// * any work based  on the software)  you  agree  to acknowledge its *
// * use in resulting scientific publications.		              *
// ********************************************************************
//
// File:        MODAR.C
// Version:     1.18
// Created:     2010-02-09
// Author:      Cedric Carasco
// mail:        cedric.carasco@cea.fr
// Updated:     2010-07-27 by Cedric Carasco
// mail:        xxx@xxx.xxx
//
////////////////////////////////////////////////////////////////////////
// .x MODAR.C
// .x MODAR.C++

#define MODAR_VERSION 1.18

#include <stdlib.h>

#include <TROOT.h>
#include <TApplication.h>
#include <TVirtualX.h>
#include <TVirtualPadEditor.h>
#include <TGResourcePool.h>
#include <TGListBox.h>
#include <TGListTree.h>
#include <TGFSContainer.h>
#include <TGClient.h>
#include <TGFrame.h>
#include <TGIcon.h>
#include <TGLabel.h>
#include <TGButton.h>
#include <TGTextEntry.h>
#include <TGNumberEntry.h>
#include <TGMsgBox.h>
#include <TGMenu.h>
#include <TGCanvas.h>
#include <TGComboBox.h>
#include <TGTab.h>
#include <TGSlider.h>
#include <TGDoubleSlider.h>
#include <TGFileDialog.h>
#include <TGTextEdit.h>
#include <TGShutter.h>
#include <TGProgressBar.h>
#include <TGColorSelect.h>
#include <RQ_OBJECT.h>
#include <TObject.h>
#include <TRootEmbeddedCanvas.h>
#include <TCanvas.h>
#include <TColor.h>
#include <TH1.h>
#include <TH2.h>
#include <TRandom.h>
#include <TSystem.h>
#include <TSystemDirectory.h>
#include <TEnv.h>
#include <TFile.h>
#include <TKey.h>
#include <TGDockableFrame.h>
#include <TGFontDialog.h>
#include <TGStatusBar.h>
#include <TBox.h>
#include <TF1.h>
#include<TGButtonGroup.h>
#include<TGLayout.h>
#include<TPaveText.h>
#include<TStyle.h>
#include<TString.h>
#include<TGMimeTypes.h>
#include<TLegend.h>






/*************************************************************************/
/*************************************************************************/
/*********	MTArrayD	**********************************/
class MTArrayD: public TArrayD
{

RQ_OBJECT("MTArrayD")
private:
char fType;
Int_t fID;

public:
~MTArrayD(){};
Int_t GetID()const{return fID;};
void SetID(Int_t id){fID=id;};
char GetType()const{return fType;};
Bool_t HaveSameBinning(MTArrayD *Def1)
	{
	if(GetSize()==Def1->GetSize())
		{
		Int_t n;
		for(n=0;n<GetSize();n++)
		if(fabs(GetArray()[n]-Def1->GetArray()[n])>0.0000000000001*fabs(GetArray()[n]))
			return 1==0;
		return 1==1;
		}
	else
		return 1==0;
	};
void Multiply(Double_t a)
	{
	Int_t n;
	for(n=0;n<GetSize();n++)
	GetArray()[n]*=a;
	};
	
void Print(char *info)
	{
	if(this!=NULL)
		{
		if(info!=NULL) printf("%s\n",info);
		for(Int_t i=0;i<GetSize()-1;i++) printf("bin%d=[%lf;%lf]\n",i,GetArray()[i],GetArray()[i+1]);
		}
	};

MTArrayD(const MTArrayD &Hdef): TArrayD(Hdef)
	{
	fID=Hdef.GetID();
	fType=Hdef.GetType();
	};

MTArrayD & operator =(const MTArrayD &Hdef)
	{
	if(this!=&Hdef)
		{
		fID=Hdef.GetID();
		fType=Hdef.GetType();
		}
	return *this;
	};


MTArrayD(char *line)
	{
	char *w0;
	Int_t bin,OldSize=0,lw0,Nintervals=0,n=-1,THERE_HAS_BEEN_A_i=0,c_w0,c_line,c;
	Double_t lower_limit,upper_limit,dx=0,limite;

	w0=(char*)calloc(400,sizeof(char));

// The input file (".i" extension) that served to produce the MCNP output file is recalled at the beginning 
// of the MCNP output file (".o" extension). In the MCNP output file, the first word of each line corresponding
// to the input file is a line number indicator, following by a '-' character at position 5 (position 0 
// corresponding to the first character of the line). The input line itself starts at position 13. 
// Therefore, the second word of the ouput file line is the first word of the corresponding input file line. 
// For example, in a "Example.i" file, the command 
//                           "e120 0.6 739i 8 3i 10"
// is written at line 61.
// After running MCNP with "Example.i" as input, the output file "Example.o", will 
// recall the 61st line in the form 
//                           "   61-       e120 0.6 739i 8 3i 10"
// So, the second word of the line correspond to the first word read in the input file, here "e120".
	for(c_line=13,c_w0=0;c_line<(Int_t)strlen(line);c_line++)
	if(line[c_line]!=' ') w0[c_w0++]=line[c_line];
	else break;
	w0[c_w0]='\0';
// The type of the cut is given by the first character : 't' or 'T' means time cut and 'e' or 'E' means energy cut
	fType=w0[0];
// The identification of the cut follows the first character
	fID=atoi(w0+1);
	Set(0);
	do
		{
		n++;
		//get next word in the line
		// a word is something which doesn't start with space, tab or "13".
		for(c=c_line;c<(Int_t)strlen(line);c++)if(line[c]!=' '&&line[c]!='\t'&&line[c]!=13)break;
		for(c_line=c,c_w0=0;c_line<(Int_t)strlen(line);c_line++)
		if(line[c_line]!=' '&&line[c]!='\t'&&line[c]!=13)
			w0[c_w0++]=line[c_line];
		else
			break;
		w0[c_w0]='\0';
		// if no more word, or if comments heve been reached (char '$'), break
		if(c_w0==0||w0[0]=='$')
			break;

		lw0=(Int_t)strlen(w0);
// The  binning of time or energy axis is described by series of values that indicate bins upper limites.
// For example "e0 1 2 3 4" will be uderstood by MCNP as an energy cut, the 1st bin being bewteen
// 0 and 1, the 2nd bin being between 1 and 2, the 3rd between 3 and 4 and the 3rd between 4 and 5.
// The first bin between 0 and 1 handles overflows. The meaningfull bins start after.
// It is also possible to indicates MCNP that the energy (or time) axis  will be divided into N+1 intervals
// between X and Y. This is done using the syntax "e0 X Ni Y". In this case, the lengh of each interval between
// X and Y is equal to (Y-X)/(N+1).
// It is possible to combine the syntaxes, for example "e0 0 4i 5 5.5 6" will lead to 
// a cut of the energy into 7 bins:
// 					bin0=[0.000000;1.000000]
// 					bin1=[1.000000;2.000000]
// 					bin2=[2.000000;3.000000]
// 					bin3=[3.000000;4.000000]
// 					bin4=[4.000000;5.000000]
// 					bin5=[5.000000;5.500000]
// 					bin6=[5.500000;6.000000]
		if(w0[lw0-1]!='i')
			{
			if(THERE_HAS_BEEN_A_i==0)
				{
				limite=atof(w0);
				OldSize=GetSize();
				Set(OldSize+1);
				GetArray()[OldSize]=limite;
				//Set(GetSize()+1);
				}
			else	//if THERE_HAS_BEEN_A_i==1
				{
				upper_limit=atof(w0);
				lower_limit=GetArray()[GetSize()-1];
				if(Nintervals)
					{
					dx=(upper_limit-lower_limit)/(Nintervals+1);
					OldSize=GetSize();
					Set(OldSize+Nintervals+1);
					}
				for(bin=OldSize;bin<OldSize+Nintervals+1;bin++)
					{
					GetArray()[bin]=lower_limit+(bin-(OldSize-1))*dx;
					}
				THERE_HAS_BEEN_A_i=0;
				Nintervals=0;
				}
			}
		else		//if w0[lw0-1]=='i'
			{
			THERE_HAS_BEEN_A_i=1;
			w0[lw0-1]='\0';
			Nintervals=atoi(w0);
			}
		}
	while(1);		
	};
// end of MTArrayD(char *line)

MTArrayD(Int_t n, Int_t id=-1):TArrayD(n)
{
fID=id;
fType='x';
if(n>1&&fID==-1)
	{
	GetArray()[0]=0;
	GetArray()[1]=1;
	}
};


MTArrayD(Int_t n, Double_t *array):TArrayD(n,array)
{
fID=0;
fType='x';
};


MTArrayD():TArrayD(1)
{
fID=0;
fType='x';
};

MTArrayD (Int_t n, const MTArrayD &Hdef):TArrayD (n)
{
fID=Hdef.GetID();
fType=Hdef.GetType();
};

// Construction of an array from the bin limits of a 1D histogram
MTArrayD (const TH1D &H, Int_t id=0, char type='x')
{
fID=id;
fType=type;
Set(H.GetNbinsX()+1);
Int_t bin;
for(bin=1;bin<=H.GetNbinsX();bin++)
	GetArray()[bin-1]=H.GetBinLowEdge(bin);
GetArray()[H.GetNbinsX()]=H.GetBinLowEdge(H.GetNbinsX())+H.GetBinWidth(H.GetNbinsX());
};

ClassDef(MTArrayD,1)


};
// end of MTArrayD class

ClassImp(MTArrayD)




/*************************************************************************/
/*************************************************************************/
/*********	MTH2D	**********************************/

class MTH2D_Controller;

class MTH2D: public TH2D
{
RQ_OBJECT("MTH2D")
private:
// fBoX serves to define cuts along the Xaxis.
// The coordinates of the lower left (x=fX0BoxX,y=fY0BoxX) and upper right (x=fX1BoxX,y=fY1BoxX) corners
// of the box are saved in Double_t variables.
// The use of the Double_t variables allows to recover the box after calls to ROOT function that interfers 
// with the box (for example, it has been observed that setting Y axis to logartihmic sometimes makes the box 
// loosing its coordiantes. Keeping the coordinated in Double_t variables allow to recover them at any time).
// Since the bins along the X axis defines boundaries, the box cannot be placed at any position.
// The edges of the box must coincide with one of the bins edges (it is meaning less to define a cut starting
// or finishing in the middle of a bin). Therefore, when moving the box with the mouse, it will always be replaced
// fb0BoxX is the number of the bin having its low edge the nearer to the lower edge of the box.
// fb1BoxX is the number of the bin having its upper edge the nearer to the upper edge of the box.
// fb0Box and fb1BoxX start from 1.
// fb0BoxX and fb1BoxX will be used for defing histograms projections (since the projections methods in ROOT works
// with bins number).
TBox *fBoxX;Double_t fX0BoxX;Double_t fX1BoxX;Double_t fY0BoxX;Double_t fY1BoxX;Int_t fb0BoxX;Int_t fb1BoxX;
// Same for the Y axis
TBox *fBoxY;Double_t fY0BoxY;Double_t fY1BoxY;Double_t fX0BoxY;Double_t fX1BoxY;Int_t fb0BoxY;Int_t fb1BoxY;
// string describing units used for the X axis
// Type of particle: 1-> neutron, 2 ->gamma
Int_t fparticle;
// Tally identification number
Int_t fID;
// If Graphical User Interface is used.
// fF is a MTH2D_Controller object from which the detector parameters or the "Save as" options can be defined.
MTH2D_Controller *fF;
// Variable used to know if the counts (Z axis values) have been normalzed to X and Y bins width
// fScale!=1 -> counts are normalized, else not.
Int_t fScale;
// Integer indicating the current time unit (can be chosen between shake, ns, s, ms and s)
Int_t fTimeUnit;
TGCheckButton*fPlot_Errors;
TGCheckButton*fShow_FirstEscape;
TGCheckButton*fShow_SecondEscape;




// The random generator seed is different each time Inject(Int_t Nevt) is called.
// So, it is possible to call Inject(Int_t Nevt) several times with the same number of
// injected events, leading to different histograms.
// Not to erase the previously created histograms, the histograms name will contain fNInject.
// fNInject will be incremented each time Inject() is called.
Int_t fNInject;

TCanvas *fC;
TGMainFrame *fMain;
TRootEmbeddedCanvas *fEcanvas;
// Entries allowing to set the number of events to be injected by Monte Carlo following
// a probability density function built from the 2D histogram.
TGNumberEntry *fInjEvtsEntries;
// Entries allowing to set by hands position of the Tboxes
TGNumberEntry *fETCuts_Entries_0;
TGNumberEntry *fETCuts_Entries_1;
TGNumberEntry *fETCuts_Entries_2;
TGNumberEntry *fETCuts_Entries_3;
TGStatusBar *fStatusBar;
//_P
TGListBox *fList_Elements;
//_P





public:
~MTH2D()
	{
	// remove from ROOT memory the related 1D and 2D histograms
	char *w0;w0=new char[400];
	sprintf(w0,"%s_pX",GetName());  gROOT->Delete(w0);
	sprintf(w0,"%s_px",GetName());  gROOT->Delete(w0);
	sprintf(w0,"%s_pX0",GetName()); gROOT->Delete(w0);
	sprintf(w0,"%s_pY",GetName());  gROOT->Delete(w0);
	sprintf(w0,"%s_py",GetName());  gROOT->Delete(w0);
	sprintf(w0,"%s_pY0",GetName()); gROOT->Delete(w0);
	sprintf(w0,"%s",GetName());     gROOT->Delete(w0);

	fBoxX->Delete();
	fBoxY->Delete();

	// Clean up used widgets: frames, buttons, layouthints
	if(fMain!=NULL)
		{
		fMain->Cleanup();
		fMain->DeleteWindow();
		}
	};
Int_t GetParticle()const{return fparticle;};
Int_t GetID()const{return fID;};
MTH2D_Controller* GetMTH2D_Controller()const{return fF;};
Int_t GetScale()const{return fScale;};
Int_t GetTimeUnit()const{return fTimeUnit;};
Int_t GetNInject()const{return fNInject;};
void SetID(Int_t id){fID=id;};
void SetMTH2D_Controller (MTH2D_Controller *F){fF=F;};
void SetScale(Int_t s){fScale=s;};
void SetTimeUnit(Int_t u){fTimeUnit=u;};
void SetParticle(Int_t p){fparticle=p;};
TBox *GetBoxX()const{return fBoxX;};
TBox *GetBoxY()const{return fBoxY;};
Int_t Getb0BoxX()const{return fb0BoxX;};
Int_t Getb1BoxX()const{return fb1BoxX;};
Int_t Getb0BoxY()const{return fb0BoxY;};
Int_t Getb1BoxY()const{return fb1BoxY;};
TCanvas *GetC()const{return fC;};
void SetHistogram(const TH2D & H)
	{
	Int_t bx,by;
	for(bx=1;bx<=H.GetNbinsX();bx++)
	for(by=1;by<=H.GetNbinsY();by++)
		{
		SetBinContent(bx,by,H.GetBinContent(bx,by));
		SetBinError(bx,by,H.GetBinError(bx,by));
		}
	};
void DrawBoxX()
	{
	if(fBoxX==NULL)
		fBoxX=new TBox(fX0BoxX,fY0BoxX,fX1BoxX,fY1BoxX);
	fBoxX->Draw();
	};
void DrawBoxY()
	{
	if(fBoxY==NULL)
		fBoxY=new TBox(fX0BoxY,fY0BoxY,fX1BoxY,fY1BoxY);
	fBoxY->Draw();
	};

// Save all data concerning the MTH2D into the file with name "file_name"
void SaveAsMTH2DFile(char *file_name)
	{
	FILE *file;
	Int_t bin_x,bin_y;
	file=fopen(file_name,"wb");
	if(file!=NULL)
		{
		fprintf(file,"%s\n",GetName());
		// to avoid empty string, check that axis have name, else put Arbitrary Units.
		if(strcmp(GetXaxis()->GetTitle(),""))
			fprintf(file,"%s\n",GetXaxis()->GetTitle());
		else
			fprintf(file,"A.U.\n");
		if(strcmp(GetYaxis()->GetTitle(),""))
			fprintf(file,"%s\n",GetYaxis()->GetTitle());
		else
			fprintf(file,"A.U.\n");
		if(strcmp(GetZaxis()->GetTitle(),""))
			fprintf(file,"%s\n",GetZaxis()->GetTitle());
		else
			fprintf(file,"A.U.\n");
		fprintf(file,"%d\n",fparticle);
		fprintf(file,"%d\n",fID);
		fprintf(file,"%d\n",fScale);
		fprintf(file,"%d\n",fTimeUnit);		

		fprintf(file,"%d\n",GetXaxis()->GetNbins()+1);
		for(bin_x=1;bin_x<=GetXaxis()->GetNbins();bin_x++)
			fprintf(file,"%E ",GetXaxis()->GetBinLowEdge(bin_x));
		fprintf(file,"%E ",GetXaxis()->GetBinLowEdge(bin_x));
		fprintf(file,"\n%d\n",GetYaxis()->GetNbins()+1);
		for(bin_y=1;bin_y<=GetYaxis()->GetNbins();bin_y++)
			fprintf(file,"%E ",GetYaxis()->GetBinLowEdge(bin_y));
		fprintf(file,"%E ",GetYaxis()->GetBinLowEdge(bin_y));
		fprintf(file,"\n");
		
		for(bin_x=0;bin_x<=GetXaxis()->GetNbins();bin_x++,fprintf(file,"\n"))
		for(bin_y=0;bin_y<=GetYaxis()->GetNbins();bin_y++)
			fprintf(file,"%E %E ",GetBinContent(bin_x,bin_y),GetBinError(bin_x,bin_y));
		fclose(file);
		printf("%s created\n",file_name);
		}
	else
		printf("cannot create %s\n",file_name);
	};



MTH2D(const MTH2D &H):TH2D(H)
	{
	fparticle=H.GetParticle();
	fID=H.GetID();
	fTimeUnit=H.GetTimeUnit();
	fNInject=H.GetNInject();

	fX0BoxX=H.GetBoxX()->GetX1();
	fX1BoxX=H.GetBoxX()->GetX2();
	fY0BoxX=H.GetBoxX()->GetY1();
	fY1BoxX=H.GetBoxX()->GetY2();
	fX0BoxY=H.GetBoxY()->GetX1();
	fX1BoxY=H.GetBoxY()->GetX2();
	fY0BoxY=H.GetBoxY()->GetY1();
	fY1BoxY=H.GetBoxY()->GetY2();
	fBoxX=new TBox(fX0BoxX,fY0BoxX,fX1BoxX,fY1BoxX);
	fBoxY=new TBox(fX0BoxY,fY0BoxY,fX1BoxY,fY1BoxY);
	fb0BoxX=H.Getb0BoxX();
	fb1BoxX=H.Getb1BoxX();
	fb0BoxY=H.Getb0BoxY();
	fb1BoxY=H.Getb1BoxY();
	
	fF=H.GetMTH2D_Controller();
	fScale=H.GetScale();
	

	fC=NULL;
	fMain=NULL;
	};
// end of MTH2D(const MTH2D &H):TH2D(H)

// Build the MTH2D from a .MTH2D file
MTH2D(char *file_name,MTH2D_Controller *F):TH2D()
	{
	fF=F;
	
	FILE *file;
	Int_t bin_x,bin_y,SPACE;
	char *w0;w0=new char[4000];
	Int_t Nx,Ny,NumberOfLinesBeforeEdef=0;
	unsigned int c;
	Double_t *Ax,*Ay,counts,ecounts;
	char *fXaxisUnits=new char[400];
	char *fYaxisUnits=new char[400];
	char *fCountsUnit=new char[400];
	file=fopen(file_name,"r");
	fgets(w0,400*sizeof(char),file);
	fgets(w0,400*sizeof(char),file);
	fgets(w0,400*sizeof(char),file);
	fgets(w0,400*sizeof(char),file);
	NumberOfLinesBeforeEdef=0;
	SPACE=0;
	do
		{
		fgets(w0,400*sizeof(char),file);
		if(feof(file))
			break;
		for(c=0;c<strlen(w0);c++)
		if(w0[c]==' '||w0[c]=='\t')
			{
			SPACE=1;
			break;
			}
		if(SPACE)
			break;
		NumberOfLinesBeforeEdef++;
		}
	while(1);
	fclose(file);
	file=fopen(file_name,"r");
	if(file!=NULL)
		{
		fgets(w0,400*sizeof(char),file);
		// Using fgets, the last char of w0 is '\n': to be removed
		w0[strlen(w0)-1]='\0';SetName(w0);
		fgets(fXaxisUnits,400*sizeof(char),file);fXaxisUnits[strlen(fXaxisUnits)-1]='\0';
		GetXaxis()->SetTitle(fXaxisUnits);
		fgets(fYaxisUnits,400*sizeof(char),file);fYaxisUnits[strlen(fYaxisUnits)-1]='\0';
		GetYaxis()->SetTitle(fYaxisUnits);
		fgets(fCountsUnit,400*sizeof(char),file);fCountsUnit[strlen(fCountsUnit)-1]='\0';
		GetZaxis()->SetTitle(fCountsUnit);
		fscanf(file,"%d",&fparticle);
		fscanf(file,"%d",&fID);
		fscanf(file,"%d",&fScale);
		fscanf(file,"%d",&fTimeUnit);    
		if(NumberOfLinesBeforeEdef==7)
			fscanf(file,"%d%d",&Nx,&c);
		fscanf(file,"%d",&Nx);
		Ax=new Double_t[Nx];
		for(bin_x=0;bin_x<Nx;bin_x++)
			{
			fscanf(file,"%lf",Ax+bin_x);
			if(bin_x)
			if(Ax[bin_x]<Ax[bin_x-1])printf("bug at %d %E %E\n",bin_x,Ax[bin_x],Ax[bin_x-1]);
			}
		fscanf(file,"%d",&Ny);
		Ay=new Double_t[Ny];
		for(bin_y=0;bin_y<Ny;bin_y++)
			fscanf(file,"%lf",Ay+bin_y);
		SetBins(Nx-1,Ax,Ny-1,Ay);
	
		for(bin_x=0;bin_x<Nx;bin_x++)
		for(bin_y=0;bin_y<Ny;bin_y++)
			{
			fscanf(file,"%lf%lf",&counts,&ecounts);
			SetBinContent(bin_x,bin_y,counts);
			SetBinError(bin_x,bin_y,ecounts);
			}
		fclose(file);
		}


	fX0BoxX=GetXaxis()->GetBinLowEdge(1);
	fX1BoxX=GetXaxis()->GetBinUpEdge(GetXaxis()->GetNbins());
	fY0BoxX=-1000000000.;
	fY1BoxX= 1000000000.;
	fX0BoxY=GetYaxis()->GetBinLowEdge(1);
	fX1BoxY=GetYaxis()->GetBinUpEdge(GetYaxis()->GetNbins());
	fY0BoxY=-1000000000.;
	fY1BoxY= 1000000000.;
	
	fb0BoxY=1;
	fb1BoxY=GetYaxis()->GetNbins();
	fb0BoxX=1;
	fb1BoxX=GetXaxis()->GetNbins();

	
	fBoxX=new TBox(fX0BoxX,fY0BoxX,fX1BoxX,fY1BoxX);
	fBoxY=new TBox(fX0BoxY,fY0BoxY,fX1BoxY,fY1BoxY);
	
	fNInject=0;



	fC=NULL;
	fMain=NULL;
	};
// end of MTH2D(char *file_name,MTH2D_Controller *F):TH2D()

// This constructor is used when reading MCNP files.
// Defx and Defy are related to the energy and time cards cuts.
// units depends on the tally (flux at point position, surface, dose...)
MTH2D (MTArrayD *Defx,MTArrayD *Defy,char *name,char *Xtitle,char *Ytitle,char *units):TH2D(name,name,Defx->GetSize()-1,Defx->GetArray(),Defy->GetSize()-1,Defy->GetArray())
	{
	GetXaxis()->SetTitle(Xtitle);
	GetYaxis()->SetTitle(Ytitle);
	GetZaxis()->SetTitle(units);
// When built from a MCNP file, the time unit is shake, -> fTimeUnit=1;
	fTimeUnit=1;
	
	fX0BoxX=GetXaxis()->GetBinLowEdge(1);
	fX1BoxX=GetXaxis()->GetBinUpEdge(GetXaxis()->GetNbins());
	fY0BoxX=-1000000000.;
	fY1BoxX= 1000000000.;
	fX0BoxY=GetYaxis()->GetBinLowEdge(1);
	fX1BoxY=GetYaxis()->GetBinUpEdge(GetYaxis()->GetNbins());
	fY0BoxY=-1000000000.;
	fY1BoxY= 1000000000.;
	
	fb0BoxY=1;
	fb1BoxY=GetYaxis()->GetNbins();
	fb0BoxX=1;
	fb1BoxX=GetXaxis()->GetNbins();


	fBoxX=new TBox(fX0BoxX,fY0BoxX,fX1BoxX,fY1BoxX);
	fBoxY=new TBox(fX0BoxY,fY0BoxY,fX1BoxY,fY1BoxY);
	
	fScale=0;
	fNInject=0;
	fF=NULL;
	
	fC=NULL;
	fMain=NULL;

	};
// end of MTH2D (MTArrayD *Defx,MTArrayD *Defy,char *name,char *Xtitle,char *Ytitle,char *units):TH2D(name,name,Defx->GetSize()-1,Defx->GetArray(),Defy->GetSize()-1,Defy->GetArray())


// The y axis of the MTH2D is transformed with an arbitrary function.
// A new MTH2D is build, with the predefined MTArray Defy to define its y axis.
MTH2D (const MTH2D & H,MTArrayD *Defy,TF1 *fct,char *name,char *Ytitle):TH2D(name,name,H.GetXaxis()->GetNbins(),H.GetXaxis()->GetXbins()->GetArray(),Defy->GetSize()-1,Defy->GetArray())
	{
	// If H has been built with the TH2D constructor: 
	// TH2D TH2D(const char* name, const char* title, Int_t nbinsx, Double_t xlow, Double_t xup, Int_t nbinsy, Double_t ylow, Double_t yup)
	// the arrays associated to the X and Y axis are empty.
	// Defining a MTArrayD with H projection on X axis assures we get the correct x axis, independently
	// of the TH2D constructor that have been used. 
	MTArrayD *Defx=new MTArrayD(*H.ProjectionX());
	SetBins(Defx->GetSize()-1,Defx->GetArray(),Defy->GetSize()-1,Defy->GetArray());

	GetXaxis()->SetTitle(H.GetXaxis()->GetTitle());
	GetYaxis()->SetTitle(Ytitle);
	GetZaxis()->SetTitle(H.GetZaxis()->GetTitle());
	fparticle=H.GetParticle();
	fID=H.GetID();
	// Time units may not be relevant anymore (for example if positions are reconstructed 
	// from a Time Of Flight measurement).
	fTimeUnit=-1;
	fNInject=0;

	fF=H.GetMTH2D_Controller();
	fScale=H.GetScale();

	fC=NULL;
	fMain=NULL;

	Int_t by0,bx0,by,Counter;
	Double_t Y_0,Y_1,Y0_0,Y0_1;
	Double_t counts,previous_counts,error,previous_error;
	printf("Transformation...\n");
// What is basically done here is a reassigment of the bins content of the first histogram
// into bins of the new histogram, according to the Y axis transformation function.
	for(bx0=0,Counter=0;bx0<H.GetXaxis()->GetNbins();bx0++)
		{
	 	Counter++;
		if( Counter>H.GetXaxis()->GetNbins()/50)
			{
			Counter=0;
			printf("%.0lf%c done\n",100.*(Double_t)bx0/(Double_t)H.GetXaxis()->GetNbins(),37);
			}
		for(by0=0;by0<H.GetYaxis()->GetNbins();by0++)// old y axis
		if(H.GetBinContent(bx0+1,by0+1))
		for(by=0;by<Defy->GetSize()-1;by++) // new y axis
			{
			counts=0;
			error=0;
			// bin interval in the new Yaxis=[Y_0;Y_1]
			Y_0=fct->Eval(Defy->GetArray()[by]);
			Y_1=fct->Eval(Defy->GetArray()[by+1]);
			// bin interval in the old Yaxis=[Y0_0;Y0_1]
			Y0_0=H.GetYaxis()->GetBinLowEdge(by0+1);
			Y0_1=H.GetYaxis()->GetBinUpEdge(by0+1); 	      
			if(Y0_1<=Y0_0||Y_1<=Y_0)
				continue;
			// if 
			//
			// ___________Y_0___________________Y_1___________________bin to be filled
			// _____Y0_0______________________________Y0_1____________bin from original histo
			//             |---------------------|
			//
			// take the fraction (Y_1-Y_0)/(Y0_1-Y0_0) of content between Y0_0 and Y0_1
			// of the original histogram (corresponding to bin #by0) and put it in the new histogram
			if(Y_0>=Y0_0&&Y_0<Y0_1&&Y_1<=Y0_1&&Y_1>Y0_0)
				{
				counts+=H.GetBinContent(bx0+1,by0+1)*(Y_1-Y_0)/(Y0_1-Y0_0);
				error+=(H.GetBinError(bx0+1,by0+1)*(Y_1-Y_0)/(Y0_1-Y0_0))*(H.GetBinError(bx0+1,by0+1)*(Y_1-Y_0)/(Y0_1-Y0_0));
				}
			// if
			//
			// ___________Y_0___________________Y_1___________________bin to be filled
			// ____Y0_0___________________Y0_1________________________bin from original histo
			//             |----------------|
			//
			// take the fraction (Y0_1-Y_0)/(Y0_1-Y0_0) of content between Y0_0 and Y0_1
			// of the original histogram (corresponding to bin #by0) and put it in the new histogram
			if(Y_0>=Y0_0&&Y_0<Y0_1&&Y_1>Y0_1)
				{
				counts+=H.GetBinContent(bx0+1,by0+1)*(Y0_1-Y_0)/(Y0_1-Y0_0);
				error+=(H.GetBinError(bx0+1,by0+1)*(Y0_1-Y_0)/(Y0_1-Y0_0))*(H.GetBinError(bx0+1,by0+1)*(Y0_1-Y_0)/(Y0_1-Y0_0));
				}
			// if
			//
			// ___________Y_0___________________Y_1___________________bin to be filled
			// ___________________Y0_0___________________Y0_1_________bin from original histo
			//                      |------------|
			//
			// take the fraction (Y_1-Y0_0)/(Y0_1-Y0_0) of content between Y0_0 and Y0_1
			// of the original histogram (corresponding to bin #by0) and put it in the new histogram
			if(Y_0<Y0_0&&Y_1<=Y0_1&&Y_1>Y0_0)
				{
				counts+=H.GetBinContent(bx0+1,by0+1)*(Y_1-Y0_0)/(Y0_1-Y0_0);
				error+=(H.GetBinError(bx0+1,by0+1)*(Y_1-Y0_0)/(Y0_1-Y0_0))*(H.GetBinError(bx0+1,by0+1)*(Y_1-Y0_0)/(Y0_1-Y0_0));
				}
			// if
			//
			// ___________Y_0___________________Y_1___________________bin to be filled
			// _________________Y0_0_____Y0_1_________________________bin from original histo
			//                    |--------|
			//
			// take all bin content between Y0_0 and Y0_1 of the original histogram
			// (corresponding to bin #by0) and put it in the new histogram
			if(Y_0<Y0_0&&Y_1>Y0_1)
				{
				counts+=H.GetBinContent(bx0+1,by0+1);
				error+=(H.GetBinError(bx0+1,by0+1))*(H.GetBinError(bx0+1,by0+1));
				}
			previous_counts=GetBinContent(bx0+1,by+1);
			SetBinContent(bx0+1,by+1,previous_counts+counts);
			previous_error=GetBinError(bx0+1,by+1);
			SetBinError(bx0+1,by+1,sqrt(previous_error*previous_error+error));
			}
		}	


	fX0BoxX=H.GetBoxX()->GetX1();
	fX1BoxX=H.GetBoxX()->GetX2();
	fY0BoxX=H.GetBoxX()->GetY1();
	fY1BoxX=H.GetBoxX()->GetY2();
	fX0BoxY=Defy->GetArray()[0];
	fX1BoxY=Defy->GetArray()[Defy->GetSize()-1];
	fY0BoxY=H.GetBoxY()->GetY1();
	fY1BoxY=H.GetBoxY()->GetY2();
	fBoxX=new TBox(fX0BoxX,fY0BoxX,fX1BoxX,fY1BoxX);
	fBoxY=new TBox(fX0BoxY,fY0BoxY,fX1BoxY,fY1BoxY);
	fb0BoxX=H.Getb0BoxX();
	fb1BoxX=H.Getb1BoxX();
	fb0BoxY=1;
	fb1BoxY=GetYaxis()->GetNbins();


	};
// end of MTH2D (const MTH2D &H,MTArrayD *Defy,TF1 *fct,char *name,char *Ytitle)


// To define a MTH2D from a TH2D
MTH2D (const TH2D &H,char *name=""):TH2D(H)
	{
	// by default the name of the new MTH2D will be the same as H.
	// The constructor allows however to specify the name.
	if(name==NULL)name=new char[200];
	if(strcmp(name,""))
		SetName(name);
	else
		SetName(H.GetName());


	fX0BoxX=GetXaxis()->GetBinLowEdge(1);
	fX1BoxX=GetXaxis()->GetBinUpEdge(GetXaxis()->GetNbins());
	fY0BoxX=-1000000000.;
	fY1BoxX= 1000000000.;
	fX0BoxY=GetYaxis()->GetBinLowEdge(1);
	fX1BoxY=GetYaxis()->GetBinUpEdge(GetYaxis()->GetNbins());
	fY0BoxY=-1000000000.;
	fY1BoxY= 1000000000.;
	
	fb0BoxY=1;
	fb1BoxY=GetXaxis()->GetNbins();
	fb0BoxX=1;
	fb1BoxX=GetYaxis()->GetNbins();


	fBoxX=new TBox(fX0BoxX,fY0BoxX,fX1BoxX,fY1BoxX);
	fBoxY=new TBox(fX0BoxY,fY0BoxY,fX1BoxY,fY1BoxY);
	
	fScale=0;

	if(!strcmp(GetYaxis()->GetTitle(),"Time (ns)"))
		fTimeUnit=0;
	else if(!strcmp(GetYaxis()->GetTitle(),"Time (shake)"))
		fTimeUnit=1;
	else if(!strcmp(GetYaxis()->GetTitle(),"Time (s)"))
		fTimeUnit=2;
	else if(!strcmp(GetYaxis()->GetTitle(),"Time (ms)"))
		fTimeUnit=3;
	else if(!strcmp(GetYaxis()->GetTitle(),"Time (s)"))
		fTimeUnit=4;	
	else
		fTimeUnit=-1;

	fID=0;
	


	fNInject=0;
	fC=NULL;
	fMain=NULL;
	fF=NULL;

	};
// end of MTH2D (const TH2D &H,char *name=""):TH2D(H)

// To define a MTH2D from a TH1D
MTH2D (const TH1D &H,char *name="",char *Xtitle="",char *units=""):TH2D()
	{
	
	if(strcmp(Xtitle,""))
		GetXaxis()->SetTitle(H.GetXaxis()->GetTitle());
	else
		GetXaxis()->SetTitle(Xtitle);
	if(strcmp(units,""))
		GetZaxis()->SetTitle(H.GetYaxis()->GetTitle());
	else
		GetZaxis()->SetTitle(units);
	GetYaxis()->SetTitle("");

	fID=0;
	
	if(strcmp(name,""))
		SetName(name);
	else
		SetName(H.GetName());

	MTArrayD* Definition_x=new MTArrayD(H);
	MTArrayD* Definition_y=new MTArrayD(2,-1);
	SetBins(Definition_x->GetSize()-1,Definition_x->GetArray(),Definition_y->GetSize()-1,Definition_y->GetArray());
	Int_t bin;
	for(bin=0;bin<=H.GetNbinsX();bin++)
		{
		SetBinContent(bin,1,H.GetBinContent(bin));
		SetBinError(bin,1,H.GetBinError(bin));
		}

	fX0BoxX=GetXaxis()->GetBinLowEdge(1);
	fX1BoxX=GetXaxis()->GetBinUpEdge(GetXaxis()->GetNbins());
	fY0BoxX=-1000000000.;
	fY1BoxX= 1000000000.;
	fX0BoxY=GetYaxis()->GetBinLowEdge(1);
	fX1BoxY=GetYaxis()->GetBinUpEdge(GetYaxis()->GetNbins());
	fY0BoxY=-1000000000.;
	fY1BoxY= 1000000000.;
	
	fb0BoxY=1;
	fb1BoxY=GetYaxis()->GetNbins();
	fb0BoxX=1;
	fb1BoxX=GetXaxis()->GetNbins();


	fBoxX=new TBox(fX0BoxX,fY0BoxX,fX1BoxX,fY1BoxX);
	fBoxY=new TBox(fX0BoxY,fY0BoxY,fX1BoxY,fY1BoxY);
	
	fScale=0;
	fNInject=0;

	fTimeUnit=-1;

	
	fC=NULL;
	fMain=NULL;
	fF=NULL;
	};
// end of MTH2D (TH1D *H,char *name="",char *Xtitle="",char *units=""):TH2D()

// Default constructor without arguments
MTH2D():TH2D()
	{
	
	fX0BoxX=GetXaxis()->GetBinLowEdge(1);
	fX1BoxX=GetXaxis()->GetBinUpEdge(GetXaxis()->GetNbins());
	fY0BoxX=-1000000000.;
	fY1BoxX= 1000000000.;
	fX0BoxY=GetYaxis()->GetBinLowEdge(1);
	fX1BoxY=GetYaxis()->GetBinUpEdge(GetYaxis()->GetNbins());
	fY0BoxY=-1000000000.;
	fY1BoxY= 1000000000.;
	
	fb0BoxY=1;
	fb1BoxY=GetYaxis()->GetNbins();
	fb0BoxX=1;
	fb1BoxX=GetXaxis()->GetNbins();

	fBoxX=new TBox(fX0BoxX,fY0BoxX,fX1BoxX,fY1BoxX);
	fBoxY=new TBox(fX0BoxY,fY0BoxY,fX1BoxY,fY1BoxY);
	
	fScale=0;
	fNInject=0;
	fTimeUnit=-1;
	fID=0;


	fC=NULL;
	fMain=NULL;
	fF=NULL;
	};
// end of MTH2D ():TH2D()


/////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////
// DoDraw
Int_t DoDraw(Int_t level=0);
/////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////


Int_t GetPlot_Errors_Status()
{
if(fPlot_Errors->IsDown())
	return 1;
else
	return 0;
};

Int_t GetShow_FirstEscape_Status()
{
if(fShow_FirstEscape->IsDown())
	return 1;
else
	return 0;
};


Int_t GetShow_SecondEscape_Status()
{
if(fShow_SecondEscape->IsDown())
	return 1;
else
	return 0;
};

// When the boxes X limites are defined with the Embedded Canvas text entrie rather with the mouse
void SetBoxesLimites()
	{
	fBoxX->SetX1(fETCuts_Entries_0->GetNumber());
	fBoxX->SetX2(fETCuts_Entries_1->GetNumber());
	fBoxY->SetX1(fETCuts_Entries_2->GetNumber());
	fBoxY->SetX2(fETCuts_Entries_3->GetNumber());
	DoDraw();
	fETCuts_Entries_0->SetNumber(fBoxX->GetX1());
	fETCuts_Entries_1->SetNumber(fBoxX->GetX2());
	fETCuts_Entries_2->SetNumber(fBoxY->GetX1());
	fETCuts_Entries_3->SetNumber(fBoxY->GetX2());
	}; 
// end of void SetBoxesLimites()

// The vertical upper and lower limites (=Y coordinates) of the boxes should stay out of view 
// (= they are out of the frame current range).
// However, it can happen, after using the logX or logY options for example, that the vertical upper or lower limites of the
// boxes appear on the frame.
// MTH2D::UpdateBoxLimites() makes sure that the upper and lower boxes limites will stay out of the current frame. 
// Also, MTH2D::UpdateBoxLimites() updates the bin integers values related to the left/right positions of the boxes.
// These integers are used to define the projections since it is not possible to perform 1D projections of
// a 2D histogram with double values (see ROOT ProjectionX or ProjectionY methods).
Int_t UpdateBoxLimites()
	{
	if(GetXaxis()->GetNbins()==1||GetYaxis()->GetNbins()==1)
		{
		fX0BoxY=GetYaxis()->GetBinLowEdge(1);
		fb0BoxY=1;
		fX1BoxY=GetYaxis()->GetBinUpEdge(GetYaxis()->GetNbins());
		fb1BoxY=GetYaxis()->GetNbins();
		
		fX0BoxX=GetXaxis()->GetBinLowEdge(1);
		fb0BoxX=1;
		fX1BoxX=GetXaxis()->GetBinUpEdge(GetXaxis()->GetNbins());
		fb1BoxX=GetXaxis()->GetNbins();
		return 0;
		}
	Int_t limite;

	fY0BoxX=-1000000000.;
	fY1BoxX= 1000000000.;
	fY0BoxY=-1000000000.;
	fY1BoxY= 1000000000.;
	
	if(fBoxY==NULL)
		fBoxY=new TBox(fX0BoxY,fY0BoxY,fX1BoxY,fY1BoxY);
		
	fX0BoxY=fBoxY->GetX1();
	fX1BoxY=fBoxY->GetX2();
	
	if(GetYaxis()->GetNbins()!=1)
	{
	if(fX0BoxY<GetYaxis()->GetBinLowEdge(1))
		{
		fX0BoxY=GetYaxis()->GetBinLowEdge(1);
		fb0BoxY=1;
		}
	if(fX0BoxY>=GetYaxis()->GetBinUpEdge(GetYaxis()->GetNbins()))
		{
		fX0BoxY=GetYaxis()->GetBinLowEdge(GetYaxis()->GetNbins());
		fb0BoxY=GetYaxis()->GetNbins();
		}
	if(fX1BoxY>GetYaxis()->GetBinUpEdge(GetYaxis()->GetNbins()))
		{
		fX1BoxY=GetYaxis()->GetBinUpEdge(GetYaxis()->GetNbins());
		fb1BoxY=GetYaxis()->GetNbins();
		}
	if(fX1BoxY<GetYaxis()->GetBinLowEdge(1))
		{
		fX1BoxY=GetYaxis()->GetBinUpEdge(1);
		fb1BoxY=1;
		}
	for(limite=1;limite<=GetYaxis()->GetNbins();limite++)
		{
		if(fX0BoxY>=GetYaxis()->GetBinLowEdge(limite)&&fX0BoxY<GetYaxis()->GetBinUpEdge(limite))
			{
			fX0BoxY=GetYaxis()->GetBinLowEdge(limite);
			fb0BoxY=limite;
			}
		if(fX1BoxY>GetYaxis()->GetBinLowEdge(limite)&&fX1BoxY<=GetYaxis()->GetBinUpEdge(limite))
			{
			fX1BoxY=GetYaxis()->GetBinUpEdge(limite);	
			fb1BoxY=limite;
			}
		}	
	fBoxY->SetX1(fX0BoxY);
	fBoxY->SetX2(fX1BoxY);
	fBoxY->SetY1(fY0BoxY);
	fBoxY->SetY2(fY1BoxY);
	fBoxY->SetFillStyle(0);
	fBoxY->SetLineColor(2);
	fBoxY->SetLineStyle(2);
	}
		
	if(fBoxX==NULL)
		fBoxX=new TBox(fX0BoxX,fY0BoxX,fX1BoxX,fY1BoxX);
		
	fX0BoxX=fBoxX->GetX1();
	fX1BoxX=fBoxX->GetX2();
	if(fX0BoxX<GetXaxis()->GetBinLowEdge(1))
		{
		fX0BoxX=GetXaxis()->GetBinLowEdge(1);
		fb0BoxX=1;
		}
	if(fX0BoxX>GetXaxis()->GetBinUpEdge(GetXaxis()->GetNbins()))
		{
		fX0BoxX=GetXaxis()->GetBinLowEdge(GetXaxis()->GetNbins());
		fb0BoxX=GetXaxis()->GetNbins();
		}
	if(fX1BoxX>GetXaxis()->GetBinUpEdge(GetXaxis()->GetNbins()))
		{
		fX1BoxX=GetXaxis()->GetBinUpEdge(GetXaxis()->GetNbins());
		fb1BoxX=GetXaxis()->GetNbins();
		}
	if(fX1BoxX<GetXaxis()->GetBinLowEdge(1))
		{
		fX1BoxX=GetXaxis()->GetBinUpEdge(1);
		fb1BoxX=1;
		}
	for(limite=1;limite<=GetXaxis()->GetNbins();limite++)
		{
		if(fX0BoxX>=GetXaxis()->GetBinLowEdge(limite)&&fX0BoxX<GetXaxis()->GetBinUpEdge(limite))
			{
			fX0BoxX=GetXaxis()->GetBinLowEdge(limite);
			fb0BoxX=limite;
			}
		if(fX1BoxX>GetXaxis()->GetBinLowEdge(limite)&&fX1BoxX<=GetXaxis()->GetBinUpEdge(limite))
			{
			fX1BoxX=GetXaxis()->GetBinUpEdge(limite);	
			fb1BoxX=limite;
			}
		}
	fBoxX->SetX1(fX0BoxX);
	fBoxX->SetX2(fX1BoxX);
	fBoxX->SetY1(fY0BoxX);
	fBoxX->SetY2(fY1BoxX);
	fBoxX->SetFillStyle(0);
	fBoxX->SetLineColor(2);
	fBoxX->SetLineStyle(2);


	fETCuts_Entries_0->SetNumber(fBoxX->GetX1());
	fETCuts_Entries_1->SetNumber(fBoxX->GetX2());
	fETCuts_Entries_2->SetNumber(fBoxY->GetX1());
	fETCuts_Entries_3->SetNumber(fBoxY->GetX2());

	return 1;

	}; 
// end of void UpdateBoxLimites()

// The Print output can be copied and pasted into a MCNP input file
// to use the energy or time distribution as a density for a source definition 
// (cf SDEF card in MCNP manual)
void PrintMTH2D(char *name,TH1D *H)
	{
	FILE *file;
	Int_t bin;
	
	file=fopen(name,"wb");
	if(file!=NULL)
		{
		fprintf(file,"c SumOfWeights=%E\n",H->GetSumOfWeights());
		fprintf(file,"#     SI1         SP1\n");
		fprintf(file,"      H           D\n");
		fprintf(file,"      0           0\n");
		fprintf(file,"      %E      0\n",H->GetBinLowEdge(1));
		for(bin=1;bin<=H->GetNbinsX();bin++)
			fprintf(file,"      %E      %E\n",H->GetBinLowEdge(bin)+H->GetBinWidth(bin),H->GetBinContent(bin));
		fclose(file);
		}
	}; 
// end of void Print(char *name,TH1D *H)



///////////////////////////////////////////////////////////////////////////////////////////////
/////// THESE 2 ROUTINES ALLOW TO GET THE MOUSE COORDINATES IN THE SELECTED CANVAS ////////////
///////////////////////////////////////////////////////////////////////////////////////////////
void EventInfo(Int_t event,Int_t px,Int_t py,TObject*selected)
	{
	Int_t evt=event;
	evt++;
	fStatusBar->SetText(selected->GetTitle(),0);
	fStatusBar->SetText(selected->GetName(),1);
	fStatusBar->SetText(selected->GetObjectInfo(px,py),2);
	};
void ProcessedEvent(Int_t event,Int_t px,Int_t py,TObject*selected)
	{
	fC->ProcessedEvent(event,px,py,selected);
	};



///////////////////////////////////////////////////////////////////////////////////////////////
/////// FILL BINS CONTENTS OF THE MTH2D WITH DATA FROM MCNP FILE //////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////

void Fill_From_MCNP_File(FILE *mcnp_file)
	{
	char *line,*w0;
	Double_t energy,count,ecount;
	Int_t b,Nblock_T,e,t,t_max;
	line=(char*)calloc(400,sizeof(char));
	w0=(char*)calloc(200,sizeof(char));
	TString *St;St=new TString();

// Concerning data presentation in a MCNP .o file:
// Each line is associated to an energy bin.
// For each line, the first column indicates the energy bin value. The columns which follow are associated to 
// a time bin.
// There are 5 times bins maximum per line. So, if there are N time bins, M blocks have to be read, M being
// the smallest integral value not less than N/5.
	Nblock_T=(Int_t)ceil((Double_t)(GetYaxis()->GetNbins()+1)/5.);

// Each block starts with:
//         time:       T1           T2           T3           T4           T5
//      energy
// , T1 being the first time bin upper limite, T2 the second bin upper limite, ...		
// Then  the bin content values are displayed, in the form
//    E1   N11 e11   N12 e12   N13 e13      N14 e14   N15 e15
//    E2   N21 e21   N22 e22   N23 e23      N24 e24   N25 e25
//    E3   N31 e31   N32 e32   N33 e33      N34 e34   N35 e35
//	...
// , with E1 the first energy bin upper limite, E2 the second bin upper limite, ...		
// , with Nij the bin content of the 2D histogram for energy bin #i and time bin #j,
// , with eij the error associated to Nij,
// Each block finishes with:
//      total    Total_T1 eTotal_T1   Total_T2 eTotal_T2   Total_T3 eTotal_T3   Total_T4 eTotal_T4   Total_T5 eTotal_T5
// where Total_Tj is the total bin content associated to the time bin #j
// and eTotal_Tj is the error associated to Total_Tj.
	for(b=0;b<Nblock_T;b++)
		{
		if(GetXaxis()->GetNbins()!=1)
			{
			do
				{
				St->Gets(mcnp_file);
				St->ReplaceAll(" ","");
				if(St->BeginsWith("energy"))
					break;
				}
			while(1);
			}
		if(GetYaxis()->GetNbins()!=1&&GetXaxis()->GetNbins()==1)
			{
			do
				{
				St->Gets(mcnp_file);
				St->ReplaceAll(" ","");
				if(St->BeginsWith("time:"))
					break;
				}
			while(1);
			}
		//no energy and time binning -> only one real value (a tally modified for dose measurement)
		if(GetYaxis()->GetNbins()==1&&GetXaxis()->GetNbins()==1)
			{
			do
				{
				fscanf(mcnp_file,"%s",w0);
				if(!strcmp(w0,"x,y,z")||!strcmp(w0,"bin:"))
					break;
				}	
			while(1);
			for(e=0;e<3;e++)
				fscanf(mcnp_file,"%s",w0);
			fscanf(mcnp_file,"%lf%lf",&count,&ecount);
			SetBinContent(1,1,count);
			SetBinError(1,1,count*ecount);
			break;
			}
		if(GetYaxis()->GetNbins()+1>5*b)
			{
			t_max=5*(b+1);
			if(GetYaxis()->GetNbins()+1<5*(b+1))
				t_max=GetYaxis()->GetNbins()+1;
			if(GetYaxis()->GetNbins()==1)t_max=1;
			
			if(GetXaxis()->GetNbins()!=1)
				for(e=0;e<GetXaxis()->GetNbins()+1;e++)
					{
					fscanf(mcnp_file,"%lf",&energy);
					if(GetYaxis()->GetNbins()!=1)
					for(t=5*b;t<t_max;t++)
						{
						fscanf(mcnp_file,"%lf%lf",&count,&ecount);
						SetBinContent(e,t,count);
						SetBinError(e,t,count*ecount);
						}
					else
						{
						fscanf(mcnp_file,"%lf%lf",&count,&ecount);
						SetBinContent(e,1,count);
						SetBinError(e,1,count*ecount);
						}
					if(GetYaxis()->GetNbins()!=1)
					if(GetYaxis()->GetNbins()+1<5*(b+1)&&GetYaxis()->GetNbins()+1!=1)
						fscanf(mcnp_file,"%lf%lf",&count,&ecount);
					}
			else
				{
				for(t=5*b;t<t_max;t++)
						{
						fscanf(mcnp_file,"%lf%lf",&count,&ecount);
						SetBinContent(1,t,count);
						SetBinError(1,t,count*ecount);
						}				
				}
			}//if(GetYaxis()->GetNbins()+1>5*b)
		}//for(b=0;b<Nblock_T;b++)
	};
// end of void Fill_From_MCNP_File(FILE *mcnp_file)





///////////////////////////////////////////////////////////////////////////////////////////////
/////// TIME-ENERGY SMEARING WITH ENERGY RESPONSE FUNCTION AND TIME GAUSSIAN FUNCTION /////////
///////////////////////////////////////////////////////////////////////////////////////////////
MTH2D* Smear(char *_ResponseFunctionFile_=NULL,Double_t _s_=0,Bool_t SHOW=1);
///////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////

// simple histogram scaling
void Scaling()
{
Scale(fInjEvtsEntries->GetNumber());
DoDraw();
}

// From the energy vs time histogram it is possible to define a 2D Probability Density Function (PDF).
// If the total sum of weights of the 2D histogram is T and the number of source particle is N, the number
// of "Counts" is (int)N*T, when N*T>1. The N*T counts are distributed according to the PDF.
MTH2D* Inject(Double_t Nevt=0,Bool_t SHOW=1)
{
Int_t evt;
Double_t *RandomTable,RandomTableNormalization=0,u;
Int_t bin_x,bin_y,PositionOnRandomTable=1,RandomTableLengh;
Int_t *Bin_xOnRandomTable,*Bin_yOnRandomTable;
Int_t *QuantilesPositionOnRandomTable,NQuantiles,q;
Double_t *Quantile;
Double_t Nevents;
char *w0;w0=new char[400];
if(Nevt<=0)
	{
	sprintf(w0,"%s_%E_particles_%d",GetName(),fInjEvtsEntries->GetNumber(),fNInject);
	Nevt=fInjEvtsEntries->GetNumber();
	}
else
	sprintf(w0,"%s_%E_particles_%d",GetName(),Nevt,fNInject);

Nevents=(Nevt*GetSumOfWeights());
	
// In case the injection of the same number of events has already been performed, do nothing
if(gROOT->FindObject(w0)!=NULL)
	{
	TPaveText *info_text=new TPaveText(0.,0.3,1,0.8);
	info_text->Clear();
	sprintf(w0,"%s %d:\n",GetName(),fNInject);
	info_text->AddText(w0);	
	info_text->AddText("Injection of ");	
	sprintf(w0,"Injection of %E source particle\n",Nevt);
	info_text->AddText(w0);	
	info_text->AddText("already done.");	
	info_text->SetTextFont(22);info_text->SetTextColor(2);
	info_text->SetBorderSize(0);info_text->SetFillColor(0);
	if(fC!=NULL&&GetXaxis()->GetNbins()!=1&&GetYaxis()->GetNbins()!=1)
		{
		fC->cd(6)->Clear();
		info_text->Draw();
		fC->Update();
		}
	else
		{
		TCanvas *Cinfo=new TCanvas("Cinfo","Cinfo");
		Cinfo->cd();
		info_text->Draw();
		Cinfo->Update();
		}
	
	return (new MTH2D());
	}

if(GetYaxis()->GetNbins()!=1&&GetXaxis()->GetNbins()!=1&&fC!=NULL)
	{
	fC->cd(6);
	TPaveText *info_text=new TPaveText(0.,0.3,1,0.8);
	//info_text->SetTextSize(0.08);
	if(Nevents<1)
		info_text->SetTextColor(2);
	sprintf(w0,"%s:\n",GetName());
	info_text->AddText(w0);	
	sprintf(w0,"Number of source particle");
	info_text->AddText(w0);	
	sprintf(w0,"must be greater than");
	info_text->AddText(w0);	
	sprintf(w0,"%E.",1./GetSumOfWeights());
	info_text->AddText(w0);	
	sprintf(w0,"%.3E particle injected.",(Double_t)Nevents);
	info_text->AddText(w0);	
	info_text->SetTextFont(22);
	info_text->SetBorderSize(0);info_text->SetFillColor(0);
	info_text->Draw();
	fC->Update();
	}
else
	{
	TCanvas *Cinfo=new TCanvas("Cinfo","Cinfo");
	TPaveText *info_text=new TPaveText(0.,0.3,1,0.8);
	//info_text->SetTextSize(0.08);
	if(Nevents<1)
		info_text->SetTextColor(2);
	sprintf(w0,"%s:\n",GetName());
	info_text->AddText(w0);	printf("%s\n",w0);
	sprintf(w0,"Number of source particle");
	info_text->AddText(w0);		printf("%s\n",w0);
	sprintf(w0,"must be greater than");
	info_text->AddText(w0);		printf("%s\n",w0);
	sprintf(w0,"%E.",1./GetSumOfWeights());
	info_text->AddText(w0);		printf("%s\n",w0);
	sprintf(w0,"%.3E particle injected.",(Double_t)Nevents);
	info_text->AddText(w0);		printf("%s\n",w0);
	info_text->SetTextFont(22);
	info_text->SetBorderSize(0);info_text->SetFillColor(0);
	info_text->Draw();
	Cinfo->Update();
	}
		

sprintf(w0,"%s_%E_particles_%d",GetName(),Nevt,fNInject);
MTH2D *T_i=new MTH2D(*this);
T_i->GetZaxis()->SetTitle("Counts");
T_i->SetName(w0);
T_i->Scale(0);
// The use of fNInject as a Inject() calls counter allows to get
// several versions of histograms with the same number of injected events without erasing the previous ones.
// Since the random generator seed is set with time, the histograms are different.
fNInject++;

if(Nevents<1)
	return T_i;


// Prepare the array from which bins number will be sampled
NQuantiles=200;
QuantilesPositionOnRandomTable=new Int_t[NQuantiles];
Quantile=new Double_t[NQuantiles];
for(q=0;q<NQuantiles;q++)
	Quantile[q]=(Double_t)(q)/(Double_t)(NQuantiles-1);


RandomTable=new	Double_t[RandomTableLengh=1+(GetXaxis()->GetNbins())*(GetYaxis()->GetNbins())];
Bin_xOnRandomTable=new	Int_t[RandomTableLengh];
Bin_yOnRandomTable=new	Int_t[RandomTableLengh];
RandomTable[0]=0;
for(bin_x=1,PositionOnRandomTable=1;bin_x<=GetXaxis()->GetNbins();bin_x++)
for(bin_y=1;bin_y<=GetYaxis()->GetNbins();bin_y++)
	{
	if(PositionOnRandomTable>=RandomTableLengh)
		{
		printf("RandomTable lengh exeeded: %d >=%d\n",PositionOnRandomTable,RandomTableLengh);
		return (new MTH2D());
		}
	RandomTable[PositionOnRandomTable]=RandomTable[PositionOnRandomTable-1]+GetBinContent(bin_x,bin_y);
	RandomTableNormalization=RandomTable[PositionOnRandomTable];
	Bin_xOnRandomTable[PositionOnRandomTable]=bin_x;
	Bin_yOnRandomTable[PositionOnRandomTable]=bin_y;
	PositionOnRandomTable++;
	}

for(PositionOnRandomTable=1,q=0;PositionOnRandomTable<RandomTableLengh;PositionOnRandomTable++)
	RandomTable[PositionOnRandomTable]/=RandomTableNormalization;

for(q=0;q<NQuantiles;q++)
for(PositionOnRandomTable=0;PositionOnRandomTable<RandomTableLengh;PositionOnRandomTable++)
if(Quantile[q]>=RandomTable[PositionOnRandomTable]&&Quantile[q]<RandomTable[PositionOnRandomTable+1])
		{
		QuantilesPositionOnRandomTable[q]=PositionOnRandomTable;
		break;
		}

if(GetXaxis()->GetNbins()!=1&&GetYaxis()->GetNbins()!=1&&fC!=NULL)
	fC->cd(6);
else
	{
	TCanvas *Cinfo=new TCanvas("Cinfo","Cinfo");
	Cinfo->cd();
	}	

TPaveText *info_progress_text=new TPaveText(0.,0.9,1,1.);
info_progress_text->SetTextSize(0.08);
sprintf(w0,"Injecting events...");
info_progress_text->AddText(w0);
info_progress_text->SetTextFont(8);
info_progress_text->SetTextSize(12);
info_progress_text->SetBorderSize(0);info_progress_text->SetFillColor(0);
info_progress_text->Draw();
TPave *info_progress=new TPave(0.,0.8,0.01,0.9);
info_progress->SetFillColor(4);
info_progress->SetBorderSize(0);
info_progress->Draw();

if(GetXaxis()->GetNbins()!=1&&GetYaxis()->GetNbins()!=1&&fC!=NULL)
	fC->Update();	

TRandom *r0 = new TRandom(time(NULL));
Int_t Counter;
printf("Injecting %E events...\n",Nevents);
// sample the 2D histogram
for(evt=0,Counter=0;evt<Nevents;evt++)
	{
	Counter++;
	if(Counter>Nevents/50)
		{
		Counter=0;
		info_progress->SetX2NDC((Double_t)evt/(Double_t)Nevents);
		info_progress->Draw();
		printf("%.0lf%c done\n",100.*(Double_t)evt/(Double_t)Nevents,37);
		if(GetXaxis()->GetNbins()!=1&&GetYaxis()->GetNbins()!=1&&fC!=NULL)
			fC->Update();
		}
	u=r0->Rndm(evt);
	PositionOnRandomTable=0;
	for(q=0;q<NQuantiles-1;q++)
	if(u>=Quantile[q]&&u<Quantile[q+1])
		break;
	for(PositionOnRandomTable=QuantilesPositionOnRandomTable[q];PositionOnRandomTable<RandomTableLengh;PositionOnRandomTable++)
	if(u<=RandomTable[PositionOnRandomTable]&&u>RandomTable[PositionOnRandomTable-1])
		{
		T_i->AddBinContent(Bin_yOnRandomTable[PositionOnRandomTable]*(GetXaxis()->GetNbins()+2) + Bin_xOnRandomTable[PositionOnRandomTable]);
		break;
		}
	}
for(bin_x=1;bin_x<=GetXaxis()->GetNbins();bin_x++)
for(bin_y=1;bin_y<=GetYaxis()->GetNbins();bin_y++)
	{
	u=T_i->GetBinContent(bin_x,bin_y);
	T_i->SetBinError(bin_x,bin_y,sqrt(fabs(u)));
	}

if(SHOW)
	T_i->DoDraw();

if(GetXaxis()->GetNbins()!=1&&GetYaxis()->GetNbins()!=1&&fC!=NULL)
	fC->cd(6)->Clear();

return T_i;

};
// end of Int_t MTH2D::Inject(Double_t Nevt=0,Bool_t SHOW=1)

Int_t ShowElementsPeaks();




// When the bins don't have all the same size, it can be sometime usefull to scale the bins
// contents with the bin widths to get a readable picture.
// (for example in case the first time bin width is 100 shakes and the next bin widthes are 100000 shakes)
void ScaleWithBinWidth()
{
Int_t binx,biny,c;
Double_t dx,dy,count,ecount;
char *w0;w0=new char[400];
char *w1;w1=new char[400];
char *w2;w2=new char[400];
char *fCountsUnit=new char[400];

sprintf(w0,"%s_px",GetName());
gROOT->Delete(w0);
sprintf(w0,"%s_py",GetName());
gROOT->Delete(w0);
TH1D *Hx=new TH1D(*ProjectionX());
TH1D *Hy=new TH1D(*ProjectionY());

(!fScale)?(fScale=1):(fScale=0);

for(binx=1;binx<=GetNbinsX();binx++)
for(biny=1;biny<=GetNbinsY();biny++)
	{
	dx=Hx->GetBinWidth(binx);
	dy=Hy->GetBinWidth(biny);
	if(dx==0||dy==0)
		continue;

	count=GetBinContent(binx,biny);
	ecount=GetBinError(binx,biny);
	if(fScale)
		{
		count/=dx*dy;
		ecount/=dx*dy;
		}
	else
		{
		count*=dx*dy;
		ecount*=dx*dy;
		}
	SetBinContent(binx,biny,count);
	SetBinError(binx,biny,ecount);
	}
if(fScale)
	{
	// Check for units in X and Y axes.
	// By definition, units are between parenthesis, at the very end
	// of the axis title -> check for the last opened parenthesis.

	for(c=(Int_t)strlen(GetXaxis()->GetTitle());c>=0;c--)
	if(GetXaxis()->GetTitle()[c]=='(') break;
	sprintf(w1,"%s",GetXaxis()->GetTitle()+c);

	for(c=(Int_t)strlen(GetYaxis()->GetTitle());c>=0;c--)
	if(GetYaxis()->GetTitle()[c]=='(') break;
	sprintf(w2,"%s",GetYaxis()->GetTitle()+c);
	
	w0[0]='\0';
	if((Int_t)strlen(w1))
		sprintf(w0,"%s#times%s^{-1}",GetZaxis()->GetTitle(),w1);
	if((Int_t)strlen(w2))
		sprintf(w0,"%s#times%s^{-1}",w0,w2);
	sprintf(fCountsUnit,"%s",w0);	
	}
else
	{
	sprintf(fCountsUnit,"%s",GetZaxis()->GetTitle());
	if((Int_t)strlen(fCountsUnit))
	for(c=0;c<(Int_t)strlen(fCountsUnit)-1;c++)
	if(fCountsUnit[c]=='#'&&fCountsUnit[c+1]=='t')
		{
		fCountsUnit[c]='\0';
		break;
		}
	}
GetZaxis()->SetTitle(fCountsUnit);
DoDraw();
};
// end of void ScaleWithBinWidth()



// Calling this method changes the time unit
void ChangeTimeUnits(Int_t t)
{
Int_t u;
Double_t **p;
char **units;
units=(char**)calloc(5,sizeof(char*));
p=(Double_t**)calloc(5,sizeof(Double_t*));
for(u=0;u<5;u++)
	{
	units[u]=(char*)calloc(50,sizeof(char));
	p[u]=(Double_t*)calloc(5,sizeof(Double_t));
	}
sprintf(units[0],"Time (ns)");
sprintf(units[1],"Time (shake)");
sprintf(units[2],"Time (s)");
sprintf(units[3],"Time (ms)");
sprintf(units[4],"Time (s)");
for(u=0;u<5;u++)
if(!strcmp(GetYaxis()->GetTitle(),units[u]))
	fTimeUnit=u;
// correspondance array
p[0][1]=-1; // go from ns to shake 	-> divide by 10
p[0][2]=-3; // go from ns to s    	-> divide by 10^3
p[0][3]=-6; // go from ns to ms    	-> divide by 10^6
p[0][4]=-9; // go from ns to  s    	-> divide by 10^9

p[1][0]= 1; // go from shake to ns 	-> multiply by 10
p[1][2]=-2; // go from shake to s	-> divide by 10^2
p[1][3]=-5; // go from shake to ms	-> divide by 10^5
p[1][4]=-8; // go from shake to  s	-> divide by 10^8

p[2][0]= 3; // go from s to ns 	-> multiply by 10^3
p[2][1]= 2; // go from s to shake    	-> multiply by 10^2
p[2][3]=-3; // go from s to ms    	-> divide by 10^3
p[2][4]=-6; // go from s to  s    	-> divide by 10^6

p[3][0]= 6; // go from ms to ns 	-> multiply by 10^6
p[3][1]= 5; // go from ms to shake    	-> multiply by 10^5
p[3][2]= 3; // go from ms to s    	-> multiply by 10^3
p[3][4]=-3; // go from ms to  s    	-> divide by 10^3

p[4][0]= 9; // go from s to ns 		-> multiply by 10^9
p[4][1]= 8; // go from s to shake    	-> multiply by 10^8
p[4][2]= 6; // go from s to s    	-> multiply by 10^6
p[4][3]= 3; // go from s to  ms    	-> multiply by 10^3

if(GetYaxis()->GetNbins()!=1)
	{
	Double_t *Newbins;
	Int_t bin;
	Newbins=new Double_t[GetYaxis()->GetNbins()+2];
	for(bin=0;bin<=GetYaxis()->GetNbins();bin++)
		Newbins[bin]=GetYaxis()->GetXbins()->GetArray()[bin]*pow(10.,p[fTimeUnit][t]);	
	fBoxY->SetX2(fX1BoxY*pow(10.,p[fTimeUnit][t]));
	fX1BoxY=fBoxY->GetX2();
	fBoxY->SetX1(fX0BoxY*pow(10.,p[fTimeUnit][t]));
	fX0BoxY=fBoxY->GetX1();
	GetYaxis()->Set(GetYaxis()->GetNbins(),Newbins);
	GetYaxis()->SetTitle(units[t]);
	}

fTimeUnit=t;
if(fC)
	DoDraw();
};

ClassDef(MTH2D,1)

};
// end of MTH2D class definition

ClassImp(MTH2D)





/*************************************************************************/
/*************************************************************************/
/*********	MCNP_run	**********************************/

// Once the MCNP output file has been read, the tally that have been recognized
// are displayed in a frame having the same name as the MCNP file.
// Each button on this frame corresponds to an identified tally.
// A mouse click on one of the buttons will call the DoDraw method associated to the
// corresponding MTH2D.
// Also, it proved to be usefull to add the possibility to add tallies issued by the same
// MCNP file. The tallies that one wants to add can be given in a text area.
// For example tallies F15P, F25P and F35P have been identified in a MCNP output file.
// There will be 3 buttons labeled "F15P", "F25P" and "F35P".
// To add tally 15 and tally 25, one writes "F15P F25P" in the text area and click the "Add" button.

class MCNP_run
{
RQ_OBJECT("MCNP_run")
private:
TGMainFrame *fMainFrame;
TGHorizontalFrame *fHframe_Add;
TGTextButton *fAddButton;
TGTextEntry *fAdd_Entry;
TGTextBuffer *fAdd_TextBuffer;
char *fAdd_Text;
Int_t fNTally_Found;
Int_t fNphoton;
char *fMCNP_File_name;
MTH2D_Controller *fControler;
Int_t *ftally_id;
Int_t *fparticle;
char** MTH2Dname;

public:

MCNP_run()
	{
	};

// In a MCNP output file, all tallies are searched and linked with the corresponding energy and time cards.
// The number of tallies is counted and stored in the MTH2D class static variables.
MCNP_run(const char *mcnp_file_name,MTH2D_Controller *F=NULL,Bool_t SHOW=1)
	{
	MTArrayD* Edef;
	MTArrayD* Tdef;
	Int_t NEdef=0,NTdef=0;
	Int_t Source_Particle_I;
	char *Source_Particle_S;
	Source_Particle_S=new char[200];
	Int_t NSDdef=0,*SDdef;SDdef=new Int_t[100];	
	Int_t NDFdef=0,*DFdef,THERE_IS_DF_CARD;DFdef=new Int_t[100];	
	Int_t *DEdef;DEdef=new Int_t[100];	
	char *w0,*w1,*w2;
	char **Edef_line;
	char **Tdef_line;
	w0=new char[400];w1=new char[400];w2=new char[400];
	Int_t id,e,t,n,FINISHED_READING_SOURCE_CODE=0;
	Int_t TallyType,sd;
	Int_t last_slash;
	fparticle=new Int_t[100];
	ftally_id=new Int_t[100];
	Int_t NTH2D=0;
	char *tally_name,*counts;
	MTH2D *Tally;

	tally_name=new char[400];
	counts=new char[400];
	TString *St,*St_;
	St=new TString(500);
	St_=new TString(500);
	MTArrayD *tdef,*edef;
	MTArrayD *Array1D; Array1D=new MTArrayD(2,-1);

	fControler=F;

	MTH2Dname=(char**)calloc(50,sizeof(char*));
	for(t=0;t<50;t++)
		MTH2Dname[t]=(char*)calloc(200,sizeof(char));

	Edef_line=(char**)calloc(50,sizeof(char*));
	for(t=0;t<50;t++)
		Edef_line[t]=(char*)calloc(400,sizeof(char));
	Tdef_line=(char**)calloc(50,sizeof(char*));
	for(t=0;t<50;t++)
		Tdef_line[t]=(char*)calloc(400,sizeof(char));
		
	fMCNP_File_name=new char[400];

	sprintf(fMCNP_File_name,"%s",mcnp_file_name);
	// No need to keep the path name of the mcnp_file in fMCNP_File_name.
	for(t=0,last_slash=0;t<(Int_t)strlen(fMCNP_File_name);t++)
	if((Int_t)fMCNP_File_name[t]==47||(Int_t)fMCNP_File_name[t]==92)
		last_slash=t;
	if(last_slash)
		sprintf(fMCNP_File_name,"%s",fMCNP_File_name+last_slash+1);
	FILE *mcnp_file;
	mcnp_file=fopen(mcnp_file_name,"rb");
	if(mcnp_file==NULL)
		{
		printf("cannot open mcnp file\n");
		TCanvas *Cinfo=new TCanvas("Cinfo","Cinfo");
		TPaveText *info_text=new TPaveText(0.,0.3,1,0.8);
		sprintf(w0,"cannot open %s",mcnp_file_name);
		info_text->AddText(w0);	
		info_text->SetBorderSize(0);
		info_text->SetFillColor(0);
		info_text->SetTextColor(2);
		info_text->Draw();
		Cinfo->Update();	
		}
	else
	{
	// first line gives the MCNP version number 
	St->Gets(mcnp_file);
	printf("\n");
	fNphoton=0;
	do
		{
		St->Gets(mcnp_file);
	
		// In the input file, a '&' char at the end of the line means that the
		// line reading is not finished and have to be continued with
		// the next line: all lines being linked by '&' have to be
		// concatenated together. One should concatenate lines until
		// there is no more the '&' char.
		if(St->Contains("&"))
			{
			// Initialize St_ with the first line.
			St_->Replace(0,St_->Sizeof(),"");
			St_->Replace(0,St->Sizeof(),*St);
			St_->ReplaceAll("&"," ");
			do
				{
				St->Gets(mcnp_file);
				// remove the first 13th lines before concatenating
				St->Replace(0,13,"");
				if(St->Contains("&"))
					St->ReplaceAll("&"," ");
				else
					break;
				// concatenate strings
				St_->Append(*St,St_->Sizeof());
				}
			while(1);
			// concatenate the last string
			St_->Append(*St,St_->Sizeof());
			
			St->Replace(0,St_->Sizeof(),*St_);
			}
			
		if(feof(mcnp_file))
			break;

		if(!FINISHED_READING_SOURCE_CODE)
			{
// A key word starting with '1' placed at the beginning of the line means that the
// input file reading is finished, the tallies and energy/time cards can be 
// linked together. Tallies and card having the same ID can be linked together.
// Cards with ID 0, if defined, are used if no specific card can be linked to 
// a tally (exepted for F8 tallies which do not allow time cards.). 
			if(St->Data()[0]=='1'&&St->Data()[1]!='m'&&St->Data()[2]!='c'&&St->Data()[3]!='n')
				{
				FINISHED_READING_SOURCE_CODE=1;
				fNTally_Found=NTH2D;
				for(n=0;n<NTH2D;n++)
					{
					/*look which energy cut corresponds to the tally*/
					for(e=0,edef=NULL;e<NEdef;e++)
						{
						Edef=new MTArrayD(Edef_line[e]);
						if(Edef->GetID()==ftally_id[n])
							edef=Edef;
						}
					/*if no energy cut has been found, search for E0 energy definition*/
					// E0 is the default energy axis cut if no energy card has been defined specifically
					// for this tally
					if(edef==NULL)
					for(e=0;e<NEdef;e++)
						{
						Edef=new MTArrayD(Edef_line[e]);
						if(Edef->GetID()==0)
							edef=Edef;
						}
					if(edef==NULL)
						edef=Array1D;  // If no card found for this tally, the MTArray with 
									  // ID=-1 will inform that the tally does not depend
									  // on energy
					/*look for the time definition corresponding to the tally*/
					for(t=0,tdef=NULL;t<NTdef;t++)
						{
						Tdef=new MTArrayD(Tdef_line[t]);
						if(Tdef->GetID()==ftally_id[n])
							tdef=Tdef;
						}
					/*if no time definition found, search for T0 definition*/
					// T0 is the default time axis cut if no time card has been defined specifically
					// for this tally
					if(tdef==NULL)
					for(t=0;t<NTdef;t++)
						{
						Tdef=new MTArrayD(Tdef_line[t]);
						if(Tdef->GetID()==0)
							tdef=Tdef;
						}
					if(ftally_id[n]%10==8||tdef==NULL)
						tdef=Array1D;	// If no card found for this tally, the MTArray with ID=-1 will inform that the tally does not depend of time
					TallyType=ftally_id[n]%10;
					//to remove the ".o" from the name
					sprintf(w0,"%s",fMCNP_File_name);
					w0[(Int_t)strlen(w0)-2]='\0';
					sprintf(MTH2Dname[n],"%s_F%d%s",w0,ftally_id[n],(fparticle[n]==1)?("N"):("P"));
					// result of type 1 or 2 tally is function of source particle.
					if(TallyType==1||TallyType==2)
							sprintf(counts,"Counts (%s^{-1})",Source_Particle_S);
					if(TallyType==4||TallyType==2)
							{
							// by default, the unit is source particle per surface unit
							sprintf(counts,"Counts (cm^{-2}#upoint %s^{-1})",Source_Particle_S);
							// If a SD card has been defined for a tally of type 4 or 2
							// the unit is source particle per volume unit
							for(sd=0;sd<NSDdef;sd++)
							if(SDdef[sd]==ftally_id[n])
								sprintf(counts,"Counts (cm^{-3}#upoint %s^{-1})",Source_Particle_S);
							}
					if(TallyType==6)
							{
							// by default, the unit is source particle per energy unit
							sprintf(counts,"Counts (MeV#upoint g^{-1}#upoint %s^{-1})",Source_Particle_S);
							THERE_IS_DF_CARD=0;
							for(sd=0;sd<NDFdef;sd++)
							if(DFdef[sd]==ftally_id[n])
								THERE_IS_DF_CARD=1;
							// If a DF card has been defined, a dose estimate is asked for
							// this tally, no time or energy card (scalar information).
							if(THERE_IS_DF_CARD)
								{edef=Array1D;tdef=Array1D;}
							}
					if(TallyType==5)
							{
							// by default, the unit is source particle per surface unit
							sprintf(counts,"Counts (cm^{-2}#upoint %s^{-1})",Source_Particle_S);
							THERE_IS_DF_CARD=0;							
							for(sd=0;sd<NDFdef;sd++)
							if(DFdef[sd]==ftally_id[n])
								THERE_IS_DF_CARD=1;
							// If a DF card has been defined, a dose estimate is asked for
							// this tally, no time or energy card (scalar information).
							if(THERE_IS_DF_CARD)
								{edef=Array1D;tdef=Array1D;}
							}
					if(TallyType==8)
						sprintf(counts,"Counts (%s^{-1})",Source_Particle_S);
					Tally=new MTH2D(edef,tdef,MTH2Dname[n],"Energy (MeV)","Time (shake)",counts);
					}
				}
			// The line number is written at the beginning of each line of the MCNP output file.
			// Finding the '-' character at column #5 means we are still reading the part of the file that 
			// recalls the source file.
			if(St->Data()[5]=='-')
				{
				St_->Replace(0,St->Sizeof(),*St);
				// remove the first 13th char (they are used to indicate the line number)
				St_->Replace(0,13,"");
				St_->ReplaceAll(" ","");
				// If an energy card is identified,
				// create the corresponding MTArray
				if(St_->BeginsWith("E",TString::kIgnoreCase))
					{
					St_->Replace(0,1,"");
					// if the char that follows 'E' is numerical, the line is really 
					// related to a E card. 
					if((St_->Atoi()||(!St_->Atoi()&&St_->Data()[0]=='0')))
						{
						printf("%s\n",St->Data());
						sprintf(Edef_line[NEdef],"%s",(char*)St->Data());
						NEdef++;
						}
					}
				// If a Time card is identified,
				// create the corresponding MTArray
				if(St_->BeginsWith("T",TString::kIgnoreCase))
					{
					St_->Replace(0,1,"");
					// if the char that follows 'T' is numerical, the line is really 
					// related to a T card. 
					if((St_->Atoi()||(!St_->Atoi()&&St_->Data()[0]=='0')))
						{
						printf("%s\n",St->Data());
						sprintf(Tdef_line[NTdef],"%s",(char*)St->Data());
						NTdef++;
						}
					}
				// a tally card has been identified
				if(St_->BeginsWith("F",TString::kIgnoreCase))
					{
					St_->Replace(0,1,"");
					// if the char that follows 'F' is numerical, the line is really 
					// related to a F card. 
					if((St_->Atoi()||(!St_->Atoi()&&St_->Data()[0]=='0')))
						{
						printf("%s\n",St->Data());
						id=St_->Atoi();
						ftally_id[NTH2D]=id;
						sprintf(w1,"%d",id);
						if(St_->Contains(":N",TString::kIgnoreCase))
							fparticle[NTH2D]=1;
						if(St_->Contains(":P",TString::kIgnoreCase))
							fparticle[NTH2D]=2;
						if(St_->Contains(":E",TString::kIgnoreCase))
							fparticle[NTH2D]=3;
						if(St_->Contains(":E",TString::kIgnoreCase)||St_->Contains(":P",TString::kIgnoreCase))
							fNphoton++;
						NTH2D++;
						}
					}
				// SD card
				if(St_->BeginsWith("SD",TString::kIgnoreCase)&&!St_->BeginsWith("SDEF",TString::kIgnoreCase))
					{
					printf("%s\n",St->Data());
					St_->Replace(0,2,"");
					SDdef[NSDdef]=St_->Atoi();
					NSDdef++;
					}
				// SDEF card, idenfy the source particle (keyword "PAR")
				// If no "PAR" keyword, default is neutron
				if(St_->BeginsWith("SDEF",TString::kIgnoreCase))
					{
					printf("%s\n",St->Data());
					if(St_->Contains("par",TString::kIgnoreCase))
						{
						do
							{
							if(St_->BeginsWith("par",TString::kIgnoreCase))
								break;
							St_->Replace(0,1,"");
							}
						while(1);
						do
							{
							if(St_->BeginsWith("=",TString::kIgnoreCase))
								break;
							St_->Replace(0,1,"");
							}
						while(1);
						St_->Replace(0,1,"");
						Source_Particle_I=St_->Atoi();
						if(Source_Particle_I==3)
							sprintf(Source_Particle_S,"e^{-}");
						if(Source_Particle_I==2)
							sprintf(Source_Particle_S,"#gamma");
						if(Source_Particle_I==1)
							sprintf(Source_Particle_S,"neutron");
						}
					else
						sprintf(Source_Particle_S,"neutron");
					}
				// DF card
				if(St_->BeginsWith("DF",TString::kIgnoreCase))
					{
					printf("%s\n",St->Data());
					St_->Replace(0,2,"");
					DFdef[NDFdef]=St_->Atoi();
					NDFdef++;
					}
				}/*if(w0[strlen(w0)-1]=='-')*/
			}/*if(!FINISHED_READING_SOURCE_CODE)*/
		// The calculation result related to a tally is displayed after the line starting with the keyword "1tally"
		// followed by the tally ID and "nps".
		if(FINISHED_READING_SOURCE_CODE)
		if(St->BeginsWith("1tally"))
			{
			if(St->Contains("nps"))
				{
				//remove "1tally" to get the ID which appears just after with TString::Atoi()
				St->Replace(0,6," ");
				id=St->Atoi();
				for(t=0;t<NTH2D;t++)
				if(ftally_id[t]==id)
					{
					Tally=(MTH2D*)gROOT->FindObject(MTH2Dname[t]);
					if(Tally!=NULL)
						Tally->Fill_From_MCNP_File(mcnp_file);
					else
						printf("%s=NULL\n",MTH2Dname[t]);
					}
				}
			}
		}
	while(1);
	fclose(mcnp_file);
// Display the tallies that have been identified.
	if(SHOW)
		ShowTallyButtons();
	}// end of "if(mcnp_file!=NULL)"

	};
// end of MCNP_run(char *mcnp_file_name,MTH2D_Controller *F=NULL,Bool_t SHOW=1)

void ShowTallyButtons()
	{
	Int_t n;
	char *w0;w0=new char[400];
	Int_t option;

	fMainFrame= new TGMainFrame(gClient->GetRoot(),400,200);
	sprintf(w0,"%s",fMCNP_File_name);w0[(Int_t)strlen(fMCNP_File_name)-3]='\0';
	fMainFrame->SetName(w0);fMainFrame->SetWMSize(250,100);

	for(n=0;n<fNTally_Found;n++)
		{	
		sprintf(w0,"F%d%s",ftally_id[n],(fparticle[n]==1)?("N"):("P"));
		TGHorizontalFrame *H_frame=new TGHorizontalFrame(fMainFrame,400,200);
		TGButton *button=new TGTextButton(H_frame,w0);
		sprintf(w0,"Draw_Tally(=%d)",n);
		button->Connect("Clicked()","MCNP_run",this,w0);
		if(fparticle[n]==1)
			H_frame->AddFrame(button, new TGLayoutHints(kLHintsLeft,5,5,5,5));
		else
			H_frame->AddFrame(button, new TGLayoutHints(kLHintsRight,5,5,5,5));
		fMainFrame->AddFrame(H_frame, new TGLayoutHints(kLHintsCenterX,2,2,2,2));
		}

	option=0;
	


	sprintf(w0,"%s",fMCNP_File_name);w0[(Int_t)strlen(fMCNP_File_name)-2]='\0';
	fMainFrame->SetWindowName(w0);
	
	fMainFrame->MapSubwindows();
	fMainFrame->Resize(fMainFrame->GetDefaultSize());
	fMainFrame->MapWindow();		
	};
// end of ShowTallyButtons()
	


~MCNP_run()
	{
	delete fMainFrame;
	//delete []fHframe_option;
	delete fMCNP_File_name;
	delete fAdd_Text;
	};

void Draw_Tally(Int_t n)
	{
	if(n<50)
		{
		TH2D *H=(TH2D*)gROOT->FindObject(MTH2Dname[n]);
		if(H!=NULL)
			{
			MTH2D *Tally=new MTH2D(*H);
			Tally->SetMTH2D_Controller(fControler);
			Tally->DoDraw();
			}
		}
	};

MTH2D* Get_Tally(Int_t n)
	{
	if(n<50)
		{
		MTH2D *Tally;
		Tally=(MTH2D*)gROOT->FindObject(MTH2Dname[n]);
		return Tally;
		}
	else
		{
		printf("Tally number greater than 50\n");
		return 0;
		}
	};
char* Get_TallyName(Int_t n)
	{
	if(n<50)
		return MTH2Dname[n];
	else
		{
		printf("Tally number greater than 50\n");
		return 0;
		}
	};
	
};

ClassImp(MCNP_run)


/*************************************************************************/
/*************************************************************************/
/*********	MTH2D_Controller	**********************************/

// Fot the use of GUI.
// Allow to explore the folders and to click on the MCNP ouput files
// This class has been built starting from an example shown in ROOT tutorials.
class MTH2D_Controller {

RQ_OBJECT("MTH2D_Controller")
private:
char *PlotsnASCIIsDirectory;
//_P
TGTextEntry *fPeaksDataBase_Entry;
char **fReaction_Name;
Double_t **fReaction_PeaksEnergy;
TGTextBuffer *fPeaksDataBase_TextBuffer;
Double_t **fReaction_PeaksAmplitude;
Int_t *fReaction_NumberOfPeaks;
Int_t fNReaction;
//_P
Bool_t fUSE_OnlyGaussianSmearing;
TGTextEntry *fWhereToStart_Entry;
TGTextEntry *fResponseFunction_Entry;
TGCheckButton*fSaveAsOption_ps;
TGCheckButton*fSaveAsOption_eps;
TGCheckButton*fSaveAsOption_pdf;
TGCheckButton*fSaveAsOption_svg;
TGCheckButton*fSaveAsOption_gif;
TGCheckButton*fSaveAsOption_C;
TGCheckButton*fSaveAsOption_root;
TGCheckButton*fSaveAsOption_xml;
TGCheckButton*fSaveAsOption_png;
TGCheckButton*fSaveAsOption_xpm;
TGCheckButton*fSaveAsOption_jpg;
TGCheckButton*fSaveAsOption_tiff;
TGCheckButton*fSaveAsOption_xcf;
TGCheckButton*fSaveAsOption_txt;
TGCheckButton*fSaveAsOption_MTH2D;
TGCheckButton* fCanvasToBeSaved_E;
TGCheckButton* fCanvasToBeSaved_T;
TGCheckButton* fCanvasToBeSaved_e;
TGCheckButton* fCanvasToBeSaved_t;
TGCheckButton* fCanvasToBeSaved_EvsT;
TGCheckButton* fCanvasToBeSaved_C6;
TGCheckButton* fCanvasToBeSaved_All;
char **fSaveAs;
char *fFirstDirectoryPosition;

   TGTransientFrame *fMain;
   TGFileContainer  *fContents;
   TGPopupMenu      *fMenu;
   TGNumberEntry *fSmearingEntry_GTB;
   TGTextEntry *fPlotsnASCII_Entry;
   TGTextBuffer *fPlotsnASCIIs_TextBuffer;

public:
virtual ~MTH2D_Controller()
{
   // Cleanup.

   delete fContents;
   delete fMenu;
   fMain->DeleteWindow();  // deletes fMain
};


   // slots
   Bool_t USE_OnlyGaussianSmearing(){return fUSE_OnlyGaussianSmearing;};
   char* GetSaveAs(Int_t s){return fSaveAs[s];};
   char* GetPlotsnASCIIsDirectory()
	{
	char *w0;w0=new char[400];
	int c;
	sprintf(w0,"%s",fPlotsnASCII_Entry->GetText());
// The '\' character may cause problems, in Linux environment for instance.
// Thus all '\' characters (ascii number=92) are replaced by the harmless 
// '/' characters (ascii number=47).
	for(c=0;c<(Int_t)strlen(w0);c++)
		if((Int_t)w0[c]==92)w0[c]=(char)47;
	fPlotsnASCII_Entry->SetText(w0);
	sprintf(w0,"%s",fPlotsnASCII_Entry->GetText());
	return w0;
	};



//_P
   char* GetfPeaksDataBase_Entry()
	{
	char *w0;w0=new char[400];
	int c;
	sprintf(w0,"%s",fPeaksDataBase_Entry->GetText());
// The '\' character may cause problems, in Linux environment for instance.
// Thus all '\' characters (ascii number=92) are replaced by the more harmless 
// '/' characters (ascii number=47).
	for(c=0;c<(Int_t)strlen(w0);c++)
		if((Int_t)w0[c]==92)w0[c]=47;
	fPeaksDataBase_Entry->SetText(w0);
	sprintf(w0,"%s",fPeaksDataBase_Entry->GetText());
	return w0;
	};

void Build_Reaction_Peaks(Int_t h)
	{
	TH1D *H,*H_FirstEscape,*H_SecondEscape;
	char *w0;w0=new char[200];
	Int_t p;
	Double_t previous_bin_content;
	if(h>-1&&h<fNReaction)
		{
		H=new TH1D(fReaction_Name[h],fReaction_Name[h],24000,0,12);
		for(p=0;p<fReaction_NumberOfPeaks[h];p++)
			{
			previous_bin_content=H->GetBinContent((Int_t)(2*fReaction_PeaksEnergy[h][p]*1000));
			H->Fill(fReaction_PeaksEnergy[h][p],fReaction_PeaksAmplitude[h][p]+previous_bin_content);
			}
		sprintf(w0,"%s_FirstEscape",fReaction_Name[h]);
		H_FirstEscape=new TH1D(w0,w0,24000,0,12);
		for(p=0;p<fReaction_NumberOfPeaks[h];p++)
		if(fReaction_PeaksEnergy[h][p]>2*0.511)
			{
			H_FirstEscape->Fill(fReaction_PeaksEnergy[h][p]-0.511,fReaction_PeaksAmplitude[h][p]);
			printf("%lf\n",fReaction_PeaksAmplitude[h][p]);
			}
		sprintf(w0,"%s_SecondEscape",fReaction_Name[h]);
		H_SecondEscape=new TH1D(w0,w0,24000,0,12);
		for(p=0;p<fReaction_NumberOfPeaks[h];p++)
		if(fReaction_PeaksEnergy[h][p]>2*0.511)
			H_SecondEscape->Fill(fReaction_PeaksEnergy[h][p]-2*0.511,fReaction_PeaksAmplitude[h][p]);
		}
	};

void Build_Reaction_Peaks2(Int_t h,Double_t Emin=0,Double_t Emax=100)
	{
	TH1D *H;
	char *w0;w0=new char[200];
	Double_t *EnergyBins;EnergyBins=new Double_t[50000];
	Int_t NbinsE,b=1;
	Int_t p;

	if(h>-1&&h<fNReaction)
		{
		EnergyBins[0]=0;
		for(p=0,NbinsE=1;p<fReaction_NumberOfPeaks[h];p++)
		if(fReaction_PeaksEnergy[h][p]>Emin&&fReaction_PeaksEnergy[h][p]<Emax)
			{
			EnergyBins[NbinsE++]=fReaction_PeaksEnergy[h][p];
			EnergyBins[NbinsE++]=fReaction_PeaksEnergy[h][p];
			}
		// The last has to be given two times, else there is a display bug
		// (after the last bin, a line appears at the level of the last bin
		// content)
		if(fReaction_PeaksEnergy[h][p-1]>Emin&&fReaction_PeaksEnergy[h][p-1]<Emax)
			{
			EnergyBins[NbinsE++]=fReaction_PeaksEnergy[h][p-1];
			EnergyBins[NbinsE++]=fReaction_PeaksEnergy[h][p-1];
			}
		H=new TH1D(fReaction_Name[h],fReaction_Name[h],NbinsE-1,EnergyBins);
		for(p=0;p<=fReaction_NumberOfPeaks[h];p++)
		for(b=1;b<=NbinsE;b++)
		if(H->GetBinCenter(b)>fReaction_PeaksEnergy[h][p]-0.001&&H->GetBinCenter(b)<fReaction_PeaksEnergy[h][p]+0.001)
			{
			H->SetBinContent(b,fReaction_PeaksAmplitude[h][p]);
			break;
			}
		H->SetBinContent(b,0);

		EnergyBins[0]=0;
		for(p=0,NbinsE=1;p<fReaction_NumberOfPeaks[h];p++)
		if(fReaction_PeaksEnergy[h][p]>1.1)
		if(fReaction_PeaksEnergy[h][p]-0.511>Emin&&fReaction_PeaksEnergy[h][p]-0.511<Emax)
			{
			EnergyBins[NbinsE++]=fReaction_PeaksEnergy[h][p]-0.511;
			EnergyBins[NbinsE++]=fReaction_PeaksEnergy[h][p]-0.511;
			}
		if(NbinsE)
			{
			// The last has to be given two times, else there is a display bug
			// (after the last bin, a line appears at the level of the last bin
			// content)
			if(fReaction_PeaksEnergy[h][p-1]>1.1)
			if(fReaction_PeaksEnergy[h][p-1]-0.511>Emin&&fReaction_PeaksEnergy[h][p-1]-0.511<Emax)
				{
				EnergyBins[NbinsE++]=fReaction_PeaksEnergy[h][p-1]-0.511;
				EnergyBins[NbinsE++]=fReaction_PeaksEnergy[h][p-1]-0.511;
				}
			sprintf(w0,"%s_FirstEscape",fReaction_Name[h]);
			H=new TH1D(w0,w0,NbinsE-1,EnergyBins);
			for(p=0;p<=fReaction_NumberOfPeaks[h];p++)
			if(fReaction_PeaksEnergy[h][p]>1.1)
			for(b=1;b<=NbinsE;b++)
			if(H->GetBinCenter(b)>fReaction_PeaksEnergy[h][p]-0.512&&H->GetBinCenter(b)<fReaction_PeaksEnergy[h][p]-0.510)
				{
				H->SetBinContent(b,fReaction_PeaksAmplitude[h][p]);
				break;
				}
			}
		
		EnergyBins[0]=0;
		for(p=0,NbinsE=1;p<fReaction_NumberOfPeaks[h];p++)
		if(fReaction_PeaksEnergy[h][p]>1.5)
		if(fReaction_PeaksEnergy[h][p]-2*0.511>Emin&&fReaction_PeaksEnergy[h][p]-2*0.511<Emax)
			{
			EnergyBins[NbinsE++]=fReaction_PeaksEnergy[h][p]-2*0.511;
			EnergyBins[NbinsE++]=fReaction_PeaksEnergy[h][p]-2*0.511;
			}
		if(NbinsE)
			{
			// The last has to be given two times, else there is a display bug
			// (after the last bin, a line appears at the level of the last bin
			// content)
			if(fReaction_PeaksEnergy[h][p-1]>1.5)
			if(fReaction_PeaksEnergy[h][p-1]-2*0.511>Emin&&fReaction_PeaksEnergy[h][p-1]-2*0.511<Emax)
				{
				EnergyBins[NbinsE++]=fReaction_PeaksEnergy[h][p-1]-2*0.511;
				EnergyBins[NbinsE++]=fReaction_PeaksEnergy[h][p-1]-2*0.511;
				}
			sprintf(w0,"%s_SecondEscape",fReaction_Name[h]);
			H=new TH1D(w0,w0,NbinsE-1,EnergyBins);
			for(p=0;p<=fReaction_NumberOfPeaks[h];p++)
			for(b=1;b<=NbinsE;b++)
			if(H->GetBinCenter(b)>fReaction_PeaksEnergy[h][p]-2*0.512&&H->GetBinCenter(b)<fReaction_PeaksEnergy[h][p]-2*0.510)
				{
				H->SetBinContent(b,fReaction_PeaksAmplitude[h][p]);
				break;
				}
			}
		}
	};

Int_t GetNReaction()
	{
	return fNReaction;
	};

char* GetReaction_Name(Int_t r)
	{
	return fReaction_Name[r];
	};

Double_t GetEnergyOfReaction(Int_t r,Int_t e)
	{
	if(e<fReaction_NumberOfPeaks[r])
		return fReaction_PeaksEnergy[r][e];
	else
		return 0.;
	};

Double_t GetAmplitudeOfReaction(Int_t r,Int_t e)
	{
	if(e<fReaction_NumberOfPeaks[r])
		return fReaction_PeaksAmplitude[r][e];
	else
		return 0.;
	};

Int_t GetNumberOfPeaks(Int_t r)
	{
	return fReaction_NumberOfPeaks[r];
	};
//_P

   Double_t Get_GTBParameter()
	{return fSmearingEntry_GTB->GetNumber();};

	
//_P
void Fill_ElementPeaks()
	{
	FILE *file;
	Int_t c,p,MaxReactions=300,c_line,c_w0;
	char *w0;w0=new char[400];
	Double_t E,A;
	TString *St,*St_;
	St=new TString();
	St_=new TString();
	fNReaction=0;
	// The peaks database files should be built as follow:
	// first line: 		"Name of the reaction"
	// second line: 	"gamma-ray peak energy#1"	"associated production cross-section"	
	// third line: 		"gamma-ray peak energy#2"	"associated production cross-section"	
	// ...	
	// Nth line:		"Name of the reaction"	
	// Nth+1 line: 		"gamma-ray peak energy#1"	"associated production cross-section"	
	// Nth+2 line: 		"gamma-ray peak energy#2"	"associated production cross-section"	
	// ...	
	// Note that the gamma-ray production cross sections associated to one element need not to be absolute,
	// relative cross sections works as well.
	file=fopen(GetfPeaksDataBase_Entry(),"rt");
	if(file!=NULL)
		{
		fNReaction=MaxReactions;
		fReaction_Name=(char**)calloc(fNReaction,sizeof(char*));
		fReaction_PeaksEnergy=(Double_t**)calloc(fNReaction,sizeof(Double_t*));
		fReaction_PeaksAmplitude=(Double_t**)calloc(fNReaction,sizeof(Double_t*));
		fReaction_NumberOfPeaks=(Int_t*)calloc(fNReaction,sizeof(Int_t));
		for(c=0;c<fNReaction;c++)
			{
			fReaction_Name[c]=(char*)calloc(1000,sizeof(char));
			fReaction_PeaksEnergy[c]=(Double_t*)calloc(1000,sizeof(Double_t));
			fReaction_PeaksAmplitude[c]=(Double_t*)calloc(1000,sizeof(Double_t));
			}
		fNReaction=0;
		// first line tells MODAR it is a peaks database file
		St->Gets(file);
		if(St->CompareTo("//MODAR_PEAKS_DATABASE")&&St->CompareTo("//MODAR_PEAKS_DATABASE\r")&&St->CompareTo("//MODAR_PEAKS_DATABASE\n")&&St->CompareTo("//MODAR_PEAKS_DATABASE\r\n"))
			{
			printf("!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!\n");
			printf("!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!\n");
			printf("%s is not a peaks data base file\n",GetfPeaksDataBase_Entry());
			printf("The first line of the file must be\n");
			printf("//MODAR_PEAKS_DATABASE\n");
			printf("!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!\n");
			printf("!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!\n");
			}
		do
			{
			St->Gets(file);
			if(feof(file))
				break;
			St_->Replace(0,St_->Sizeof(),"");
			St_->Replace(0,St->Sizeof(),*St);
			// remove all spaces, points, replace \r and \n
			St_->ReplaceAll(" ","");St_->ReplaceAll(".","");
			St_->ReplaceAll("\r\n","\0");
			St_->ReplaceAll("\n","\0");St_->ReplaceAll("\r","\0");
			St->ReplaceAll("\r\n","\0");
			St->ReplaceAll("\n","\0");St->ReplaceAll("\r","\0");
			// if there is something else than numerical values, that means 
			// a new gamma-ray production reaction is found -> create the associated histogram
			if(!St_->IsFloat())
				{
				sprintf(fReaction_Name[fNReaction],"%s",St->Data());
				delete gROOT->FindObject(fReaction_Name[fNReaction]);
				sprintf(w0,"%s_FirstEscape",fReaction_Name[fNReaction]);
				delete gROOT->FindObject(w0);
				sprintf(w0,"%s_SecondEscape",fReaction_Name[fNReaction]);
				delete gROOT->FindObject(w0);
				fNReaction++;
				if(fNReaction>=MaxReactions)
					break;
				}	
			// if only numerical values, the line gives a gamma-ray cross production section
			// -> Fill the associated histogram
			else
				{
				for(c_line=0,c_w0=0;c_line<(Int_t)strlen(St->Data());c_line++)
					{
					if(St->Data()[c_line]!=' ')
						w0[c_w0++]=St->Data()[c_line];
					else
						break;
					}
				w0[c_w0]='\0';
				E=atof(w0);
				for(c=c_line;c<(Int_t)strlen(St->Data());c++)if(St->Data()[c]!=' '&&St->Data()[c]!='\t'&&St->Data()[c]!=13)break;
				for(c_line=c,c_w0=0;c_line<(Int_t)strlen(St->Data());c_line++)
				if(St->Data()[c_line]!=' '&&St->Data()[c]!='\t'&&St->Data()[c]!=13)
					w0[c_w0++]=St->Data()[c_line];
				else
					break;
				w0[c_w0]='\0';
				A=atof(w0);
				// There 24000 bins between 0 and 12 MeV, E is given in MeV.
				// The peak will be two bins width.
				fReaction_PeaksEnergy[fNReaction-1][fReaction_NumberOfPeaks[fNReaction-1]]=E;
				fReaction_PeaksAmplitude[fNReaction-1][fReaction_NumberOfPeaks[fNReaction-1]]=A;
				fReaction_NumberOfPeaks[fNReaction-1]++;
				}
			}
		while(1);
		fclose(file);
		for(c=0;c<fNReaction;c++)
			Build_Reaction_Peaks2(c);
		}
	// to avoid having fElementPeaks a NULL object, just get some default reactions
	else
		{
		printf("%s not found, using default peaks database\n",GetfPeaksDataBase_Entry());
		fNReaction=50;
		fReaction_Name=(char**)calloc(fNReaction,sizeof(char*));
		fReaction_PeaksEnergy=(Double_t**)calloc(fNReaction,sizeof(Double_t*));
		fReaction_PeaksAmplitude=(Double_t**)calloc(fNReaction,sizeof(Double_t*));
		fReaction_NumberOfPeaks=(Int_t*)calloc(fNReaction,sizeof(Int_t));
		for(c=0;c<fNReaction;c++)
			{
			fReaction_Name[c]=(char*)calloc(80,sizeof(char));
			fReaction_PeaksEnergy[c]=(Double_t*)calloc(100,sizeof(Double_t));
			fReaction_PeaksAmplitude[c]=(Double_t*)calloc(100,sizeof(Double_t));
			}
		c=0;
		p=0;
		sprintf(fReaction_Name[c],"C,fast n");
		fReaction_PeaksEnergy[c][p]=4.439;
		fReaction_PeaksAmplitude[c][p++]=187;
		fReaction_NumberOfPeaks[c++]=p;
		p=0;
		sprintf(fReaction_Name[c],"O,fast n");
		fReaction_PeaksEnergy[c][p]=2.742;
		fReaction_PeaksAmplitude[c][p++]=38;
		fReaction_PeaksEnergy[c][p]=3.089;
		fReaction_PeaksAmplitude[c][p++]=22;
		fReaction_PeaksEnergy[c][p]=3.684;
		fReaction_PeaksAmplitude[c][p++]=58;
		fReaction_PeaksEnergy[c][p]=3.854;
		fReaction_PeaksAmplitude[c][p++]=34;
		fReaction_PeaksEnergy[c][p]=4.439;
		fReaction_PeaksAmplitude[c][p++]=17;
		fReaction_PeaksEnergy[c][p]=6.130;
		fReaction_PeaksAmplitude[c][p++]=330;
		fReaction_PeaksEnergy[c][p]=6.917;
		fReaction_PeaksAmplitude[c][p++]=47;
		fReaction_PeaksEnergy[c][p]=7.117;
		fReaction_PeaksAmplitude[c][p++]=53;
		fReaction_NumberOfPeaks[c++]=p;
		p=0;
		sprintf(fReaction_Name[c],"N,fast n");
		fReaction_PeaksEnergy[c][p]=0.727;
		fReaction_PeaksAmplitude[c][p++]=22;
		fReaction_PeaksEnergy[c][p]=1.632;
		fReaction_PeaksAmplitude[c][p++]=21;
		fReaction_PeaksEnergy[c][p]=2.125;
		fReaction_PeaksAmplitude[c][p++]=26;
		fReaction_PeaksEnergy[c][p]=2.313;
		fReaction_PeaksAmplitude[c][p++]=42;
		fReaction_PeaksEnergy[c][p]=3.684;
		fReaction_PeaksAmplitude[c][p++]=30;
		fReaction_PeaksEnergy[c][p]=4.442;
		fReaction_PeaksAmplitude[c][p++]=54;
		fReaction_PeaksEnergy[c][p]=5.106;
		fReaction_PeaksAmplitude[c][p++]=45;
		fReaction_PeaksEnergy[c][p]=6.743;
		fReaction_PeaksAmplitude[c][p++]=13;
		fReaction_PeaksEnergy[c][p]=7.029;
		fReaction_PeaksAmplitude[c][p++]=32;
		fReaction_NumberOfPeaks[c++]=p;
		p=0;
		sprintf(fReaction_Name[c],"Si,fast n");
		fReaction_PeaksEnergy[c][p]=0.390;
		fReaction_PeaksAmplitude[c][p++]=25;
		fReaction_PeaksEnergy[c][p]=0.585;
		fReaction_PeaksAmplitude[c][p++]=41;
		fReaction_PeaksEnergy[c][p]=0.844;
		fReaction_PeaksAmplitude[c][p++]=10;
		fReaction_PeaksEnergy[c][p]=0.941;
		fReaction_PeaksAmplitude[c][p++]=13;
		fReaction_PeaksEnergy[c][p]=0.975;
		fReaction_PeaksAmplitude[c][p++]=41;
		fReaction_PeaksEnergy[c][p]=0.983;
		fReaction_PeaksAmplitude[c][p++]=25;
		fReaction_PeaksEnergy[c][p]=1.014;
		fReaction_PeaksAmplitude[c][p++]=22;
		fReaction_PeaksEnergy[c][p]=1.589;
		fReaction_PeaksAmplitude[c][p++]=24;
		fReaction_PeaksEnergy[c][p]=1.779;
		fReaction_PeaksAmplitude[c][p++]=1264;
		fReaction_PeaksEnergy[c][p]=2.839;
		fReaction_PeaksAmplitude[c][p++]=59;
		fReaction_PeaksEnergy[c][p]=5.100;
		fReaction_PeaksAmplitude[c][p++]=37;
		fReaction_PeaksEnergy[c][p]=6.879;
		fReaction_PeaksAmplitude[c][p++]=36;
		fReaction_NumberOfPeaks[c++]=p;
		p=0;
		sprintf(fReaction_Name[c],"Fe,fast n");
		fReaction_PeaksEnergy[c][p]=0.411;
		fReaction_PeaksAmplitude[c][p++]=53;
		fReaction_PeaksEnergy[c][p]=0.476;
		fReaction_PeaksAmplitude[c][p++]=50;
		fReaction_PeaksEnergy[c][p]=0.847;
		fReaction_PeaksAmplitude[c][p++]=785;
		fReaction_PeaksEnergy[c][p]=0.931;
		fReaction_PeaksAmplitude[c][p++]=126;
		fReaction_PeaksEnergy[c][p]=1.038;
		fReaction_PeaksAmplitude[c][p++]=52;
		fReaction_PeaksEnergy[c][p]=1.305;
		fReaction_PeaksAmplitude[c][p++]=76;
		fReaction_PeaksEnergy[c][p]=1.317;
		fReaction_PeaksAmplitude[c][p++]=76;
		fReaction_PeaksEnergy[c][p]=1.408;
		fReaction_PeaksAmplitude[c][p++]=50;
		fReaction_PeaksEnergy[c][p]=1.671;
		fReaction_PeaksAmplitude[c][p++]=36;
		fReaction_PeaksEnergy[c][p]=1.811;
		fReaction_PeaksAmplitude[c][p++]=41;
		fReaction_PeaksEnergy[c][p]=2.113;
		fReaction_PeaksAmplitude[c][p++]=41;
		fReaction_PeaksEnergy[c][p]=2.523;
		fReaction_PeaksAmplitude[c][p++]=21;
		fReaction_PeaksEnergy[c][p]=2.598;
		fReaction_PeaksAmplitude[c][p++]=35;
		fReaction_NumberOfPeaks[c++]=p;
		p=0;
		sprintf(fReaction_Name[c],"Al,fast n");
		fReaction_PeaksEnergy[c][p]=0.472;
		fReaction_PeaksAmplitude[c][p++]=57;
		fReaction_PeaksEnergy[c][p]=0.844;
		fReaction_PeaksAmplitude[c][p++]=153;
		fReaction_PeaksEnergy[c][p]=0.985;
		fReaction_PeaksAmplitude[c][p++]=28;
		fReaction_PeaksEnergy[c][p]=1.014;
		fReaction_PeaksAmplitude[c][p++]=177;
		fReaction_PeaksEnergy[c][p]=1.698;
		fReaction_PeaksAmplitude[c][p++]=30;
		fReaction_PeaksEnergy[c][p]=1.720;
		fReaction_PeaksAmplitude[c][p++]=56;
		fReaction_PeaksEnergy[c][p]=1.809;
		fReaction_PeaksAmplitude[c][p++]=184;
		fReaction_PeaksEnergy[c][p]=2.211;
		fReaction_PeaksAmplitude[c][p++]=145;
		fReaction_PeaksEnergy[c][p]=3.004;
		fReaction_PeaksAmplitude[c][p++]=111;
		fReaction_PeaksEnergy[c][p]=3.204;
		fReaction_PeaksAmplitude[c][p++]=32;
		fReaction_NumberOfPeaks[c++]=p;
		p=0;
		sprintf(fReaction_Name[c],"Cl,fast n");
		fReaction_PeaksEnergy[c][p]=1.219;
		fReaction_PeaksAmplitude[c][p++]=96;
		fReaction_PeaksEnergy[c][p]=1.300;
		fReaction_PeaksAmplitude[c][p++]=42.7;
		fReaction_PeaksEnergy[c][p]=1.727;
		fReaction_PeaksAmplitude[c][p++]=289;
		fReaction_PeaksEnergy[c][p]=1.763;
		fReaction_PeaksAmplitude[c][p++]=108;
		fReaction_PeaksEnergy[c][p]=2.127;
		fReaction_PeaksAmplitude[c][p++]=235;
		fReaction_PeaksEnergy[c][p]=3.163;
		fReaction_PeaksAmplitude[c][p++]=73;
		fReaction_NumberOfPeaks[c++]=p;

		p=0;
		sprintf(fReaction_Name[c],"S,fast n");
		fReaction_PeaksEnergy[c][p]=1.273;
		fReaction_PeaksAmplitude[c][p++]=127;
		fReaction_PeaksEnergy[c][p]=1.720;
		fReaction_PeaksAmplitude[c][p++]=175;
		fReaction_PeaksEnergy[c][p]=2.028;
		fReaction_PeaksAmplitude[c][p++]=115;
		fReaction_PeaksEnergy[c][p]=2.230;
		fReaction_PeaksAmplitude[c][p++]=291;
		fReaction_PeaksEnergy[c][p]=2.770;
		fReaction_PeaksAmplitude[c][p++]=43;
		fReaction_PeaksEnergy[c][p]=4.460;
		fReaction_PeaksAmplitude[c][p++]=94;
		fReaction_NumberOfPeaks[c++]=p;
		fNReaction=c;
		}
	};
//_P


   char* GetDirectory()
	{
	char *directory;directory=new char[400];
	int c;
	sprintf(directory,"%s",fContents->GetDirectory());
	/* change '\' into '/' */
// The '\' character may cause problems, in Linux environment for instance.
// Thus all '\' characters (ascii number=92) are replaced by the harmless 
// '/' characters (ascii number=47).
	for(c=0;c<(Int_t)strlen(directory);c++)
		if((Int_t)directory[c]==92)directory[c]=47;
	return directory;
	};
   char* GetResponseFunction()
	{
	char *w0;w0=new char[400];
	int c;
	sprintf(w0,"%s",fResponseFunction_Entry->GetText());
// The '\' character may cause problems, in Linux environment for instance.
// Thus all '\' characters (ascii number=92) are replaced by the harmless 
// '/' characters (ascii number=47).
	for(c=0;c<(Int_t)strlen(w0);c++)
		if((Int_t)w0[c]==92)w0[c]=47;
	fResponseFunction_Entry->SetText(w0);
	sprintf(w0,"%s",fResponseFunction_Entry->GetText());
	return w0;
	};


// To avoid writting again and again the text fields, a MODAR_LAST_SETUP keep in memory
// the settings used during the last use of MODAR.
// When starting MODAR again, the last settings will be searched in MODAR_LAST_SETUP
MTH2D_Controller(const TGWindow *p, const TGWindow *main, UInt_t w, UInt_t h,Bool_t use_gb=0)
{
int e;
Double_t*Entries;Entries=new Double_t[50];
char *w0;w0=new char[600];
char *plotsnASCII_directory;plotsnASCII_directory=new char[600];
//_P
char *peaksdatabase_file;peaksdatabase_file=new char[600];
//_P
char *WhereToStart;WhereToStart=new char[600];
char *ResponseFunction;ResponseFunction=new char[600];
char *GTB;
GTB=new char[600];
// Default values, in case MODAR_LAST_SETUP exists but does not contain some of the data
sprintf(peaksdatabase_file,"D:/");
sprintf(plotsnASCII_directory,"D:/");
sprintf(WhereToStart,"C:/");
sprintf(ResponseFunction,"D:/");
sprintf(GTB,"0.090");

fUSE_OnlyGaussianSmearing=use_gb;

FILE *file;
// The default options are saved in MODAR_LAST_SETUP.
// This file must be written and read from the place where
// MODAR is called.
// When exiting, MODAR_LAST_SETUP is overwritten, the options which have 
// been used replacing the options that where first loaded.
// When exiting, if writing in MODAR_LAST_SETUP is done with the syntax
// file=fopen("MODAR_LAST_SETUP","wb");, MODAR LAST_SETUP will
// be written in the current working directory, which is not in necessarely 
// the directory from where MODAR has been called (the current working directory is
// the one displayed by the browser and changes when one explores directories.).
// Thus, it is necessary to keep in memory the position from where MODAR is started.
fFirstDirectoryPosition=new char[1200];
sprintf(fFirstDirectoryPosition,"%s",gSystem->WorkingDirectory());
sprintf(w0,"%s/MODAR_LAST_SETUP",fFirstDirectoryPosition);
file=fopen(w0,"rb");
if(file!=NULL)
	{
	do
		{
		if(feof(file))
			break;
		fgets(w0,400*sizeof(char),file);
		w0[(Int_t)strlen(w0)-1]='\0';
		if(!strcmp(w0,"__PlotsnASCII__"))	fgets(plotsnASCII_directory,400*sizeof(char),file);
//_P
		if(!strcmp(w0,"__PeaksDataBase__"))	fgets(peaksdatabase_file,400*sizeof(char),file);
//_P
		if(!strcmp(w0,"__WhereToStart__"))	fgets(WhereToStart,400*sizeof(char),file);
		if(!strcmp(w0,"__ResponseFunction__"))	fgets(ResponseFunction,400*sizeof(char),file);
		if(!strcmp(w0,"__GTB__"))		fgets(GTB,400*sizeof(char),file);
		}
	while(1);
	fclose(file);
	
	if((Int_t)strlen(peaksdatabase_file))
		peaksdatabase_file[(Int_t)strlen(peaksdatabase_file)-1]='\0';
	if((Int_t)strlen(plotsnASCII_directory))
		plotsnASCII_directory[(Int_t)strlen(plotsnASCII_directory)-1]='\0';
	if((Int_t)strlen(WhereToStart))
		WhereToStart[(Int_t)strlen(WhereToStart)-1]='\0';
	if((Int_t)strlen(ResponseFunction))
		ResponseFunction[(Int_t)strlen(ResponseFunction)-1]='\0';
	if((Int_t)strlen(GTB))
		GTB[(Int_t)strlen(GTB)-1]='\0';
	}
else
	{
	sprintf(peaksdatabase_file,"D:/");
	sprintf(plotsnASCII_directory,"D:/");
	sprintf(WhereToStart,"D:/");
	sprintf(ResponseFunction,"D:/");
	sprintf(GTB,"0.090");
	}
// Create transient frame containing a MTH2D_Controller widget.

   TGLayoutHints *lo;

   fMain = new TGTransientFrame(p, main, w, h);
   fMain->Connect("CloseWindow()", "MTH2D_Controller", this, "CloseWindow()");
   fMain->DontCallClose(); // to avoid double deletions.
   // use hierarchical cleaning
   fMain->SetCleanup(kDeepCleanup);

   TGMenuBar* mb = new TGMenuBar(fMain);
   lo = new TGLayoutHints(kLHintsTop | kLHintsLeft | kLHintsExpandX, 0, 0, 1, 1);
   fMain->AddFrame(mb, lo);

   fMenu = mb->AddPopup("&View");
   fMenu->AddEntry("Lar&ge Icons",kLVLargeIcons);
   fMenu->AddEntry("S&mall Icons",kLVSmallIcons);
   fMenu->AddEntry("&List",       kLVList);
   fMenu->AddEntry("&Details",    kLVDetails);
   fMenu->AddSeparator();
   fMenu->AddEntry("&Close",      10);
   fMenu->Connect("Activated(Int_t)","MTH2D_Controller",this,"DoMenu(Int_t)");

   TGListView* lv = new TGListView(fMain, w, h);
   lo = new TGLayoutHints(kLHintsExpandX | kLHintsExpandY);
   fMain->AddFrame(lv,lo);

   Pixel_t white;
   gClient->GetColorByName("white", white);
   fContents = new TGFileContainer(lv, kSunkenFrame,white);
   fContents->Connect("DoubleClicked(TGFrame*,Int_t)", "MTH2D_Controller", this,
                      "OnDoubleClick(TGLVEntry*,Int_t)");

   // position relative to the parent's window
   fMain->CenterOnParent();
   sprintf(w0,"MODAR %.2lf",MODAR_VERSION);
   fMain->SetWindowName(w0);
   fMain->MapSubwindows();
   fMain->MapWindow();
   fContents->SetDefaultHeaders();
   fContents->DisplayDirectory();
   fContents->AddFile("..");        // up level directory
   fContents->Resize();
   fContents->StopRefreshTimer();   // stop refreshing


//////////////////////////////////////////////////////////////////////////////////////////////////////////
TGHorizontalFrame *hframe_PlotsnASCII = new TGHorizontalFrame(fMain,100,40);
TGLayoutHints *fL2 = new TGLayoutHints(kLHintsCenterY | kLHintsLeft, 2, 2, 2, 2);

fPlotsnASCIIs_TextBuffer=new TGTextBuffer(100);
fPlotsnASCIIs_TextBuffer->AddText(0, plotsnASCII_directory);
fPlotsnASCII_Entry = new TGTextEntry(hframe_PlotsnASCII, fPlotsnASCIIs_TextBuffer);
fPlotsnASCII_Entry->SetWidth(600);
hframe_PlotsnASCII->AddFrame(fPlotsnASCII_Entry, fL2);
TGLabel  *Label_PlotsnASCII= new TGLabel(hframe_PlotsnASCII, "Save As directory");
hframe_PlotsnASCII->AddFrame(Label_PlotsnASCII, fL2);

fMain->AddFrame(hframe_PlotsnASCII, new TGLayoutHints(kLHintsLeft, 2, 2, 2, 2));




//_P
//////////////////////////////////////////////////////////////////////////////////////////////////////////
TGHorizontalFrame *hframe_PeaksDataBase = new TGHorizontalFrame(fMain,100,40);

fPeaksDataBase_TextBuffer=new TGTextBuffer(400);
fPeaksDataBase_TextBuffer->AddText(0,peaksdatabase_file);
fPeaksDataBase_Entry = new TGTextEntry(hframe_PeaksDataBase, fPeaksDataBase_TextBuffer);
fPeaksDataBase_Entry->SetWidth(600);
hframe_PeaksDataBase->AddFrame(fPeaksDataBase_Entry, fL2);
TGLabel  *Label_PeaksDataBase= new TGLabel(hframe_PeaksDataBase, "Peaks Library");
hframe_PeaksDataBase->AddFrame(Label_PeaksDataBase, fL2);
Fill_ElementPeaks();
fMain->AddFrame(hframe_PeaksDataBase, new TGLayoutHints(kLHintsLeft, 2, 2, 2, 2));
//_P


//////////////////////////////////////////////////////////////////////////////////////////////////////////
TGHorizontalFrame *hframe_WhereToStart = new TGHorizontalFrame(fMain,100,40);
TGLayoutHints *fL2_WhereToStart = new TGLayoutHints(kLHintsCenterY | kLHintsLeft, 2, 2, 2, 2);

TGTextBuffer *WhereToStart_TextBuffer=new TGTextBuffer(600);
WhereToStart_TextBuffer->AddText(0,WhereToStart);
fWhereToStart_Entry = new TGTextEntry(hframe_WhereToStart, WhereToStart_TextBuffer);
fWhereToStart_Entry->SetWidth(600);
hframe_WhereToStart->AddFrame(fWhereToStart_Entry, fL2_WhereToStart);
TGLabel  *Label_WhereToStart= new TGLabel(hframe_WhereToStart, "Browser position");
hframe_WhereToStart->AddFrame(Label_WhereToStart, fL2_WhereToStart);

// With Linux it is possible to browse from any directory to the other.
// But tests performed with Windows showed that one cannot browse from D: to C: for example.
// Therefore, the possibility to jump explicitely (without browsing) from one directory to the other
// has been added here, in order to jump from D: to C: for example.
TGTextButton *GoThere_button = new TGTextButton(hframe_WhereToStart,"&change position");
GoThere_button->Connect("Clicked()","MTH2D_Controller",this,"GoThere()");
hframe_WhereToStart->AddFrame(GoThere_button, new TGLayoutHints(kLHintsCenterX,5,5,3,4));


fMain->AddFrame(hframe_WhereToStart, new TGLayoutHints(kLHintsLeft, 2, 2, 2, 2));


//////////////////////////////////////////////////////////////////////////////////////////////////////////
TGHorizontalFrame *hframe_ResponseFunction = new TGHorizontalFrame(fMain,100,40);
TGLayoutHints *fL2_ResponseFunction = new TGLayoutHints(kLHintsLeft | kLHintsTop , 2, 2, 2, 2);

TGTextBuffer *ResponseFunction_TextBuffer=new TGTextBuffer(400);
ResponseFunction_TextBuffer->AddText(0,ResponseFunction);
fResponseFunction_Entry = new TGTextEntry(hframe_ResponseFunction, ResponseFunction_TextBuffer);
fResponseFunction_Entry->SetWidth(600);
hframe_ResponseFunction->AddFrame(fResponseFunction_Entry, fL2_ResponseFunction);
TGLabel  *Label_ResponseFunction= new TGLabel(hframe_ResponseFunction, "Energy response function");
hframe_ResponseFunction->AddFrame(Label_ResponseFunction, fL2_ResponseFunction);

fMain->AddFrame(hframe_ResponseFunction, new TGLayoutHints(kLHintsLeft, 2, 2, 2, 2));

//////////////////////////////////////////////////////////////////////////////////////////////////////////


// Gaussian time broadening parameters (only the standard deviation is needed).
			TGHorizontalFrame *hframe_GTB = new TGHorizontalFrame(fMain,200,40);
			TGLayoutHints *fL2_GTB = new TGLayoutHints(kLHintsCenterY | kLHintsLeft, 2, 2, 2, 2);

			TGLabel  *Label_GTB= new TGLabel(hframe_GTB, "GTB parameter (sigma):");
			hframe_GTB->AddFrame(Label_GTB, fL2_GTB);

			fSmearingEntry_GTB=new TGNumberEntry(hframe_GTB,atof(GTB),9,1,TGNumberFormat::kNESReal,TGNumberFormat::kNEAAnyNumber,TGNumberFormat::kNELLimitMinMax,0.,100.);		
			hframe_GTB->AddFrame(fSmearingEntry_GTB, fL2_GTB);
			TGLabel  *Label_GTB_= new TGLabel(hframe_GTB, "s (shake)");
			hframe_GTB->AddFrame(Label_GTB_, fL2_GTB);

			fMain->AddFrame(hframe_GTB, new TGLayoutHints(kLHintsCenterX,2,2,2,2));




//////////////////////////////////////////////////////////////////////////////////////////////////////////
// Drawing options
// Seems more convenient to display the drawing options in this way rather than with the usual ROOT
// option menu bar.

//_________________________________________________________________________________________
//____________ output format ______________________________________________________________
//_________________________________________________________________________________________
TGHorizontalFrame *hframe_SaveAsOption = new TGHorizontalFrame(fMain,100,40);
hframe_SaveAsOption->SetLayoutManager(new TGMatrixLayout(hframe_SaveAsOption, 1, 16, 1));
TGLayoutHints *lh = new TGLayoutHints(kLHintsLeft | kLHintsTop, 100, 100, 100, 100);

fSaveAs=(char**)calloc(16,sizeof(char*));
for(e=0;e<16;e++)
	fSaveAs[e]=(char*)calloc(15,sizeof(char));
sprintf(fSaveAs[0],"ps");sprintf(fSaveAs[1],"eps");sprintf(fSaveAs[2],"pdf");
sprintf(fSaveAs[3],"svg");sprintf(fSaveAs[4],"gif");sprintf(fSaveAs[5],"C");
sprintf(fSaveAs[6],"root");sprintf(fSaveAs[7],"xml");sprintf(fSaveAs[8],"png");
sprintf(fSaveAs[9],"xpm");sprintf(fSaveAs[10],"jpg");sprintf(fSaveAs[11],"tiff");
sprintf(fSaveAs[12],"xcf");sprintf(fSaveAs[13],"txt");sprintf(fSaveAs[14],"MTH2D");

TGLabel  *Label_SaveAsOption= new TGLabel(hframe_SaveAsOption, "Save As: ");
hframe_SaveAsOption->AddFrame(Label_SaveAsOption, fL2);

e=0;
fSaveAsOption_ps= new TGCheckButton(hframe_SaveAsOption, new TGHotString(fSaveAs[e++]), 2);
fSaveAsOption_ps->SetState(kButtonUp);
hframe_SaveAsOption->AddFrame(fSaveAsOption_ps, lh);

fSaveAsOption_eps= new TGCheckButton(hframe_SaveAsOption, new TGHotString(fSaveAs[e++]), 2);
fSaveAsOption_eps->SetState(kButtonUp);
hframe_SaveAsOption->AddFrame(fSaveAsOption_eps, lh);

fSaveAsOption_pdf= new TGCheckButton(hframe_SaveAsOption, new TGHotString(fSaveAs[e++]), 2);
fSaveAsOption_pdf->SetState(kButtonUp);
hframe_SaveAsOption->AddFrame(fSaveAsOption_pdf, lh);

fSaveAsOption_svg= new TGCheckButton(hframe_SaveAsOption, new TGHotString(fSaveAs[e++]), 2);
fSaveAsOption_svg->SetState(kButtonUp);
hframe_SaveAsOption->AddFrame(fSaveAsOption_svg, lh);

fSaveAsOption_gif= new TGCheckButton(hframe_SaveAsOption, new TGHotString(fSaveAs[e++]), 2);
fSaveAsOption_gif->SetState(kButtonUp);
hframe_SaveAsOption->AddFrame(fSaveAsOption_gif, lh);

fSaveAsOption_C= new TGCheckButton(hframe_SaveAsOption, new TGHotString(fSaveAs[e++]), 2);
fSaveAsOption_C->SetState(kButtonUp);
hframe_SaveAsOption->AddFrame(fSaveAsOption_C, lh);

fSaveAsOption_root= new TGCheckButton(hframe_SaveAsOption, new TGHotString(fSaveAs[e++]), 2);
fSaveAsOption_root->SetState(kButtonUp);
hframe_SaveAsOption->AddFrame(fSaveAsOption_root, lh);

fSaveAsOption_xml= new TGCheckButton(hframe_SaveAsOption, new TGHotString(fSaveAs[e++]), 2);
fSaveAsOption_xml->SetState(kButtonUp);
hframe_SaveAsOption->AddFrame(fSaveAsOption_xml, lh);

fSaveAsOption_png= new TGCheckButton(hframe_SaveAsOption, new TGHotString(fSaveAs[e++]), 2);
fSaveAsOption_png->SetState(kButtonUp);
hframe_SaveAsOption->AddFrame(fSaveAsOption_png, lh);

fSaveAsOption_xpm= new TGCheckButton(hframe_SaveAsOption, new TGHotString(fSaveAs[e++]), 2);
fSaveAsOption_xpm->SetState(kButtonUp);
hframe_SaveAsOption->AddFrame(fSaveAsOption_xpm, lh);

fSaveAsOption_jpg= new TGCheckButton(hframe_SaveAsOption, new TGHotString(fSaveAs[e++]), 2);
fSaveAsOption_jpg->SetState(kButtonUp);
hframe_SaveAsOption->AddFrame(fSaveAsOption_jpg, lh);

fSaveAsOption_tiff= new TGCheckButton(hframe_SaveAsOption, new TGHotString(fSaveAs[e++]), 2);
fSaveAsOption_tiff->SetState(kButtonUp);
hframe_SaveAsOption->AddFrame(fSaveAsOption_tiff, lh);

fSaveAsOption_xcf= new TGCheckButton(hframe_SaveAsOption, new TGHotString(fSaveAs[e++]), 2);
fSaveAsOption_xcf->SetState(kButtonUp);
hframe_SaveAsOption->AddFrame(fSaveAsOption_xcf, lh);

fSaveAsOption_txt= new TGCheckButton(hframe_SaveAsOption, new TGHotString(fSaveAs[e++]), 2);
fSaveAsOption_txt->SetState(kButtonUp);
hframe_SaveAsOption->AddFrame(fSaveAsOption_txt, lh);

fSaveAsOption_MTH2D= new TGCheckButton(hframe_SaveAsOption, new TGHotString(fSaveAs[e++]), 2);
fSaveAsOption_MTH2D->SetState(kButtonUp);
hframe_SaveAsOption->AddFrame(fSaveAsOption_MTH2D, lh);

fMain->AddFrame(hframe_SaveAsOption, new TGLayoutHints(kLHintsCenterX,2,2,2,2));


//_________________________________________________________________________________________
//____________ Canvases to be saved _______________________________________________________
//_________________________________________________________________________________________
// The canvas related to a MTH2D object shows 5 spectra: full Energy, full Time, Energy with cut on Time, 
// Time with cut on Energy and Time versus Energy.
// When saving as gif,eps,C,pdf,..., it may not be necessary to get pictures of all spectra.
// Therefore, the following options allows to select which spectra have to be saved in gif,eps,C,...
// Legend
TGHorizontalFrame *hframe_SpectraToBeSave_Legend = new TGHorizontalFrame(fMain,100,40);
TGLabel  *Label_SpectraToBeSave= new TGLabel(hframe_SpectraToBeSave_Legend, "Spectra to be saved: ");
hframe_SpectraToBeSave_Legend->AddFrame(Label_SpectraToBeSave, fL2);
fMain->AddFrame(hframe_SpectraToBeSave_Legend, new TGLayoutHints(kLHintsCenterX,2,2,2,2));

// selection boxes
TGHorizontalFrame *hframe_SpectraToBeSave = new TGHorizontalFrame(fMain,100,40);
hframe_SpectraToBeSave->SetLayoutManager(new TGMatrixLayout(hframe_SpectraToBeSave, 3, 3, 1));

char **CanvasLegend;CanvasLegend=(char**)calloc(7,sizeof(char*));
for(e=0;e<7;e++)CanvasLegend[e]=(char*)calloc(40,sizeof(char));
sprintf(CanvasLegend[0],"Full E    ");sprintf(CanvasLegend[1],"Full T    ");sprintf(CanvasLegend[2],"TvsE    ");
sprintf(CanvasLegend[3],"E,cut on T");sprintf(CanvasLegend[4],"T,cut on E");sprintf(CanvasLegend[5],"Canvas#6");
sprintf(CanvasLegend[6],"Full Canvas");

fCanvasToBeSaved_E=new TGCheckButton(hframe_SpectraToBeSave, new TGHotString(CanvasLegend[0]), 2);
fCanvasToBeSaved_E->SetState(kButtonDown);
hframe_SpectraToBeSave->AddFrame(fCanvasToBeSaved_E, lh);

fCanvasToBeSaved_T=new TGCheckButton(hframe_SpectraToBeSave, new TGHotString(CanvasLegend[1]), 2);
fCanvasToBeSaved_T->SetState(kButtonDown);
hframe_SpectraToBeSave->AddFrame(fCanvasToBeSaved_T, lh);

fCanvasToBeSaved_e=new TGCheckButton(hframe_SpectraToBeSave, new TGHotString(CanvasLegend[2]), 2);
fCanvasToBeSaved_e->SetState(kButtonDown);
hframe_SpectraToBeSave->AddFrame(fCanvasToBeSaved_e, lh);

fCanvasToBeSaved_t=new TGCheckButton(hframe_SpectraToBeSave, new TGHotString(CanvasLegend[3]), 2);
fCanvasToBeSaved_t->SetState(kButtonDown);
hframe_SpectraToBeSave->AddFrame(fCanvasToBeSaved_t, lh);

fCanvasToBeSaved_EvsT=new TGCheckButton(hframe_SpectraToBeSave, new TGHotString(CanvasLegend[4]), 2);
fCanvasToBeSaved_EvsT->SetState(kButtonDown);
hframe_SpectraToBeSave->AddFrame(fCanvasToBeSaved_EvsT, lh);

fCanvasToBeSaved_C6=new TGCheckButton(hframe_SpectraToBeSave, new TGHotString(CanvasLegend[5]), 2);
fCanvasToBeSaved_C6->SetState(kButtonDown);
hframe_SpectraToBeSave->AddFrame(fCanvasToBeSaved_C6, lh);

fCanvasToBeSaved_All=new TGCheckButton(hframe_SpectraToBeSave, new TGHotString(CanvasLegend[6]), 2);
fCanvasToBeSaved_All->SetState(kButtonDown);
hframe_SpectraToBeSave->AddFrame(fCanvasToBeSaved_All, lh);

fMain->AddFrame(hframe_SpectraToBeSave, new TGLayoutHints(kLHintsCenterX,2,2,2,2));




//////////////////////////////////////////////////////////////////////////////////////////////////////////
TGHorizontalFrame *hframe_Exit = new TGHorizontalFrame(fMain,200,40);
TGTextButton *exit = new TGTextButton(hframe_Exit,"&Exit");
exit->Connect("Clicked()","MTH2D_Controller",this,"Exit()");
hframe_Exit->AddFrame(exit, new TGLayoutHints(kLHintsCenterX,5,5,3,4));

fMain->AddFrame(hframe_Exit, new TGLayoutHints(kLHintsCenterX,2,2,2,2));


fMain->MapSubwindows();
// Initialize the layout algorithm
fMain->Resize(fMain->GetDefaultSize());
// Map main frame
fMain->MapWindow();


   fMain->Resize();

// Check that it is possible to access the files given as entries in the Main frame
// If there is no access to one of the files, tell it
TPaveText *info_text_access_problem=new TPaveText(0.,0.3,1,0.8);
info_text_access_problem->SetTextColor(1);	
info_text_access_problem->SetTextAlign(12);	
info_text_access_problem->SetBorderSize(0);
info_text_access_problem->SetFillColor(0);
if(gSystem->AccessPathName(plotsnASCII_directory,kWritePermission))
	{
	sprintf(w0,"Save As directory:");
	info_text_access_problem->AddText(w0);	
	sprintf(w0,"CANNOT WRITE IN %s",plotsnASCII_directory);
	info_text_access_problem->AddText(w0);	
	info_text_access_problem->SetTextColor(2);	
	}
/*
if(gSystem->AccessPathName(peaksdatabase_file,kReadPermission))
	{
	sprintf(w0,"Peaks Library:");
	info_text_access_problem->AddText(w0);	
	sprintf(w0,"CANNOT ACCESS %s",peaksdatabase_file);
	info_text_access_problem->AddText(w0);	
	info_text_access_problem->AddText("(using default peaks library)");	
	info_text_access_problem->SetTextColor(2);	
	}
*/

if(gSystem->AccessPathName(ResponseFunction,kReadPermission))
	{
	sprintf(w0,"Energy response function:");
	info_text_access_problem->AddText(w0);	
	sprintf(w0,"CANNOT ACCESS %s",ResponseFunction);
	info_text_access_problem->AddText(w0);	
	info_text_access_problem->SetTextColor(2);	
	}

if(info_text_access_problem->GetTextColor()==2)
	{
	TCanvas *Cinfo=new TCanvas("Cinfo","Cinfo");
	info_text_access_problem->Draw();
	Cinfo->Update();
	}

};
// end of MTH2D_Controller(const TGWindow *p, const TGWindow *main, UInt_t w, UInt_t h)


void DoMenu(Int_t mode)
{
   // Switch view mode.

   if (mode<10) {
      fContents->SetViewMode((EListViewMode)mode);
   } else {
      delete this;
   }
};

void DisplayFile(const TString &fname)
{
   // Display content of ROOT file.

   TFile file(fname);
   fContents->RemoveAll();
   fContents->AddFile(gSystem->WorkingDirectory());
   fContents->SetPagePosition(0,0);
   fContents->SetColHeaders("Name","Title");

   TIter next(file.GetListOfKeys());
   TKey *key;

   while ((key=(TKey*)next())) {
      TString cname = key->GetClassName();
      TString name = key->GetName();
      TGLVEntry *entry = new TGLVEntry(fContents,name,cname);
      entry->SetSubnames(key->GetTitle());
      fContents->AddItem(entry);

      // user data is a filename
      entry->SetUserData((void*)StrDup(fname));
   }
   fMain->Resize();
};
// end of void DisplayFile(const TString &fname)

void DisplayDirectory(const TString &fname)
{
   // Display content of directory.

   fContents->SetDefaultHeaders();
   gSystem->ChangeDirectory(fname);
   fContents->ChangeDirectory(fname);
   fContents->DisplayDirectory();
   fContents->AddFile("..");  // up level directory
   fMain->Resize();
};

void DisplayObject(const TString& fname,const TString& name)
{
   // Browse object located in file.

   TDirectory *sav = gDirectory;

   static TFile *file = 0;
   if (file) delete file;     // close
   file = new TFile(fname);   // reopen

   TObject* obj = file->Get(name);
   if (obj) {
      if (!obj->IsFolder()) {
         obj->Browse(0);
      } else obj->Print();
   }
   gDirectory = sav;
};
// end of void MTH2D_Controller::DisplayObject(const TString& fname,const TString& name)

void OnDoubleClick(TGLVEntry *f, Int_t btn)
{
// Handle double click.
if (btn != kButton1) return;

// set kWatch cursor
ULong_t cur = gVirtualX->CreateCursor(kWatch);
gVirtualX->SetCursor(fContents->GetId(), cur);

TString name(f->GetTitle());
const char* fname = (const char*)f->GetUserData();
char *w0; w0=new char[600];

if(fname)
	DisplayObject(fname, name);
else if(name.EndsWith(".o"))
	{
	new MCNP_run(name,this,1==1);
	}
else if(name.EndsWith(".MTH2D"))
	{
	sprintf(w0,"%s",name.Data());
	MTH2D *mth2d=new MTH2D(w0,this);
	mth2d->DoDraw();
	}
else if(name.EndsWith(".C")||name.EndsWith(".cc"))
	{
	FILE *file;
	file=fopen(name.Data(),"rb");
	fscanf(file,"%s",w0);
	// the first line of a response function macro file
	// should start with //MODAR_DETECTOR_RESPONSE_FUNCTION
	// Then, a click on the file will automatically load the 
	// corresponding response function
	if(!strcmp(w0,"//MODAR_DETECTOR_RESPONSE_FUNCTION"))
		{
		sprintf(w0,"%s/%s",gSystem->WorkingDirectory(),name.Data());
		fResponseFunction_Entry->SetText(w0);
		fclose(file);
		}
	else
		{
		fclose(file);
		// To avoid calling again MODAR
		if(!name.BeginsWith("MODAR."))
			gROOT->Macro(name.Data());
		}
	}
else if(name.EndsWith(".txt"))
	{
	FILE *file;
	file=fopen(name.Data(),"rb");
	fscanf(file,"%s",w0);
	// the first line of a peaks database file
	// should start with //MODAR_PEAKS_DATABASE
	// Then, a click on the file will automatically load the 
	// corresponding response function
//_P
	if(!strcmp(w0,"//MODAR_PEAKS_DATABASE"))
		{
		sprintf(w0,"%s/%s",gSystem->WorkingDirectory(),name.Data());
		fPeaksDataBase_Entry->SetText(w0);
		Fill_ElementPeaks();
		}
//_P
	fclose(file);
	}
else
	DisplayDirectory(name);
// set kPointer cursor
cur = gVirtualX->CreateCursor(kPointer);
gVirtualX->SetCursor(fContents->GetId(), cur);
};
// end of void MTH2D_Controller::OnDoubleClick(TGLVEntry *f, Int_t btn)

void CloseWindow()
{
   delete this;
};

// Before leaving, the text entries and numerical values are put into MODAR_LAST_SETUP
// to avoid writting them again the next time MODAR is called
void Exit()
{
FILE *file;
char *w0;w0=new char[400];int c;
sprintf(w0,"%s",fWhereToStart_Entry->GetText());
for(c=0;c<(Int_t)strlen(w0);c++)
	if((Int_t)w0[c]==92)w0[c]=47;
fWhereToStart_Entry->SetText(w0);
// MODAR_LAST_SETUP will be written in the directory from which MODAR has been called
sprintf(w0,"%s/MODAR_LAST_SETUP",fFirstDirectoryPosition);
file=fopen(w0,"wb");
if(file!=NULL)
	{
	fprintf(file,"__PlotsnASCII__\n");
	fprintf(file,"%s\n",fPlotsnASCII_Entry->GetText());
//_P
	fprintf(file,"__PeaksDataBase__\n");
	fprintf(file,"%s\n",fPeaksDataBase_Entry->GetText());
//_P
	fprintf(file,"__WhereToStart__\n");
	fprintf(file,"%s\n",fWhereToStart_Entry->GetText());
	fprintf(file,"__ResponseFunction__\n");
	fprintf(file,"%s\n",fResponseFunction_Entry->GetText());
	fprintf(file,"__GTB__\n");
	fprintf(file,"%lf\n",fSmearingEntry_GTB->GetNumber());
	fclose(file);
	}
gApplication->Terminate(0);
};
// end of void MTH2D_Controller::Exit()

Int_t GetSaveAsOptionStatus(Int_t o)
{
if(o==0) {if(fSaveAsOption_ps->IsDown()) return 1; else return 0;}
if(o==1) {if(fSaveAsOption_eps->IsDown()) return 1; else return 0;}
if(o==2) {if(fSaveAsOption_pdf->IsDown()) return 1; else return 0;}
if(o==3) {if(fSaveAsOption_svg->IsDown()) return 1; else return 0;}
if(o==4) {if(fSaveAsOption_gif->IsDown()) return 1; else return 0;}
if(o==5) {if(fSaveAsOption_C->IsDown()) return 1; else return 0;}
if(o==6) {if(fSaveAsOption_root->IsDown()) return 1; else return 0;}
if(o==7) {if(fSaveAsOption_xml->IsDown()) return 1; else return 0;}
if(o==8) {if(fSaveAsOption_png->IsDown()) return 1; else return 0;}
if(o==9) {if(fSaveAsOption_xpm->IsDown()) return 1; else return 0;}
if(o==10) {if(fSaveAsOption_jpg->IsDown()) return 1; else return 0;}
if(o==11) {if(fSaveAsOption_tiff->IsDown()) return 1; else return 0;}
if(o==12) {if(fSaveAsOption_xcf->IsDown()) return 1; else return 0;}
if(o==13) {if(fSaveAsOption_txt->IsDown()) return 1; else return 0;}
if(o==14) {if(fSaveAsOption_MTH2D->IsDown()) return 1; else return 0;}	
else return 0;
};

Int_t GetSpectraToBeSavedStatus(Int_t o)
{
if(o==0){if(fCanvasToBeSaved_E->IsDown()) return 1; else return 0;}
if(o==1){if(fCanvasToBeSaved_T->IsDown()) return 1; else return 0;}
if(o==2){if(fCanvasToBeSaved_e->IsDown()) return 1; else return 0;}
if(o==3){if(fCanvasToBeSaved_t->IsDown()) return 1; else return 0;}
if(o==4){if(fCanvasToBeSaved_EvsT->IsDown()) return 1; else return 0;}
if(o==5){if(fCanvasToBeSaved_C6->IsDown()) return 1; else return 0;}
if(o==6){if(fCanvasToBeSaved_All->IsDown()) return 1; else return 0;}
else return 0;
};


// The possibility to jump explicitely (without browsing) from one place to the other
// has been added here (text entry to fill).
void GoThere()
{
char *w0;w0=new char[600];
int c;
sprintf(w0,"%s",fWhereToStart_Entry->GetText());
// The '\' character may cause problems, in Linux environment for instance.
// Thus all '\' characters (ascii number=92) are replaced by the harmless 
// '/' characters (ascii number=47).
for(c=0;c<(Int_t)strlen(w0);c++)
	if((Int_t)w0[c]==92)w0[c]=47;
fWhereToStart_Entry->SetText(w0);
if(!gSystem->AccessPathName(fWhereToStart_Entry->GetText(),kReadPermission))
	{
	gSystem->ChangeDirectory(fWhereToStart_Entry->GetText());
	fContents->AddFile(gSystem->WorkingDirectory());
	fContents->ChangeDirectory(fWhereToStart_Entry->GetText());
	fContents->DisplayDirectory();
	fContents->AddFile("..");  // up level directory
	fMain->Resize();
	}
};
// end of void MTH2D_Controller::GoThere()

ClassDef(MTH2D_Controller,1)

};
// end of MTH2D_Controller class

ClassImp(MTH2D_Controller)




//_P

//_P
Int_t MTH2D::ShowElementsPeaks()
{
if(fF==NULL)
	{
	printf("No Main window-> no peaks Database available\n");
	return 0;
	}
Int_t entry,color,Nshow;
char *name;name=new char[400];
TH1D *H,*H_FirstEscape,*H_SecondEscape,*Px;
Double_t A;

if(GetYaxis()->GetNbins()!=1&&GetXaxis()->GetNbins()!=1)
	fC->cd(4)->Clear();
else
	fC->cd()->Clear();

sprintf(name,"%s_pX",GetName());
gROOT->GetObject(name,Px);
Px->Draw("h");
Nshow=2;
TLegend *PeaksLegend=new TLegend(0.8,0.7,1,1);
PeaksLegend->SetFillColor(0);

for(entry=1;entry<=fList_Elements->GetNumberOfEntries();entry++)
if(fList_Elements->GetSelection(entry))
	{

	delete gROOT->FindObject(fF->GetReaction_Name(entry-1));
	sprintf(name,"%s_FirstEscape",fF->GetReaction_Name(entry-1));
	delete gROOT->FindObject(name);
	sprintf(name,"%s_SecondEscape",fF->GetReaction_Name(entry-1));
	delete gROOT->FindObject(name);

	fF->Build_Reaction_Peaks2(entry-1,Px->GetBinLowEdge(1),Px->GetBinLowEdge(Px->GetNbinsX()));
	gROOT->GetObject(fF->GetReaction_Name(entry-1),H);
	sprintf(name,"%s_FirstEscape",fF->GetReaction_Name(entry-1));
	gROOT->GetObject(name,H_FirstEscape);
	sprintf(name,"%s_SecondEscape",fF->GetReaction_Name(entry-1));
	gROOT->GetObject(name,H_SecondEscape);

	A=H->GetMaximum();
	if(A!=0)
		H->Scale(0.9*Px->GetMaximum()/A);
	// avoid yellow line color (not clearly visible)
	(Nshow<5)?(color=Nshow):(color=Nshow+1);
	H->SetLineColor(color);
	H->Draw("same");
	PeaksLegend->AddEntry(H,H->GetName(),"l");
	Nshow++;
	if(GetShow_FirstEscape_Status())
		{
		if(A!=0)
			H_FirstEscape->Scale(0.5*Px->GetMaximum()/A);
		H_FirstEscape->SetLineColor(H->GetLineColor());
		H_FirstEscape->SetLineStyle(3);
		H_FirstEscape->Draw("same");
		}
	if(GetShow_SecondEscape_Status())
		{
		if(A!=0)
			H_SecondEscape->Scale(0.5*Px->GetMaximum()/A);
		H_SecondEscape->SetLineColor(H->GetLineColor());
		H_SecondEscape->SetLineStyle(3);
		H_SecondEscape->Draw("same");
		}
	}
PeaksLegend->Draw();
fC->Update();
return 1;
};
//_P
// end of Int_t MTH2D::ShowElementsPeaks()


///////////////////////////////////////////////////////////////////////////////////////////////
/////// DRAW ALL PROJECTIONS OF THE 2D HISTO AND CREATE gif,eps,jpg,... FILES (IF LEVEL!=0) ///
/////// DRAW ALSO POSITIONS OF THE CUT BOXES AS WELL AS THE HISTO 2D VIEW /////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////
Int_t MTH2D::DoDraw(Int_t level)
	{
	char *name;name=new char[400];
	char *w0;w0=new char[400];
	if(fMain==NULL)
		{
		// Create a main frame
		fMain = new TGMainFrame(gClient->GetRoot(),1200,900);
		// Create canvas widget
		sprintf(name,"_%s",GetName());
		fEcanvas = new TRootEmbeddedCanvas(name,fMain,900,600);
		fMain->AddFrame(fEcanvas, new TGLayoutHints(kLHintsExpandX| kLHintsExpandY,10,10,10,1));
		
		// no need to draw frames and text buttons for zero dimention data.
		if(GetYaxis()->GetNbins()!=1||GetXaxis()->GetNbins()!=1)
			{
			TGLayoutHints *fL2 = new TGLayoutHints(kLHintsCenterY | kLHintsLeft, 2, 2, 2, 2);
//////////////////////////////////////////////////////////////////////////////////////////////////////////
			// Create a horizontal frame widget with buttons	
			TGHorizontalFrame *hframe = new TGHorizontalFrame(fMain,200,40);
			
			TGMenuBar *menu=new TGMenuBar(hframe);
			hframe->AddFrame(menu, new TGLayoutHints(kLHintsCenterX,5,5,3,4));
			TGTextButton *draw = new TGTextButton(hframe,"&Draw");
			draw->Connect("Clicked()","MTH2D",this,"DoDraw()");
			hframe->AddFrame(draw, new TGLayoutHints(kLHintsCenterX,5,5,3,4));
			
			// The directory where to save data is given via fF
			// fF==NULL means fF has been deleted or that the user has created
			// the MTH2D independently of a MTH2D_Controller object (=he does everything by hands) 
			if(fF!=NULL)
				{
				TGTextButton *draw = new TGTextButton(hframe,"&SaveAs");
				draw->Connect("Clicked()","MTH2D",this,"DoDraw(=1)");
				hframe->AddFrame(draw, new TGLayoutHints(kLHintsCenterX,5,5,3,4));
				}
	
			TGTextButton *scale = new TGTextButton(hframe,"&scaling");
			scale->Connect("Clicked()","MTH2D",this,"ScaleWithBinWidth()");
			hframe->AddFrame(scale, new TGLayoutHints(kLHintsCenterX,5,5,3,4));
			

	
			fPlot_Errors= new TGCheckButton(hframe, new TGHotString("No error bars"), 2);
			fPlot_Errors->SetState(kButtonUp);
			hframe->AddFrame(fPlot_Errors, new TGLayoutHints(kLHintsCenterX,5,5,3,4));


			fMain->AddFrame(hframe, new TGLayoutHints(kLHintsCenterX,2,2,2,2));


			if(fF!=NULL&&strcmp(GetZaxis()->GetTitle(),"Counts"))
				{
				TGHorizontalFrame *hframe_GB = new TGHorizontalFrame(fMain,200,40);
	
				TGTextButton *fSetButton = new TGTextButton(hframe_GB, " Smearing with response function", 2);
				fSetButton->Connect("Clicked()","MTH2D",this,"Smear()");
				hframe_GB->AddFrame(fSetButton, fL2);
				fMain->AddFrame(hframe_GB, new TGLayoutHints(kLHintsCenterX,2,2,2,2));
				}

//////////////////////////////////////////////////////////////////////////////////////////////////////////
// It doesn't make sense to inject particles in an histogram having "Counts" as units since it
// has already been built after injecting source particles.
			if(strcmp(GetZaxis()->GetTitle(),"Counts"))
				{
				TGHorizontalFrame *hframe_Inject = new TGHorizontalFrame(fMain,200,40);
	
				fInjEvtsEntries = new TGNumberEntry(hframe_Inject,0,9,1,TGNumberFormat::kNESReal,TGNumberFormat::kNEAAnyNumber,TGNumberFormat::kNELLimitMinMax,0,pow(2,31.));               
				hframe_Inject->AddFrame(fInjEvtsEntries, fL2);
				TGLabel  *Label_Nsrc= new TGLabel(hframe_Inject, "number of source particle");
				hframe_Inject->AddFrame(Label_Nsrc, fL2);
	
				TGTextButton *fSetButton_Inject = new TGTextButton(hframe_Inject, " Inject events ", 2);
				fSetButton_Inject->Connect("Clicked()","MTH2D",this,"Inject()");
				hframe_Inject->AddFrame(fSetButton_Inject, fL2);

				TGTextButton *fSetButton_Scaling = new TGTextButton(hframe_Inject, " Scale ", 2);
				fSetButton_Scaling->Connect("Clicked()","MTH2D",this,"Scaling()");
				hframe_Inject->AddFrame(fSetButton_Scaling, fL2);
				fMain->AddFrame(hframe_Inject, new TGLayoutHints(kLHintsCenterX,2,2,2,2));
				}
//////////////////////////////////////////////////////////////////////////////////////////////////////////
// To define directly the boxes left/right limites with numerical values


			if(GetYaxis()->GetNbins()!=1)
				{
				TGHorizontalFrame *hframe_ETCuts = new TGHorizontalFrame(fMain,200,40);
	
				fETCuts_Entries_0 = new TGNumberEntry(hframe_ETCuts,GetXaxis()->GetBinLowEdge(1),9,1,TGNumberFormat::kNESReal,TGNumberFormat::kNEAAnyNumber,TGNumberFormat::kNELLimitMinMax,-100.,100.);	       
				hframe_ETCuts->AddFrame(fETCuts_Entries_0, fL2);
				TGLabel  *LabelE_cut_min= new TGLabel(hframe_ETCuts, "E cut min");
				hframe_ETCuts->AddFrame(LabelE_cut_min, fL2);
	
				fETCuts_Entries_1 = new	TGNumberEntry(hframe_ETCuts,GetXaxis()->GetBinUpEdge(GetXaxis()->GetNbins()),9,1,TGNumberFormat::kNESReal,TGNumberFormat::kNEAAnyNumber,TGNumberFormat::kNELLimitMinMax,-100.,100.);	       
				hframe_ETCuts->AddFrame(fETCuts_Entries_1, fL2);
				TGLabel  *LabelE_cut_max= new TGLabel(hframe_ETCuts, "E cut max");
				hframe_ETCuts->AddFrame(LabelE_cut_max, fL2);
	
				fETCuts_Entries_2 = new TGNumberEntry(hframe_ETCuts,GetYaxis()->GetBinLowEdge(1),9,1,TGNumberFormat::kNESReal,TGNumberFormat::kNEAAnyNumber,TGNumberFormat::kNELLimitMinMax,-1000.,1000.);	       
				hframe_ETCuts->AddFrame(fETCuts_Entries_2, fL2);
				TGLabel  *LabelTime_cut_min= new TGLabel(hframe_ETCuts, "Time cut min");
				hframe_ETCuts->AddFrame(LabelTime_cut_min, fL2);
	
				fETCuts_Entries_3 = new TGNumberEntry(hframe_ETCuts,GetYaxis()->GetBinUpEdge(GetYaxis()->GetNbins()),9,1,TGNumberFormat::kNESReal,TGNumberFormat::kNEAAnyNumber,TGNumberFormat::kNELLimitMinMax,-1000.,1000.);	       
				hframe_ETCuts->AddFrame(fETCuts_Entries_3, fL2);
				TGLabel  *LabelTime_cut_max= new TGLabel(hframe_ETCuts, "Time cut max");
				hframe_ETCuts->AddFrame(LabelTime_cut_max, fL2);
	
				TGTextButton *SetBoxesLimites = new TGTextButton(hframe_ETCuts,"&Set");
				SetBoxesLimites->Connect("Clicked()","MTH2D",this,"SetBoxesLimites()");
				hframe_ETCuts->AddFrame(SetBoxesLimites, new TGLayoutHints(kLHintsCenterX,5,5,3,4));
	
				fMain->AddFrame(hframe_ETCuts, new TGLayoutHints(kLHintsCenterX,2,2,2,2));
				}
//////////////////////////////////////////////////////////////////////////////////////////////////////////
// Gives the possibility to change time unit to ns,s,ms or s instead of shakes
			if(GetYaxis()->GetNbins()!=1&&fTimeUnit>-1)
				{
				char *fct_call;fct_call=new char[400];
				char **units;units=(char**)calloc(5,sizeof(char*));
				Int_t rb;
				for(rb=0;rb<5;rb++)units[rb]=(char*)calloc(20,sizeof(char));
				sprintf(units[0],"ns");
				sprintf(units[1],"shake");
				sprintf(units[2],"s");
				sprintf(units[3],"ms");
				sprintf(units[4],"s");
						
				TGButtonGroup *TimeUnitsGroup = new TGButtonGroup(fMain, "Time Units",kHorizontalFrame|kLHintsCenterY);

				for(rb=0;rb<5;rb++)
					{
					TGRadioButton *radio=new TGRadioButton(TimeUnitsGroup,new TGHotString(units[rb]));
					sprintf(fct_call,"ChangeTimeUnits(=%d)",rb);
					radio->Connect("Pressed()","MTH2D",this,fct_call);
					if(fTimeUnit==rb)
						radio->SetState(kButtonDown);
					}			
				TimeUnitsGroup->Show();
	
				fMain->AddFrame(TimeUnitsGroup, new TGLayoutHints(kLHintsCenterX,2,2,2,2));
		
				}

//_P
//////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////
// Peaks Database
			TString *S_xaxis_E=new TString(GetXaxis()->GetTitle());
			if(fF!=NULL&&fF->GetNReaction()&&S_xaxis_E->Contains("Energy"))
				{
				TGHorizontalFrame *hframe_List_Elements = new TGHorizontalFrame(fMain,100,40);

				TGTextButton *Button_List_Elements = new TGTextButton(hframe_List_Elements,"&Show Peaks");
				Button_List_Elements->Connect("Clicked()","MTH2D",this,"ShowElementsPeaks()");
				hframe_List_Elements->AddFrame(Button_List_Elements, new TGLayoutHints(kLHintsCenterX,5,5,3,4));
				
				fList_Elements=new TGListBox(hframe_List_Elements);
				Int_t element;
     				for(element=0;element<fF->GetNReaction();element++)
					{
					sprintf(name,"%s",fF->GetReaction_Name(element));
					fList_Elements->AddEntry(name,element+1);
					}
				fList_Elements->Resize(150,80);
				fList_Elements->SetMultipleSelections(1);
				hframe_List_Elements->AddFrame(Button_List_Elements, new TGLayoutHints(kLHintsCenterX,5,5,3,4));
				hframe_List_Elements->AddFrame(fList_Elements, new TGLayoutHints(kLHintsCenterX,5,5,3,4));
	
				fShow_FirstEscape= new TGCheckButton(hframe_List_Elements, new TGHotString("1st escape"), 2);
				fShow_FirstEscape->SetState(kButtonUp);
				hframe_List_Elements->AddFrame(fShow_FirstEscape, new TGLayoutHints(kLHintsCenterX,5,5,3,4));
	
				fShow_SecondEscape= new TGCheckButton(hframe_List_Elements, new TGHotString("2nd escape"), 2);
				fShow_SecondEscape->SetState(kButtonUp);
				hframe_List_Elements->AddFrame(fShow_SecondEscape, new TGLayoutHints(kLHintsCenterX,5,5,3,4));

	
				fMain->AddFrame(hframe_List_Elements, new TGLayoutHints(kLHintsCenterX,2,2,2,2));
				}	
//_P


			}//if(GetYaxis()->GetNbins()!=1||GetXaxis()->GetNbins()!=1)
		
		if(fC==NULL)
			{
			fC = fEcanvas->GetCanvas();
			fC->SetFillColor(0);
			fC->ToggleEventStatus();
			fC->GetCanvasImp()->ShowStatusBar();
			fC->Connect("ProcessedEvent(Int_t,Int_t,Int_t,TObject*)","MTH2D",this,"EventInfo(Int_t,Int_t,Int_t,TObject*)");
			Int_t parts[]={45,15,10,30};
			fStatusBar=new TGStatusBar(fMain);
			fStatusBar->SetParts(parts,3);
			TGLayoutHints *fStatusBarLayout = new TGLayoutHints(kLHintsBottom | kLHintsLeft | kLHintsExpandX, 2, 2, 1, 1);
			fMain->AddFrame(fStatusBar,fStatusBarLayout );
			}
		
		// Set a name to the main frame
		fMain->SetWindowName(GetName());
		// Map all subwindows of main frame
		fMain->MapSubwindows();
		// Initialize the layout algorithm
		fMain->Resize(fMain->GetDefaultSize());
		// Map main frame
		fMain->MapWindow();
		}
// Only update the histograms according to the boxes positions
if(level<1)
	{
	Int_t bin,Bin;
	Double_t Error;

	fC->Clear();

	fC->cd();
	if(GetYaxis()->GetNbins()!=1)
		this->UpdateBoxLimites();
	gStyle->SetOptStat(0);
	gStyle->SetOptTitle(0);
	gStyle->SetTitleYOffset(1.3);

	if(GetYaxis()->GetNbins()!=1&&GetXaxis()->GetNbins()!=1)
		{
		fC->Divide(3,2);
// To avoid unnecessary duplications (and thus memory leaks) of the histograms every time there is a drawing, histograms are deleted 
// before being created or recreated.
// For example, if the Draw option has been selected 5 times, there will be 5 "tmp_PX0" histograms if "tmp_PX0" 
// has not been erased from ROOT memory before drawing. 
// Thus, deleting "tmp_PX0" at every drawing will avoid memory leaks.
		sprintf(name,"%s_pX0",GetName());
		delete gROOT->FindObject("tmp_PX0");
		TH1D *Hx0=new TH1D(*ProjectionX("tmp_PX0",1,GetYaxis()->GetNbins()));
		sprintf(name,"%s_pX0",GetName());
		delete gROOT->FindObject(name);
		Hx0->SetName(name);
		Hx0->GetYaxis()->SetTitle(GetZaxis()->GetTitle());
		// In projections, ROOT calculate errors with the square root of the bin contents.
		// This is correct only when bin contents are associated with counts.
		// In other cases, the errors have to be calculated explicitely.
		if(strcmp(GetZaxis()->GetTitle(),"Counts"))
		for(Bin=1;Bin<=Hx0->GetNbinsX();Bin++)
			{
			Error=0;
			for(bin=1;bin<=GetYaxis()->GetNbins();bin++)
				Error+=GetBinError(Bin,bin)*GetBinError(Bin,bin);
			Error=sqrt(Error);
			Hx0->SetBinError(Bin,Error);
			}
		fC->cd(1)->SetFillColor(0);
		fC->GetPad(1)->SetFillColor(0);
		fC->cd(1);
		Hx0->Draw("he");
		fBoxX->Draw();
	
		delete gROOT->FindObject("tmp_PY0");
		TH1D *Hy0=new TH1D(*ProjectionY("tmp_PY0",1,GetXaxis()->GetNbins()));
		sprintf(name,"%s_pY0",GetName());
		delete gROOT->FindObject(name);
		Hy0->SetName(name);
		Hy0->GetYaxis()->SetTitle(GetZaxis()->GetTitle());
		if(strcmp(GetZaxis()->GetTitle(),"Counts"))
		for(Bin=1;Bin<=Hy0->GetNbinsX();Bin++)
			{
			Error=0;
			for(bin=1;bin<=GetXaxis()->GetNbins();bin++)
				Error+=GetBinError(bin,Bin)*GetBinError(bin,Bin);
			Error=sqrt(Error);
			Hy0->SetBinError(Bin,Error);
			}
		fC->cd(2)->SetFillColor(0);
		fC->GetPad(2)->SetFillColor(0);
		fC->cd(2);	
		if(GetPlot_Errors_Status())
			for(Bin=1;Bin<=Hy0->GetNbinsX();Bin++)
				Hy0->SetBinError(Bin,0);
		Hy0->Draw("he");
		fBoxY->Draw();
		
		// blue to red color palette
		Int_t *color,u;color=new Int_t[99];
		for(u=0;u<45;u++) color[u]=56+u;
		gStyle->SetPalette(45,color);
		
		// Black and White 2D palette
		/*
		Int_t *color,u;color=new Int_t[99];
		TColor **tcolor;tcolor=(TColor**)calloc(99,sizeof(TColor*));
		Int_t n_=0;
		Double_t r,g,b;
		for(n_=0;n_<45;n_++)
			{
			r=1-n_/45.;
			g=1-n_/45.;
			b=1-n_/99.;
			tcolor[n_]=new TColor(200+n_,r,g,b);
		
			tcolor[n_]=(TColor*)(gROOT->GetListOfColors()->At(200+n_));
			tcolor[n_]->SetRGB(r,g,b);
			}
		for(u=0;u<45;u++)
			color[u]=200+u;
		gStyle->SetPalette(45,color);	
		*/
		fC->cd(3)->SetFillColor(0);
		fC->GetPad(3)->SetFillColor(0);
		fC->cd(3);
		GetYaxis()->SetTitleOffset(1.2);
		Draw("col4");
		}			

	if(GetXaxis()->GetNbins()!=1)
		{
		delete gROOT->FindObject("tmp_PX");
		TH1D *Hx=new TH1D(*ProjectionX("tmp_PX",fb0BoxY,fb1BoxY));
		sprintf(name,"%s_pX",GetName());
		delete gROOT->FindObject(name);
		Hx->SetName(name);
		Hx->GetYaxis()->SetTitle(GetZaxis()->GetTitle());
		if(strcmp(GetZaxis()->GetTitle(),"Counts"))
		for(Bin=1;Bin<=Hx->GetNbinsX();Bin++)
			{
			Error=0;
			for(bin=fb0BoxY;bin<=fb1BoxY;bin++)
				Error+=GetBinError(Bin,bin)*GetBinError(Bin,bin);
			Error=sqrt(Error);
			Hx->SetBinError(Bin,Error);
			}
		if(GetYaxis()->GetNbins()!=1)
			{
			fC->cd(4)->SetFillColor(0);
			fC->GetPad(4)->SetFillColor(0);
			fC->cd(4);
			}
		else
			fC->cd();
		if(GetPlot_Errors_Status())
		for(Bin=1;Bin<=Hx->GetNbinsX();Bin++)
			Hx->SetBinError(Bin,0);
		Hx->Draw("he");
		}
			
	TPaveText *id_text=new TPaveText(0.,0.3,1,0.7);
	if(GetXaxis()->GetNbins()!=1)
		{
		delete gROOT->FindObject("tmp_PX");
		TH1D *Hx=new TH1D(*ProjectionX("tmp_PX",fb0BoxY,fb1BoxY));
		sprintf(name,"%s_pX",GetName());
		delete gROOT->FindObject(name);
		Hx->SetName(name);
		Hx->GetYaxis()->SetTitle(GetZaxis()->GetTitle());
		if(strcmp(GetZaxis()->GetTitle(),"Counts"))
		for(Bin=1;Bin<=Hx->GetNbinsX();Bin++)
			{
			Error=0;
			for(bin=fb0BoxY;bin<=fb1BoxY;bin++)
				Error+=GetBinError(Bin,bin)*GetBinError(Bin,bin);
			Error=sqrt(Error);
			Hx->SetBinError(Bin,Error);
			}
		if(GetYaxis()->GetNbins()!=1)
			{
			fC->cd(4)->SetFillColor(0);
			fC->GetPad(4)->SetFillColor(0);
			fC->cd(4);
			}
		else
			fC->cd();
		if(GetPlot_Errors_Status())
		for(Bin=1;Bin<=Hx->GetNbinsX();Bin++)
			Hx->SetBinError(Bin,0);
		Hx->Draw("he");
		if(GetYaxis()->GetNbins()!=1)
			{
			fC->cd(6)->SetFillColor(0);
			fC->GetPad(6)->SetFillColor(0);
			fC->cd(6);
			sprintf(w0,"Area(Energy)=%E",Hx->GetSumOfWeights());
			printf("____________________________________________\n");
			printf("%s\n",w0);
			id_text->AddText(w0);
			}
		}
		
	if(GetYaxis()->GetNbins()!=1)
		{
		delete gROOT->FindObject("tmp_PY");
		TH1D *Hy=new TH1D(*ProjectionY("tmp_PY",fb0BoxX,fb1BoxX));
		sprintf(name,"%s_pY",GetName());
		delete gROOT->FindObject(name);
		Hy->SetName(name);
		Hy->GetYaxis()->SetTitle(GetZaxis()->GetTitle());
		if(strcmp(GetZaxis()->GetTitle(),"Counts"))
		for(Bin=1;Bin<=Hy->GetNbinsX();Bin++)
			{
			Error=0;
			for(bin=fb0BoxX;bin<=fb1BoxX;bin++)
				Error+=GetBinError(bin,Bin)*GetBinError(bin,Bin);
			Error=sqrt(Error);
			Hy->SetBinError(Bin,Error);
			}
		if(GetXaxis()->GetNbins()!=1)
			{
			fC->cd(5)->SetFillColor(0);
			fC->GetPad(5)->SetFillColor(0);
			fC->cd(5);
			}
		else
			fC->cd();
		if(GetPlot_Errors_Status())
		for(Bin=1;Bin<=Hy->GetNbinsX();Bin++)
			Hy->SetBinError(Bin,0);
		Hy->Draw("he");
		if(GetXaxis()->GetNbins()!=1)
			{
			sprintf(w0,"Area(Time)=%E",Hy->GetSumOfWeights());
			printf("%s\n",w0);
			printf("____________________________________________\n");
			id_text->AddText(w0);
			}
		}

	if(GetXaxis()->GetNbins()!=1&&GetYaxis()->GetNbins()!=1)
		{
		fC->cd(6);
		id_text->SetTextFont(22);
		id_text->SetBorderSize(0);id_text->SetFillColor(0);
		id_text->Draw();
		}
		
			
	if(GetYaxis()->GetNbins()==1&&GetXaxis()->GetNbins()==1)
		{
		fC->cd();
		TPaveText *id_text=new TPaveText(0.,0.9,1,1);
		if(fparticle==1)
			sprintf(w0,"Tally amplitude(neutron)=%E",GetSumOfWeights());
		else
			sprintf(w0,"Tally amplitude(gamma)=%E",GetSumOfWeights());
		id_text->AddText(w0);id_text->SetTextFont(22);
		id_text->SetBorderSize(0);id_text->SetFillColor(0);
		id_text->Draw();
		}
	fC->cd();
	fC->Update();
	}		

// Produce ps,eps,...,txt files
// ps,gif,... pictures of all histograms are produced.
// In case of 1D data (only an energy card has been used for example), only one picture is produced.
// In case of 2D data, since 5 histograms are displayed, 5 pictures will be generated:
//	full energy,
//	full time,
//	time vs energy
//	energy with time cut,
//	time with energy cut,
// To keep track of the cuts that have been used, the picture file names contain
// the cuts value that have been used to produce the cutted spectrum
TPaveText *info_text=new TPaveText(0.,0.3,1,0.8);
	if(level==1)	
		{	
		sprintf(name,"%s_pX0",GetName());
		TH1D *Hx0=(TH1D*)gROOT->FindObject(name);
		sprintf(name,"%s_pY0",GetName());
		TH1D *Hy0=(TH1D*)gROOT->FindObject(name);
		sprintf(name,"%s_pY",GetName());
		TH1D *Hy=(TH1D*)gROOT->FindObject(name);
		sprintf(name,"%s_pX",GetName());
		TH1D *Hx=(TH1D*)gROOT->FindObject(name);

		Int_t s;
		fC->cd();
		if(fF!=NULL)
			{
			if(!gSystem->AccessPathName(fF->GetPlotsnASCIIsDirectory(),kWritePermission))
				{
				info_text->SetTextFont(22);
				info_text->SetTextAlign(12);
				info_text->SetBorderSize(0);info_text->SetFillColor(0);
				sprintf(name,"In directory %s,",fF->GetPlotsnASCIIsDirectory());
				info_text->AddText(name);	
				info_text->AddText("creation of:");	
				
				// Save in a .MTH2D file all data concerning the MTH2D
				if(fF->GetSaveAsOptionStatus((Int_t)14))
					{
					sprintf(name,"%s/%s.MTH2D",fF->GetPlotsnASCIIsDirectory(),GetName());
					SaveAsMTH2DFile(name);
					sprintf(name,"%s.MTH2D",GetName());
					info_text->AddText(name);
					}
	
				// Save the global picture of the MTH2D:
				// In case of 2D data, save in one picture the Canvas and the 5 sub-Canvases 
				// (four 1D spectra + one 2D spectrum).
				// In case of 1D data, save the corresponding 1D spectrum.
				if(fF->GetSpectraToBeSavedStatus((Int_t)6)||(GetXaxis()->GetNbins()==1||GetYaxis()->GetNbins()==1))
				for(s=0;s<14;s++)
				if(fF->GetSaveAsOptionStatus((Int_t)s))
					{
					sprintf(name,"%s/%s.%s",fF->GetPlotsnASCIIsDirectory(),GetName(),fF->GetSaveAs(s));
					if(s<13)
						{
						fC->Print(name);
						sprintf(name,"%s.%s",GetName(),fF->GetSaveAs(s));
						info_text->AddText(name);
						}
					// for 1D data, write bin contents of the full time and/or energy histograms
					// into a text file.
					if(s==13)
						{
						if(GetXaxis()->GetNbins()!=1&&GetYaxis()->GetNbins()==1)
							PrintMTH2D(name,Hx);
						if(GetXaxis()->GetNbins()==1&&GetYaxis()->GetNbins()!=1)
							PrintMTH2D(name,Hy);
						sprintf(name,"%s.%s",GetName(),fF->GetSaveAs(s));
						info_text->AddText(name);
						}
					}
				} // if(!gSystem->AccessPathName(fF->GetPlotsnASCIIsDirectory(),kWritePermission))
			else
				{
				info_text->SetTextFont(22);
				info_text->SetTextAlign(12);
				info_text->SetBorderSize(0);info_text->SetFillColor(0);
				info_text->SetTextColor(2);
				info_text->SetTextAlign(22);
				info_text->AddText("ERROR, cannot write in");	
				sprintf(name,"%s",fF->GetPlotsnASCIIsDirectory());
				info_text->AddText(name);	
				TCanvas *Info=new TCanvas("Info","Info");
				Info->cd();
				info_text->Draw();
				Info->Update();
				return 0;
				}
			} // if (fF!=NULL)
		// 2D histogram -> 5 pictures to save (four 1D spectra + one 2D spectrum)
		if (fF!=NULL)
		if(GetYaxis()->GetNbins()!=1&&GetXaxis()->GetNbins()!=1)
			{
			// It has been choosen to save individual spectra separately.
			// Thus all histograms will be plotted and saved individually in the Ctmp temporary TCanvas.
			TCanvas *Ctmp=new TCanvas("Ctmp","Ctmp");
			Ctmp->cd();
			// s==6 corresponds to "save as a .root file".
			// No need to save all individual spectra as a root file.
			for(s=0;s<14;s++)
			if(fF->GetSaveAsOptionStatus((Int_t)s))
				{
				Ctmp->Update();
				
				if(fF->GetSpectraToBeSavedStatus((Int_t)0))
					{
					sprintf(name,"%s/%s_E.%s",fF->GetPlotsnASCIIsDirectory(),GetName(),fF->GetSaveAs(s));
					// No need to save as a text file, it has been done in the previous loop.
					if(s<13&&s!=6)
						{
						Ctmp->cd();
						fC->GetPad(1)->SetPad(0,0,1,1);
						fC->GetPad(1)->DrawClone();
						Ctmp->Update();Ctmp->cd(1)->Print(name);Ctmp->Clear();
						sprintf(name,"%s_E.%s",GetName(),fF->GetSaveAs(s));
						info_text->AddText(name);	
						}
					if(s==13)
						{
						PrintMTH2D(name,Hx0);
						sprintf(name,"%s_E.%s",GetName(),fF->GetSaveAs(s));
						info_text->AddText(name);	
						}
					}	
	
				if(fF->GetSpectraToBeSavedStatus((Int_t)1))
					{
					sprintf(name,"%s/%s_T.%s",fF->GetPlotsnASCIIsDirectory(),GetName(),fF->GetSaveAs(s));
					// No need to save as a text file, it has been done in the previous loop.
					if(s<13&&s!=6)
						{
						Ctmp->cd();
						fC->GetPad(2)->SetPad(0,0,1,1);
						fC->GetPad(2)->DrawClone();Ctmp->Update();Ctmp->cd(2)->Print(name);Ctmp->Clear();
						sprintf(name,"%s_T.%s",GetName(),fF->GetSaveAs(s));
						info_text->AddText(name);
						}
					if(s==13)
						{
						PrintMTH2D(name,Hy0);
						sprintf(name,"%s_T.%s",GetName(),fF->GetSaveAs(s));
						info_text->AddText(name);
						}
					}		
		
				if(fF->GetSpectraToBeSavedStatus((Int_t)2))
					if(s<13) 
						{
						sprintf(name,"%s/%s_TvsE.%s",fF->GetPlotsnASCIIsDirectory(),GetName(),fF->GetSaveAs(s));
						Ctmp->cd();
						Draw("col4");
						Ctmp->Print(name);
						Ctmp->Clear();
						sprintf(name,"%s_TvsE.%s",GetName(),fF->GetSaveAs(s));
						info_text->AddText(name);	
						}

				if(fF->GetSpectraToBeSavedStatus((Int_t)3))
					{
					// Save the energy spectrum obtained when applying the time cut
					sprintf(name,"%s/%s_E_T%.2lfto%.2lf.%s",fF->GetPlotsnASCIIsDirectory(),GetName(),fBoxY->GetX1(),fBoxY->GetX2(),fF->GetSaveAs(s));				
					// usual TCanvas::Print() method
					if(s<13&&s!=6)
						{
						Ctmp->cd();
						fC->GetPad(4)->SetPad(0,0,1,1);
						fC->GetPad(4)->DrawClone();Ctmp->Update();Ctmp->cd(4)->Print(name);Ctmp->Clear();
						sprintf(name,"%s_E_T%.2lfto%.2lf.%s",GetName(),fBoxY->GetX1(),fBoxY->GetX2(),fF->GetSaveAs(s));
						info_text->AddText(name);	
						}
					// write bin contents of the energy histogram with time cut
					// into a text file.
					if(s==13)
						{
						PrintMTH2D(name,Hx);
						sprintf(name,"%s_E_T%.2lfto%.2lf.%s",GetName(),fBoxY->GetX1(),fBoxY->GetX2(),fF->GetSaveAs(s));
						info_text->AddText(name);	
						}
					}
						
				if(fF->GetSpectraToBeSavedStatus((Int_t)4))
					{
					// Save the time spectrum obtained when applying the energy cut
					sprintf(name,"%s/%s_T_E%.2lfto%.2lf.%s",fF->GetPlotsnASCIIsDirectory(),GetName(),fBoxX->GetX1(),fBoxX->GetX2(),fF->GetSaveAs(s));
					if(s<13&&s!=6)
						{
						Ctmp->cd();
						fC->GetPad(5)->SetPad(0,0,1,1);
						fC->GetPad(5)->DrawClone();Ctmp->Update();Ctmp->cd(5)->Print(name);Ctmp->Clear();
						sprintf(name,"%s_T_E%.2lfto%.2lf.%s",GetName(),fBoxX->GetX1(),fBoxX->GetX2(),fF->GetSaveAs(s));
						info_text->AddText(name);	
						}
					// write bin contents of the time histogram with energy cut
					// into a text file.
					if(s==13)
						{
						PrintMTH2D(name,Hy);
						sprintf(name,"%s_T_E%.2lfto%.2lf.%s",GetName(),fBoxX->GetX1(),fBoxX->GetX2(),fF->GetSaveAs(s));
						info_text->AddText(name);	
						}
					}
	
				if(fF->GetSpectraToBeSavedStatus((Int_t)5))
					{
					sprintf(name,"%s/%s_C6.%s",fF->GetPlotsnASCIIsDirectory(),GetName(),fF->GetSaveAs(s));
					if(s<13)
						{
						Ctmp->cd();
						fC->GetPad(6)->SetPad(0,0,1,1);
						fC->GetPad(6)->DrawClone();Ctmp->Update();Ctmp->cd(6)->Print(name);Ctmp->Clear();
						sprintf(name,"%s_C6.%s",GetName(),fF->GetSaveAs(s));
						info_text->AddText(name);	
						}
					}
				}
			delete gROOT->FindObject("Ctmp");
			} //if (fF!=NULL)if(GetYaxis()->GetNbins()!=1&&GetXaxis()->GetNbins()!=1)	
		
		TCanvas *Info=new TCanvas("Info","Info");
		Info->cd();
		info_text->Draw();
		Info->Update();
		fC->Update();
		DoDraw();
		}//if(level==1)
	return 1;
	}; 
// end of Int_t MTH2D::DoDraw(Int_t level=0)




///////////////////////////////////////////////////////////////////////////////////////////////
/////// TIME-ENERGY SMEARING WITH ENERGY RESPONSE FUNCTION AND TIME GAUSSIAN FUNCTION /////////
///////////////////////////////////////////////////////////////////////////////////////////////
MTH2D* MTH2D::Smear(char *_ResponseFunctionFile_,Double_t _s_,Bool_t SHOW)
{
Double_t SigmaY;
char *ResponseFunctionFile;ResponseFunctionFile=new char[600];

if(_s_>0)
	SigmaY=_s_;
else
	{
	if(fF!=NULL)
		{
		SigmaY=fF->Get_GTBParameter();
		// take units into account (remember that in the GUI interface, the time
		//  smearing is given in shake units).
		if(!strcmp(GetYaxis()->GetTitle(),"Time (ns)"))
			SigmaY*=10.;
		if(!strcmp(GetYaxis()->GetTitle(),"Time (s)"))
			SigmaY/=100.;
		if(!strcmp(GetYaxis()->GetTitle(),"Time (ms)"))
			SigmaY/=100000.;
		if(!strcmp(GetYaxis()->GetTitle(),"Time (s)"))
			SigmaY/=100000000.;	
		}
	else
		SigmaY=0.;
	}

if(_ResponseFunctionFile_!=NULL)
	sprintf(ResponseFunctionFile,"%s",_ResponseFunctionFile_);
else
	{
	if(fF!=NULL)
		sprintf(ResponseFunctionFile,"%s",fF->GetResponseFunction());
	else
		ResponseFunctionFile=NULL;
	}

if(ResponseFunctionFile==NULL&&SigmaY<=0)
	return NULL;

Double_t SQRT_TwoPi;SQRT_TwoPi=sqrt(2.*acos(-1.));
Int_t Counter,THERE_IS_MACRO;
char * w0;w0=new char[400];
Int_t c,p;
FILE *Libfile;
Int_t e;
Double_t Emin=0,Emax=10,E0;

/*Gaussian broadening function along Y axis*/
if(SigmaY)
	sprintf(w0,"%lf*exp(-0.5*(x/%lf)**2)",1./(SQRT_TwoPi*SigmaY),SigmaY);
TF1 *GBY=new TF1("GBY",w0);

	
if(GetXaxis()->GetNbins()!=1&&GetYaxis()->GetNbins()!=1&&fC!=NULL)
	fC->cd(6)->Clear();
else
	{
	TCanvas *Cinfo=new TCanvas("Cinfo","Cinfo");
	Cinfo->cd();
	}	
TPaveText *info_text=new TPaveText(0.,0.3,1,0.8);
//info_text->SetTextSize(0.04);
Libfile=fopen(ResponseFunctionFile,"rb");
if(Libfile!=NULL)
	{
	info_text->AddText("Energy response function:");	
	sprintf(w0,"_{%s}",ResponseFunctionFile);
	info_text->AddText(w0);	
	fclose(Libfile);
	THERE_IS_MACRO=1;
	}
else
	{
	info_text->AddText("Energy smearing: cannot find");	
	sprintf(w0,"_{%s}",ResponseFunctionFile);
	info_text->AddText(w0);	
	THERE_IS_MACRO=0;
	}

/*copy the histograms definition, allocate the memory needed for all the 2D histograms*/
char *name;name=new char[(Int_t)strlen(GetName())+200];

sprintf(w0,"%s",ResponseFunctionFile);
// The '\' character may cause problems, in Linux environment for instance.
// Thus all '\' characters (ascii number=92) are replaced by the harmless 
// '/' characters (ascii number=47).
for(c=(Int_t)strlen(w0)-1;c>0;c--)
	{
	if((Int_t)w0[c]==47||(Int_t)w0[c]==92)
		break;
	if(w0[c]=='.')
		w0[c]='\0';
	}
// To keep track of the histogram processing history, the name of the smeared histogram contains
// the smearing parameters (time resolution+energy response function)
// Time resolution is given in picoseconds, so that the histogram name does not contain '+','-' or '.' characters.
// This can make the histogram more practical to handle within ROOT. 
if(THERE_IS_MACRO)
	sprintf(name,"%sSmeared_%.0lfps_%s",GetName(),SigmaY*10000.,w0+c+1);
else
	sprintf(name,"%sSmeared_%.0lfps",GetName(),SigmaY*10000.);
//No need to perform several times the same smearing: if a smearing with the same parameters has already be
//performed (=the name already exist), don't smear. This will avoid memory conflicts
if(gROOT->FindObject(name)!=NULL)
	{
	info_text->Clear();
	info_text->AddText("smearing with ");	
	info_text->AddText(w0+c+1);	
	info_text->AddText("and");	
	sprintf(w0,"#sigma=%.2lf shakes",SigmaY);
	info_text->AddText(w0);	
	info_text->AddText("already done");	
	info_text->SetTextFont(22);info_text->SetTextColor(2);
	info_text->SetBorderSize(0);info_text->SetFillColor(0);
	if(GetXaxis()->GetNbins()!=1&&GetYaxis()->GetNbins()!=1&&fC!=NULL)
		{
		fC->cd(6);
		info_text->Draw();
		fC->Update();
		}
	else
		{
		TCanvas *Cinfo=new TCanvas("Cinfo","Cinfo");
		Cinfo->cd();
		info_text->Draw();
		Cinfo->Update();
		}
	return NULL;
	}


Int_t bin_x,bin_y,bin_x0,bin_y0,start;
Double_t counts0,counts,previouscounts,center,center0;
Double_t dy,error0_2,error_2,previouserror_2,Integral;

// for summing and propagating errors, best is to consider the
// error squared (simple additions).
for(bin_x0=1;bin_x0<=GetXaxis()->GetNbins();bin_x0++)		/*for all bins*/
for(bin_y0=1;bin_y0<=GetYaxis()->GetNbins();bin_y0++)		/*for all bins*/
	{
	error0_2=GetBinError(bin_x0,bin_y0);
	SetBinError(bin_x0,bin_y0,error0_2*error0_2);
	}

MTH2D *Htmp0=new MTH2D(*this);
Htmp0->SetName(name);

sprintf(w0,"Time smearing function:");
info_text->AddText(w0);	
sprintf(w0,"e^{- #frac{t^{2}}{2#times GTB^{2}}}");
info_text->AddText(w0);	
info_text->SetTextFont(22);
info_text->SetBorderSize(0);info_text->SetFillColor(0);
info_text->Draw();
if(GetXaxis()->GetNbins()!=1&&GetYaxis()->GetNbins()!=1&&fC!=NULL)
	fC->Update();

TPaveText *info_progress_text=new TPaveText(0.,0.9,1,1.);
info_progress_text->SetTextSize(0.08);
sprintf(w0,"Time smearing...");
info_progress_text->AddText(w0);	
info_progress_text->SetTextFont(8);
info_progress_text->SetTextSize(12);
info_progress_text->SetBorderSize(0);info_progress_text->SetFillColor(0);
info_progress_text->Draw();
TPave *info_progress=new TPave(0.,0.8,0.01,0.9);
info_progress->SetFillColor(4);
info_progress->SetBorderSize(0);
info_progress->Draw();
if(GetXaxis()->GetNbins()!=1&&GetYaxis()->GetNbins()!=1&&fC!=NULL)
	fC->Update();

if(SigmaY&&GetYaxis()->GetNbins()!=1)
	Htmp0->Scale(0);

printf("Time smearing...\n");

/*For all bins along X axis, propagate the counts along Y axis with gaussian broadening GBY*/
if(SigmaY&&GetYaxis()->GetNbins()!=1)
for(bin_x0=1,Counter=0;bin_x0<=GetXaxis()->GetNbins();bin_x0++)		/*for all bins*/
	{
	Counter++;
	if(Counter>GetXaxis()->GetNbins()/100)
		{
		Counter=0;
		info_progress->SetX2NDC((Double_t)bin_x0/(Double_t)GetXaxis()->GetNbins());
		info_progress->Draw();
		printf("%.0lf%c done\n",100.*(Double_t)bin_x0/(Double_t)GetXaxis()->GetNbins(),37);
		if(GetXaxis()->GetNbins()!=1&&GetYaxis()->GetNbins()!=1&&fC!=NULL)
			fC->Update();
		}
	for(bin_y0=1;bin_y0<=GetYaxis()->GetNbins();bin_y0++)	/*for all bins along Y axis*/
		{
		counts0=GetBinContent(bin_x0,bin_y0);				/*Get bin count to be propagated*/
		error0_2=GetBinError(bin_x0,bin_y0);
		center0=GetYaxis()->GetBinCenter(bin_y0);	/*Get bin position from which gaussian broadening is performed*/
		/*to save computing time, don't do anything if there are no counts in the bin*/
		if(counts0)
			{
			/*to save computing time, consider only the bins having a center within 4*SigmaY from center0 */
			start=GetYaxis()->FindBin(center0-4*SigmaY)-3;
			if(start<1)start=1;
			for(bin_y=start;bin_y<=GetYaxis()->GetNbins();bin_y++)
				{
				center=GetYaxis()->GetBinCenter(bin_y);
				if(center-center0>4*SigmaY)break;	/* outside the 4*SigmaY range*/
				if(center-center0<-4*SigmaY) continue;	/* outside the 4*SigmaY range*/
				/*propagate the counts to bin bin_y with GBY*/
				dy=GetYaxis()->GetBinWidth(bin_y);
				// in case the bin at center0 position itself is huge:
				if(dy<10*SigmaY)
					{
					Integral=GBY->Integral(center-center0-dy/2,center-center0+dy/2);
					counts=counts0*Integral;
					error_2=error0_2*Integral*Integral;
					}	
				else
					{
					counts=counts0;
					error_2=error0_2;
					}
				previouscounts=Htmp0->GetBinContent(bin_x0,bin_y);
				previouserror_2=Htmp0->GetBinError(bin_x0,bin_y);
				Htmp0->SetBinContent(bin_x0,bin_y,counts+previouscounts);
				// we deal with squared errors, so they can be simply summed
				Htmp0->SetBinError(bin_x0,bin_y,error_2+previouserror_2);
				}
			}
		}
	}/*if(SigmaY)*/

/* Load Response function */
info_progress_text->Delete();
info_progress->Delete();
if(GetXaxis()->GetNbins()!=1&&GetYaxis()->GetNbins()!=1&&fC!=NULL)
	fC->Update();

if(!THERE_IS_MACRO) /*if not able to build detector response function*/
	{
	if(SHOW)
		Htmp0->DoDraw();	
	return Htmp0;
	}

MTH2D *Htmp=new MTH2D(*Htmp0);
sprintf(w0,"%s_smeared_tmp",GetName());
delete gROOT->FindObject(w0);
Htmp->SetName(w0);

if(GetXaxis()->GetNbins()!=1&&GetYaxis()->GetNbins()!=1&&fC!=NULL)
	fC->Update();

TPave *info_progress_E=new TPave(0.,0.8,0.01,0.9);
TPaveText *info_progress_text_E=new TPaveText(0.,0.9,1,1.);
TH1D *EnergyDef=new TH1D(*ProjectionX());

// call the macro associated to the detector response function
// calling this macro will reate the histograms containing all necessary information
// to build the response function (number of parametres, surface of detector, parameters,...).
if(THERE_IS_MACRO)
	gROOT->Macro(ResponseFunctionFile);
TH1I *HNparam=(TH1I*)gROOT->FindObject("H_Nparameters");
TH1D *HSurface=(TH1D*)gROOT->FindObject("HSurface");
TH1D *HIntegral=(TH1D*)gROOT->FindObject("HIntegral");
TH1D* Hparam;
TF1 *Response=(TF1*)gROOT->FindObject("Response");

Int_t Np=500;
Double_t Surface=22.,E0_Integration=0,E1_Integration=8,SumOfWeights,Iratio;

if(GetXaxis()->GetNbins()!=1)
{
EnergyDef->SetName("EnergyDef");
Emin=EnergyDef->GetBinLowEdge(1);
Emax=EnergyDef->GetBinLowEdge(EnergyDef->GetNbinsX())+EnergyDef->GetBinWidth(EnergyDef->GetNbinsX());


Np=(Int_t)HNparam->GetBinContent(1);


Surface=HSurface->GetBinContent(1);

TH1D *HIntegrationBoundaries=(TH1D*)gROOT->FindObject("HIntegrationBoundaries");
E0_Integration=HIntegrationBoundaries->GetBinContent(1);
E1_Integration=HIntegrationBoundaries->GetBinContent(2);


Response->SetNpx(EnergyDef->GetNbinsX());
Response->SetRange(Emin,Emax);


if(GetXaxis()->GetNbins()!=1&&GetYaxis()->GetNbins()!=1&&fC!=NULL)
	fC->cd(6);


Htmp0->Scale(0);

info_progress_text_E->SetX1NDC(0);
info_progress_text_E->SetX2NDC(1);
info_progress_text_E->SetY1NDC(0.9);
info_progress_text_E->SetY2NDC(1);
info_progress_text_E->SetTextSize(0.08);
sprintf(w0,"Energy smearing...");
info_progress_text_E->AddText(w0);
info_progress_text_E->SetTextFont(8);
info_progress_text_E->SetTextSize(12);
info_progress_text_E->SetBorderSize(0);info_progress_text_E->SetFillColor(0);
info_progress_E->SetFillColor(4);
info_progress_E->SetBorderSize(0);
printf("Energy smearing...\n");
}

/*For all bins along X axis, propagate the counts along Y axis according to the energy response function*/
if(GetXaxis()->GetNbins()!=1)
for(bin_x0=1,Counter=0;bin_x0<=GetXaxis()->GetNbins();bin_x0++)	/*for all bins*/
	{
	Counter++;
	if(Counter>GetXaxis()->GetNbins()/100)
		{
		if(fC!=NULL)fC->cd(6)->Clear();
		Counter=0;
		if(GetXaxis()->GetNbins())
			{
			info_progress_E->SetX2NDC((Double_t)bin_x0/(Double_t)GetXaxis()->GetNbins());
			info_progress_E->SetX1NDC(0);
			info_progress_E->SetY1NDC(0.8);
			info_progress_E->SetY2NDC(0.9);
			}
		info_progress_text_E->SetX1NDC(0);
		info_progress_text_E->SetX2NDC(1);
		info_progress_text_E->SetY1NDC(0.9);
		info_progress_text_E->SetY2NDC(1);
		info_progress_text_E->Draw();
		info_progress_E->Draw();
		info_text->Draw();
		printf("%.0lf%c done\n",100.*(Double_t)bin_x0/(Double_t)GetXaxis()->GetNbins(),37);
		if(GetXaxis()->GetNbins()!=1&&GetYaxis()->GetNbins()!=1&&fC!=NULL)
			fC->Update();
		}
	E0=EnergyDef->GetBinCenter(bin_x0);

	sprintf(w0,"H_parameters[%d]",0);
	gROOT->GetObject(w0,Hparam);

// Parameters interpolation	
	if(E0<Hparam->GetBinLowEdge(1)||E0>Hparam->GetBinLowEdge(Hparam->GetNbinsX())+Hparam->GetBinWidth(Hparam->GetNbinsX()))
		continue;
	for(e=0;e<Hparam->GetNbinsX()-1;e++)
	if(E0>Hparam->GetBinLowEdge(e)&&E0<=Hparam->GetBinLowEdge(e+1)) break;
	for(p=0;p<Np-1;p++)
		{
		sprintf(w0,"H_parameters[%d]",p);
		gROOT->GetObject(w0,Hparam);
		Response->SetParameter(p,Hparam->GetBinContent(e+1)+(Hparam->GetBinLowEdge(e+1)-E0)*(Hparam->GetBinContent(e+1)-Hparam->GetBinContent(e))/(Hparam->GetBinLowEdge(e)-Hparam->GetBinLowEdge(e+1)));		
		}
	Response->SetParameter(Np-1,E0);
	Response->Draw();

// Scaling of the response function, taking into account the energy threshold
	Integral=HIntegral->GetBinContent(e+1)+(HIntegral->GetBinLowEdge(e+1)-E0)*(HIntegral->GetBinContent(e+1)-HIntegral->GetBinContent(e))/(HIntegral->GetBinLowEdge(e)-HIntegral->GetBinLowEdge(e+1));	
	SumOfWeights=Response->GetHistogram()->GetSumOfWeights();
	if(SumOfWeights==0||Response->Integral(E0_Integration,E1_Integration)==0)continue;
	Iratio=Response->Integral(Emin,Emax)/Response->Integral(E0_Integration,E1_Integration);
	Response->GetHistogram()->Scale(Surface*Iratio*Integral/SumOfWeights);

	for(bin_y0=1;bin_y0<=GetYaxis()->GetNbins();bin_y0++)		/*for all bins*/
		{
		counts0=Htmp->GetBinContent(bin_x0,bin_y0);	/*Get bin count to be propagated*/
		error0_2=Htmp->GetBinError(bin_x0,bin_y0);	/*Get bin error to be propagated*/
		if(counts0)
		for(bin_x=1;bin_x<=GetXaxis()->GetNbins();bin_x++)
		if(Response->GetHistogram()->GetBinContent(bin_x))
			{
			/*propagate the counts from bin_x0 to bin_x according to ResponseFunction_H*/
			counts=counts0*Response->GetHistogram()->GetBinContent(bin_x);
			previouscounts=Htmp0->GetBinContent(bin_x,bin_y0);
			error_2=error0_2*Response->GetHistogram()->GetBinContent(bin_x)*Response->GetHistogram()->GetBinContent(bin_x);
			previouserror_2=Htmp0->GetBinError(bin_x,bin_y0);
			Htmp0->SetBinContent(bin_x,bin_y0,counts+previouscounts);
			Htmp0->SetBinError(bin_x,bin_y0,error_2+previouserror_2);
			}
		}
	}

// Up to now, we dealed with squared errors. so errors have to be square rooted.
for(bin_y0=1;bin_y0<=GetYaxis()->GetNbins();bin_y0++)
for(bin_x=1;bin_x<=GetXaxis()->GetNbins();bin_x++)
	{
	error_2=Htmp0->GetBinError(bin_x,bin_y0);
	Htmp0->SetBinError(bin_x,bin_y0,sqrt(error_2));
	}

TString *Units_S=new TString(Htmp0->GetZaxis()->GetTitle());
if(Units_S->Contains("neutron",TString::kIgnoreCase))
	Htmp0->GetZaxis()->SetTitle("Counts (neutron^{-1})");
if(Units_S->Contains("gamma",TString::kIgnoreCase))
	Htmp0->GetZaxis()->SetTitle("Counts (#gamma^{-1})");
if(Units_S->Contains("e^{-}",TString::kIgnoreCase))
	Htmp0->GetZaxis()->SetTitle("Counts (e^{-})");

if(GetXaxis()->GetNbins()!=1)
{
info_progress_text_E->Delete();
info_progress_E->Delete();
}
info_text->Delete();
if(fC!=NULL) fC->cd(6)->Clear();
if(GetXaxis()->GetNbins()!=1&&GetYaxis()->GetNbins()!=1&&fC!=NULL)
	fC->Update();
if(SHOW)
	Htmp0->DoDraw();
gROOT->Add(Htmp0);


// Get back to previous errors
for(bin_x0=1;bin_x0<=GetXaxis()->GetNbins();bin_x0++)		/*for all bins*/
for(bin_y0=1;bin_y0<=GetYaxis()->GetNbins();bin_y0++)		/*for all bins*/
	{
	error0_2=GetBinError(bin_x0,bin_y0);
	SetBinError(bin_x0,bin_y0,sqrt(error0_2));
	}

return Htmp0;
};
// end of MTH2D::Smear()


/*MCNP Output Data Analysis with ROOT*/
void MODAR()
{

printf("\n------------------------------------------------\n");
printf("------------------------------------------------\n");
printf("\n");
printf("Mcnp Output Data Analysis with Root\n");
printf("            (MODAR)               \n");
printf("\n");
printf("          version %.2lf            \n",MODAR_VERSION);
printf("\n");
printf("------------------------------------------------\n");
printf("------------------------------------------------\n");

// black and white panel starts at color #300
Int_t *color;color=new Int_t[99];
TColor **tcolor;tcolor=(TColor**)calloc(99,sizeof(TColor*));
Int_t n___=0;
Double_t r,g,b;
for(n___=0;n___<45;n___++)
	{
	r=1-n___/45.;
	g=1-n___/45.;
	b=1-n___/45.;
	tcolor[n___]=new TColor(300+n___,r,g,b);

	tcolor[n___]=(TColor*)(gROOT->GetListOfColors()->At(300+n___));
	tcolor[n___]->SetRGB(r,g,b);
	}

gStyle->SetFrameFillColor(0);

TGMainFrame *fMain = new TGMainFrame();

/* Add MCNP output files and MTH2D icons in ROOT icon list */
TGMimeTypes *NewTypes=gClient->GetMimeTypeList();
/* The xpm files are supposed to placed in the same directory as MODAR.C */
/* If this is not the case, modify the "NewTypes->AddType(..." commands accordingly. */
/* For example: */
/* NewTypes->AddType("[text/mcnp_output]","*.o","D:/MyDirectory/mcnp_o.xpm","D:/MyDirectory/mcnp_o.xpm"," ");  */
NewTypes->AddType("[text/mcnp_output]","*.o","D:/root/icons/mcnp_o.xpm","D:/root/icons/mcnp_o.xpm"," ");
NewTypes->AddType("[text/MTH2D]","*.MTH2D","D:/root/icons/mth2d.xpm","D:/root/macros/icons/mth2d.xpm"," ");

	
new MTH2D_Controller(gClient->GetRoot(), fMain, 850, 400,1);
}
